function MPC_Plot_Default_xValue(handles, MEAM_Object, yValue, xValue, objectList, nColors, colorIdx, fontSizes)
% default plots
% Y-axis = {'rose_E', 'E1', 'E2', 'E3', ...
%     'Evac', 'Einterstitial', ...
%     'GSFEs', 'ESurfaces', ...
%     'Substitution', 'ISubstitution', 'BindingEnergy', 'E0', 'alpha', 'H', 'Nx', 'C'};
% X-axis = '(default)'

% create colormap
nObjects = length(objectList);
c = colormap(hsv(max(nObjects,nColors)));

% symbols
s = {'o', 's', '^', 'p', 'd', 'v', 'h', '>', '*', '<'};
angstromStr = char(197);

% save points on plot(s) to file?
writePlotData = strcmpi(get(handles.File_Plot_Data, 'Checked'), 'On');
if writePlotData
    plot_dir = fullfile(pwd, 'plots');
    if ~exist(plot_dir, 'dir'), mkdir(plot_dir), end
    fid = fopen(fullfile(plot_dir, [yValue '_vs_' xValue '.CSV']), 'wt');
end

% initialize plot decorations
yAxisLabel = '';
xAxisLabel = '';
plotTitle = '';
tLegend = {};

switch yValue

    case {'rose_E', 'E1', 'E2', 'E3'}
        yAxisLabel = '(eV)';
        xAxisLabel = ['Lattice parameter (' angstromStr ')'];
        plotTitle = 'Energy-Volume';

    case {'GSFEs'}
        yAxisLabel = '(mJ/m^2)';
        xAxisLabel = ['Shift (' angstromStr ')'];
        plotTitle = 'Generalized stacking fault energy';

    case {'Substitution', 'ISubstitution', 'BindingEnergy'}
        yAxisLabel = '';
        xAxisLabel = '(mJ/m^2)';
        plotTitle = '';

    case {'E0'}
        yAxisLabel = '';
        xAxisLabel = '(eV)';
        plotTitle = yValue;

    case {'H'}
        yAxisLabel = '';
        xAxisLabel = '(eV)';
        plotTitle = 'Heat of formation energy';

    case {'Evac'}
        yAxisLabel = '';
        xAxisLabel = '(eV)';
        plotTitle = 'Vacancy energy';

    case {'Einterstitials'}
        yAxisLabel = '';
        xAxisLabel = '(eV)';
        plotTitle = 'Interstitial energy';

    case {'ESurfaces'}
        yAxisLabel = '';
        xAxisLabel = '(mJ/m^2)';
        plotTitle = 'Surface energy';

    case {'alpha'}
        yAxisLabel = '';
        xAxisLabel = '';
        plotTitle = yValue;

    case {'C'}
        yAxisLabel = '';
        xAxisLabel = '(GPa)';
        plotTitle = 'Elastic constants';

    case {'Nx'}
        yAxisLabel = '';
        xAxisLabel = '';
        plotTitle = yValue;

end

% initialize flags, limits for plots from binaries, ternaries
hasSubstitution = false;
hasISubstitution = false;
hasBindingEnergy = false;
yval = 0;
yString = {};
min_x = Inf;
max_x = -Inf;

% plot "yValue vs. xValue" curve(s) from each object
symIdx = 1;

for idx=1:nObjects
    idxObject = objectList(idx);
    objType = size(MEAM_Object{idxObject,1},2);
    tag = MEAM_Object{idxObject,2};
    lattice = MEAM_Object{idxObject,5};
    runData = MEAM_Object{idxObject,6};
    calibrationData = MEAM_Object{idxObject,7};
    colorTag = strrep(tag, '+', '_');

    switch objType

        case 1 % an element
            
            % elements are included?
            if ~strcmpi(get(handles.Plot_Include_elements, 'checked'), 'on')
                continue
            end
            
            switch yValue
                
                case {'rose_E', 'E1', 'E2', 'E3'}
                                        
                    % Rose curve to be plotted?
                    if strcmpi(get(handles.Plot_Result_rose_E, 'checked'), 'on')
                        cIdx = colorIdx.(colorTag).rose_E;
                        tLegend{end+1} = [tag ': Rose'];
                        plot(runData.SingleElement.rose_a, runData.SingleElement.rose_E, ...
                            'Linestyle', '--', 'Color', c(cIdx,:), 'LineWidth', 2)
                        hold on

                        if writePlotData
                            fprintf(fid, '\n#\n# %s: Rose\n#\n', tag);
                            fprintf(fid, '%g, %g\n', [runData.SingleElement.rose_a; runData.SingleElement.rose_E]);
                        end
                    end
                                       
                    % E1 curve to be plotted?
                    if isfield(runData.SingleElement, 'E1') && strcmpi(get(handles.Plot_Result_E1, 'checked'), 'on')
                        cIdx = colorIdx.(colorTag).E1;
                        tLegend{end+1} = [tag ': E1 (' handles.lattices{lattice(1)} ')'];
                        plot(runData.SingleElement.a1, runData.SingleElement.E1', ...
                            'Linestyle', '-', 'Color', c(cIdx,:),  'Linewidth', 3)
                        hold on
                        
                        if writePlotData
                            fprintf(fid, '\n#\n# %s: E1 (%s)\n#\n', tag, handles.lattices{lattice(1)});
                            fprintf(fid, '%g, %g\n', [runData.SingleElement.a1; runData.SingleElement.E1']);
                        end
                        
                        if ~isempty(calibrationData.E1)
                            %symIdx = symIdx+1;
                            tLegend{end+1} = ['File: ' calibrationData.E1.file '/' handles.lattices{lattice(1)}];
                            plot(calibrationData.E1.xdata, calibrationData.E1.ydata, ...
                                'Linestyle', 'none', 'Color', c(cIdx,:), 'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                            hold on
                            
                            if writePlotData
                                fprintf(fid, '\n#\n# %s %s target from file: %s\n#\n', tag, ...
                                    handles.lattices{lattice(1)}, calibrationData.E1.file);
                                fprintf(fid, '%g, %g\n', [calibrationData.E1.xdata; calibrationData.E1.ydata]);
                            end
                        end
                        
                    end
                    
                    % E2 curve to be plotted?
                    if isfield(runData.SingleElement, 'E2') && strcmpi(get(handles.Plot_Result_E2, 'checked'), 'on')
                        cIdx = colorIdx.(colorTag).E2;
                        tLegend{end+1} = [tag ': E2 (' handles.lattices{lattice(2)} ')'];
                        plot(runData.SingleElement.a2, runData.SingleElement.E2', ...
                            'Linestyle', '-', 'Color', c(cIdx,:), 'Linewidth', 3)
                        hold on
                        
                        if writePlotData
                            fprintf(fid, '\n#\n# %s: E2 (%s)\n#\n', tag, handles.lattices{lattice(2)});
                            fprintf(fid, '%g, %g\n', [runData.SingleElement.a2; runData.SingleElement.E2']);
                        end
                    
                        if ~isempty(calibrationData.E2)
                            %symIdx = symIdx+1;
                            tLegend{end+1} = ['File: ' calibrationData.E2.file '/' handles.lattices{lattice(2)}];
                            plot(calibrationData.E2.xdata, calibrationData.E2.ydata, ...
                                'Linestyle', 'none', 'Color', c(cIdx,:), 'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                            hold on
                            
                            if writePlotData
                                fprintf(fid, '\n#\n# %s %s target from file: %s\n#\n', tag, ...
                                    handles.lattices{lattice(2)}, calibrationData.E2.file);
                                fprintf(fid, '%g, %g\n', [calibrationData.E2.xdata; calibrationData.E2.ydata]);
                            end
                        end
                        
                    end
                    
                    % E3 curve to be plotted?
                    if isfield(runData.SingleElement, 'E3') && strcmpi(get(handles.Plot_Result_E3, 'checked'), 'on')
                        cIdx = colorIdx.(colorTag).E3;
                        tLegend{end+1} = [tag ': E3 (' handles.lattices{lattice(3)} ')'];
                        plot(runData.SingleElement.a3, runData.SingleElement.E3', ...
                            'Linestyle', '-', 'Color', c(cIdx,:), 'Linewidth', 3)
                        hold on
                        
                        if writePlotData
                            fprintf(fid, '\n#\n# %s: E3 (%s)\n#\n', tag, handles.lattices{lattice(3)});
                            fprintf(fid, '%g, %g\n', [runData.SingleElement.a3; runData.SingleElement.E3']);
                        end
                        
                        if ~isempty(calibrationData.E3)
                            %symIdx = symIdx+1;
                            tLegend{end+1} = ['File: ' calibrationData.E3.file '/' handles.lattices{lattice(3)}];
                            plot(calibrationData.E3.xdata, calibrationData.E3.ydata, ...
                                'Linestyle', 'none', 'Color', c(cIdx,:), 'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                            hold on

                            if writePlotData
                                fprintf(fid, '\n#\n# %s %s target from file: %s\n#\n', tag, ...
                                    handles.lattices{lattice(3)}, calibrationData.E3.file);
                                fprintf(fid, '%g, %g\n', [calibrationData.E3.xdata; calibrationData.E3.ydata]);
                            end
                        end

                    end
                    
                    
                case 'GSFEs'
                    
                    % GSFE curves to be plotted?
                    if ~( isfield(runData.SingleElement, yValue) && ...
                            strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                        continue
                    end
                        
                    for i=1:size(runData.SingleElement.GSFEs,1)
                        cIdx = colorIdx.(colorTag).(['GSFEs_' num2str(i)]);

                        tLegend{end+1} = [tag ': ' runData.SingleElement.planes{i}];
                        plot(runData.SingleElement.shift(i,:), runData.SingleElement.GSFEs(i,:), ...
                            'Linestyle', '-', 'Color', c(cIdx,:), 'Linewidth',3)
                        hold on

                        if writePlotData
                            fprintf(fid, '\n#\n# %s: GSFE (%s)\n#\n', tag, runData.SingleElement.planes{i});
                            fprintf(fid, '%g, %g\n', [runData.SingleElement.shift(i,:); runData.SingleElement.GSFEs(i,:)]);
                        end

                        if ~isempty(calibrationData.GSFEs) && ~isempty(calibrationData.GSFEs{i})
                            %symIdx = symIdx+1;
                            tLegend{end+1} = ['File: ' calibrationData.GSFEs{i}.file '/' runData.SingleElement.planes{i}];
                            plot(calibrationData.GSFEs{i}.xdata, calibrationData.GSFEs{i}.ydata, ...
                                'Linestyle', 'none', 'Color', c(cIdx,:), 'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                            hold on

                            if writePlotData
                                fprintf(fid, '\n#\n# %s %s target from file: %s\n#\n', tag, ...
                                    runData.SingleElement.planes{i}, calibrationData.GSFEs{i}.file);
                                fprintf(fid, '%g, %g\n', [calibrationData.GSFEs{i}.xdata; calibrationData.GSFEs{i}.ydata]);
                            end
                        end


                    end
                    
                    
                case {'Einterstitials'}
                    if ~( isfield(runData.SingleElement, yValue) && ...
                            strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                        continue
                    end
                    
                    cIdx = colorIdx.(colorTag).(yValue);
                    nValues = length(runData.SingleElement.interstials);
                    for i=1:nValues
                        tLabel = sprintf('%s: %s', tag, runData.SingleElement.interstials{i});
                        yval = yval-1;
                        yString{end+1} = tLabel;
                        xval = runData.SingleElement.Einterstitials(i);

                        plot(xval, yval, ...
                            'Linestyle', 'none', 'Color', c(cIdx,:), ...
                            'Marker', s{i}, 'MarkerFaceColor', c(cIdx,:))
                        text(xval, yval, ...
                            ['   ' num2str(xval) ], ...
                            'FontSize', fontSizes(1))
                        hold on

                        if writePlotData
                            fprintf(fid, '%s, %g\n', tLabel, xval);
                        end
                        if xval>max_x, max_x = xval; end
                        if xval<min_x, min_x= xval; end
                        
                    end
                    
                case {'ESurfaces'}
                    if ~( isfield(runData.SingleElement, yValue) && ...
                            strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                        continue
                    end
                    cIdx = colorIdx.(colorTag).(yValue);
                    
                    nValues = length(runData.SingleElement.Surfaces);
                    for i=1:nValues
                        tLabel = sprintf('%s: %s', tag, runData.SingleElement.Surfaces{i});
                        yval = yval-1;
                        yString{end+1} = tLabel;
                        xval = runData.SingleElement.ESurfaces(i);

                        plot(xval, yval, ...
                            'Linestyle', 'none', 'Color', c(cIdx,:), ...
                            'Marker', s{i}, 'MarkerFaceColor', c(cIdx,:))
                        text(xval, yval, ...
                            ['   ' num2str(xval) ], ...
                            'FontSize', fontSizes(1))
                        hold on

                        if writePlotData
                            fprintf(fid, '%s, %g\n', tLabel, xval);
                        end
                        if xval>max_x, max_x = xval; end
                        if xval<min_x, min_x= xval; end
                    end
                    
                    
                case {'C'}
                    if ~( isfield(runData.SingleElement, yValue) && ...
                            strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                        continue
                    end
                    cIdx = colorIdx.(colorTag).(yValue);
                    
                    nValues = 0;
                    for i=1:6
                        for j=1:6
                            if runData.SingleElement.C(i,j)==0, continue, end
                            nValues = nValues+1;
                            tLabel = sprintf('%s: C%d%d', tag, i, j);
                            yval = yval-1;
                            yString{end+1} = tLabel;
                            xval = runData.SingleElement.C(i,j);
                            
                            plot(xval, yval, ...
                                'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                'Marker', s{nValues}, 'MarkerFaceColor', c(cIdx,:))
                            text(xval, yval, ...
                                ['   ' num2str(xval) ], ...
                                'FontSize', fontSizes(1))
                            hold on
                            
                            if writePlotData
                                fprintf(fid, '%s, %g\n', tLabel, xval);
                            end
                            if xval>max_x, max_x = xval; end
                            if xval<min_x, min_x= xval; end
                        end
                    end
                    
                    
                case {'E0', 'alpha', 'Evac'}
                    
                    if ~( isfield(runData.SingleElement, yValue) && ...
                            strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                        continue
                    end
                    
                    cIdx = colorIdx.(colorTag).(yValue);
                    yval = yval-1;
                    %yString{end+1} = [tag ': ' yValue];
                    yString{end+1} = tag;
                    xval = runData.SingleElement.(yValue);
                    
                    plot(xval, yval, ...
                            'Linestyle', 'none', 'Color', c(cIdx,:), ...
                            'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                    text(xval, yval, ...
                        ['   ' num2str(xval) ], ...
                        'FontSize', fontSizes(1))
                    hold on
                    
                    if writePlotData
                        fprintf(fid, '%s, %g\n', tag, xval);
                    end
                    if xval>max_x, max_x = xval; end
                    if xval<min_x, min_x= xval; end
                    
            end
            
            
        case 2 % a binary 

            % binaries are included in plot ?
             if ~strcmpi(get(handles.Plot_Include_binaries, 'checked'), 'on')
                continue
             end
             
             % yValue is included in plot ?
            if ~strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on')
                continue
            end
            
            % Alloy sctructure available ?
            if ~isfield(runData, 'Alloy')
                continue
            end
            
            switch yValue
                
                case 'E1'
                    
                    % is yValue calculated ?
                    if ~isfield(runData.Alloy, 'BLattices') || ~isfield(runData.Alloy, yValue)
                        continue
                    end
                    
                    for i=1:size(lattice,2)
                        for j=1:objType
                            for k=1:objType % 1:(j-1)
                                if k>size(runData.Alloy.BLattices,3), continue, end
                                
                                BLattices = runData.Alloy.BLattices{i, j, k};
                                if isempty(BLattices), continue, end
                                latticeName = strrep(BLattices, '-', '_');
                                
                                %tLegend{end+1} = [tag ': E1 (' runData.Alloy.BLattices{i, j, k} ')'];
                                tLegend{end+1} = [BLattices ': E1'];
                                cIdx = colorIdx.(latticeName).(yValue);
                                
                                plot(runData.Alloy.a1(:, i, j, k), runData.Alloy.E1(:, i, j, k), ...
                                    'Linestyle', '-', 'Color', c(cIdx,:), 'Linewidth',3)
                                hold on
                                
                                if writePlotData
                                    fprintf(fid, '\n#\n# %s: E1 (%s)\n#\n', tag, BLattices);
                                    fprintf(fid, '%g, %g\n', [runData.Alloy.a1(:, i, j, k), runData.Alloy.E1(:, i, j, k)]');
                                end
                                
                                if ~isfield(calibrationData.E1, latticeName), continue, end
                                
                                if ~isempty(calibrationData.E1.(latticeName){i,j,k}.ydata)
                                    %symIdx = symIdx+1;
                                    tLegend{end+1} = ['File: ' calibrationData.E1.(latticeName){i,j,k}.file];
                                    plot(calibrationData.E1.(latticeName){i,j,k}.xdata, calibrationData.E1.(latticeName){i,j,k}.ydata, ...
                                        'Linestyle', 'none', 'Color', c(cIdx,:), 'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                                    hold on
                                    
                                    if writePlotData
                                        fprintf(fid, '\n#\n# %s %s: %s target from file: %s\n#\n', tag, ...
                                            BLattices, yValue, calibrationData.E1.(latticeName){i,j,k}.file);
                                        fprintf(fid, '%g, %g\n', ...
                                            [calibrationData.E1.(latticeName){i,j,k}.xdata; calibrationData.E1.(latticeName){i,j,k}.ydata]);
                                    end
                                end
                                
                            end
                        end
                    end
                    

                case {'E0', 'H', 'Nx'}
                    
                    % is yValue calculated ?
                    if ~isfield(runData.Alloy, 'BLattices') || ~isfield(runData.Alloy, yValue)
                        continue
                    end
                    
                    symIdx = 1;
                    for i=1:size(lattice,2)
                        for j=1:objType
                            for k=1:objType % 1:(j-1)
                                if k>size(runData.Alloy.BLattices,3), continue, end
                                
                                BLattices = runData.Alloy.BLattices{i, j, k};
                                if isempty(BLattices), continue, end
                                latticeName = strrep(BLattices, '-', '_');
                                cIdx = colorIdx.(latticeName).(yValue);

                                %tLabel = [tag ': ' yValue ' (' BLattices ')'];
                                tLabel = BLattices;
                                yval = yval-1;
                                yString{end+1} = tLabel;
                                xval = runData.Alloy.(yValue)(i, j, k);
                                
                                plot(xval, yval, ...
                                    'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                    'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                                text(xval, yval, ...
                                    ['   ' num2str(xval) ], ...
                                    'FontSize', fontSizes(1))
                                hold on
                                
                                if writePlotData
                                    fprintf(fid, '%s, %g\n', tLabel, xval);
                                end
                                if xval>max_x, max_x = xval; end
                                if xval<min_x, min_x= xval; end
                                
                            end
                        end
                    end
                    

                case {'C'}
                    
                    % is yValue calculated ?
                    if ~isfield(runData.Alloy, 'BLattices') || ~isfield(runData.Alloy, yValue)
                        continue
                    end
                    
                    symIdx = 1;
                    for i=1:size(lattice,2)
                        for j=1:objType
                            for k=1:objType % 1:(j-1)
                                if k>size(runData.Alloy.BLattices,3), continue, end
                                
                                BLattices = runData.Alloy.BLattices{i, j, k};
                                if isempty(BLattices), continue, end
                                latticeName = strrep(BLattices, '-', '_');
                                cIdx = colorIdx.(latticeName).(yValue);
                                
                                nValues = 0;
                                for row=1:6
                                    for col=1:6
                                        xval = runData.Alloy.C(row, col, i, j, k);
                                        if xval==0.0, continue, end
                                        nValues = nValues+1;
                                        tLabel = sprintf('%s: C%d%d', BLattices, row, col);
                                        
                                        yval = yval-1;
                                        yString{end+1} = tLabel;
                                        
                                        plot(xval, yval, ...
                                            'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                            'Marker', s{nValues}, 'MarkerFaceColor', c(cIdx,:))
                                        text(xval, yval, ...
                                            ['   ' num2str(xval) ], ...
                                            'FontSize', fontSizes(1))
                                        hold on
                                        
                                        if writePlotData
                                            fprintf(fid, '%s, %g\n', tLabel, xval);
                                        end
                                        if xval>max_x, max_x = xval; end
                                        if xval<min_x, min_x= xval; end
                                    end
                                end
                            end
                        end
                    end
                    
                    
                case {'Substitution', 'ISubstitution', 'BindingEnergy'}
                                        
                    % Substitution, ISubstitution  energy
                    for j=1:objType
                        for k=1:objType
                            
                            if ~isempty(runData.Alloy.SubsLabel{j,k})
                                hasSubstitution = true;
                                symIdx = 1;
                                %tLabel = [tag ': (' runData.Alloy.SubsLabel{j,k} ')'];
                                tLabel = runData.Alloy.SubsLabel{j,k};
                                cIdx = colorIdx.(colorTag).Substitution;
                                
                                yval = yval-1;
                                yString{end+1} = tLabel;
                                xval = runData.Alloy.Substitution(j,k);
                                
                                plot(xval, yval, ...
                                    'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                    'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                                text(xval, yval, ...
                                    ['   ' num2str(xval) ], ...
                                    'FontSize', fontSizes(1))
                                hold on
                                
                                if writePlotData
                                    fprintf(fid, '%s, %g\n', tLabel, xval);
                                end
                                
                                if xval>max_x, max_x = xval; end
                                if xval<min_x, min_x= xval; end
                                
                            end
                            
                            if ~isempty(runData.Alloy.ISubsLabel{j,k})
                                hasISubstitution = true;
                                symIdx = 2;
                                %tLabel = [tag ': (' runData.Alloy.ISubsLabel{j,k} ')'];
                                tLabel = runData.Alloy.ISubsLabel{j,k};
                                cIdx = colorIdx.(colorTag).ISubstitution;
                                
                                yval = yval-1;
                                yString{end+1} = tLabel;
                                xval = runData.Alloy.ISubstitution(j,k);
                                
                                plot(xval, yval, ...
                                    'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                    'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                                text(xval, yval, ...
                                    ['   ' num2str(xval) ], ...
                                    'FontSize', fontSizes(1))
                                hold on
                                
                                if writePlotData
                                    fprintf(fid, '%s, %g\n', tLabel, xval);
                                end
                                
                                if xval>max_x, max_x = xval; end
                                if xval<min_x, min_x= xval; end
                                
                            end
                        end
                    end
                    
            end
            
            
        case 3 % a ternary

            % ternaries are included in plot ?
            if ~strcmpi(get(handles.Plot_Include_ternaries, 'checked'), 'on')
                continue
            end
            % yValue is included in plot ?
            if ~strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on')
                continue
            end           
            % is Alloy structure available?
            if ~isfield(runData, 'Alloy')
                continue
            end
            
            switch yValue

                case {'Substitution', 'ISubstitution', 'BindingEnergy'}
                    
                    % is yValue calculated ?
                    if ~isfield(runData.Alloy, 'BindingEnergy')
                        continue
                    end
                                        
                    % Binding Energy
                    cIdx = colorIdx.(colorTag).BindingEnergy;
                    symIdx = 0;
                    for i=1:objType
                        for j=1:objType
                            if i==j, continue, end
                            for k=1:j
                                if k==i, continue, end
                                
                                hasBindingEnergy = true;
                                symIdx = symIdx+1;
                                %tLabel = [tag ': (' runData.Alloy.BindingLabel{j,k,i} ')'];
                                tLabel = runData.Alloy.BindingLabel{j,k,i};

                                yval = yval-1;
                                yString{end+1} = tLabel;
                                xval = runData.Alloy.BindingEnergy(j,k,i);
                                
                                plot(xval, yval, ...
                                    'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                    'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                                text(xval, yval, ...
                                    ['   ' num2str(xval) ], ...
                                    'FontSize', fontSizes(1))
                                hold on
                                
                                if writePlotData
                                    fprintf(fid, '%s, %g\n', tLabel, xval);
                                end
                                if xval>max_x, max_x = xval; end
                                if xval<min_x, min_x= xval; end
                                
                            end
                        end
                    end
                    
            end

    end

end

% font sizes [points, legend, axis, title]
if ~isempty(yString)
    % reverse yString
    tLen = length(yString);
    for i = 1:floor(tLen/2)
        t = yString{i};
        yString{i} = yString{tLen-i+1};
        yString{tLen-i+1} = t;
    end
    ext = 0.1*(max_x - min_x);
    if ext==0
        ext = 0.1*max_x;
    end
    
    if hasSubstitution && hasISubstitution
        if hasBindingEnergy
            plotTitle = 'Replacement substitution, foreign element interstitial, and binding energy';
        else
            plotTitle = 'Replacement substitution and foreign element interstitial energy';
        end
    elseif hasSubstitution 
        if hasBindingEnergy
            plotTitle = 'Replacement substitution and binding energy';
        else
            plotTitle = 'Replacement substitution energy';
        end
    elseif hasISubstitution
        if hasBindingEnergy
            plotTitle = 'Foreign element interstitial and binding energy';
        else
            plotTitle = 'Foreign element interstitial energy';
        end
    elseif hasBindingEnergy
        plotTitle = 'Binding energy';
    end
    
    set(gca, 'YTick', [yval:-1], 'TickLabelInterpreter', 'none', 'FontSize', fontSizes(3))
    set(gca, 'YTickLabel', yString, 'FontSize', fontSizes(3))
    if max_x>10
        xlim(sort([floor(min_x-ext), ceil(max_x+ext)]))
    else
        xlim(sort([min_x-ext, max_x+ext]))
    end
    ylim([yval-1,0])

else
    if ~isempty(tLegend)
        legend(tLegend, 'Interpreter', 'none', 'Location', 'Best', 'FontSize', fontSizes(2))
    end
end

if ~isempty(xAxisLabel), xlabel(xAxisLabel, 'Interpreter', 'none', 'FontSize', fontSizes(3)), end
if ~isempty(yAxisLabel), ylabel(yAxisLabel, 'Interpreter', 'none', 'FontSize', fontSizes(3)), end
if ~isempty(plotTitle), title(plotTitle, 'Interpreter', 'none', 'FontSize', fontSizes(4)), end

hold off

if writePlotData
    fclose(fid);
end
