function MPC_Plot_Scatter(handles, MEAM_Object, yValue, xValue, objectList, nColors, colorIdx, fontSizes)
% scatter plots

% create colormap
nObjects = length(objectList);
c = colormap(hsv(max(nObjects,nColors)));

% symbols
s = {'o', 's', '^', 'p', 'd', 'v', 'h', '>', '*', '<'};
angstromStr = char(197);

% save points on plot(s) to file?
writePlotData = strcmpi(get(handles.File_Plot_Data, 'Checked'), 'On');
if writePlotData
    plot_dir = fullfile(pwd, 'plots');
    if ~exist(plot_dir, 'dir'), mkdir(plot_dir), end
    fid = fopen(fullfile(plot_dir, [yValue '_vs_' xValue '.CSV']), 'wt');
end

% initialize plot decorations
xAxisLabel = '';
yAxisLabel = '';
zAxisLabel = '';
yTitle = '';
xTitle = '';
isY_many = true;
isX_many = true;
zString = {};

switch yValue

    case {'rose_E', 'E1', 'E2', 'E3'}
        zAxisLabel = '(eV)';
        yAxisLabel = ['Lattice parameter (' angstromStr ')'];
        yTitle = 'Energy-Volume';

    case {'GSFEs'}
        zAxisLabel = '(mJ/m^2)';
        yAxisLabel = ['Shift (' angstromStr ')'];
        yTitle = 'Generalized stacking fault energy';

    case {'Einterstitials'}
        yTitle = 'Interstitial energy ';
        yAxisLabel = [yTitle ' (eV)'];
        for idx=1:nObjects
            idxObject = objectList(idx);
            if size(MEAM_Object{idxObject,1},2)~=1, continue, end;
            runData = MEAM_Object{idxObject,6};
            if ~isfield(runData.SingleElement, 'interstials'), continue, end
            nValues = length(runData.SingleElement.interstials);
            for i=1:nValues
                if find(strcmpi(zString, runData.SingleElement.interstials{i})), continue, end
                zString{end+1} = runData.SingleElement.interstials{i};
            end
        end

    case {'ESurfaces'}
        yTitle = 'Surface energy';
        yAxisLabel = [yTitle ' (mJ/m^2)'];
        for idx=1:nObjects
            idxObject = objectList(idx);
            if size(MEAM_Object{idxObject,1},2)~=1, continue, end;
            runData = MEAM_Object{idxObject,6};
            if ~isfield(runData.SingleElement, 'Surfaces'), continue, end
            nValues = length(runData.SingleElement.Surfaces);
            for i=1:nValues
                if find(strcmpi(zString, runData.SingleElement.Surfaces{i})), continue, end
                zString{end+1} = runData.SingleElement.Surfaces{i};
            end
        end

    case {'C'}
        yTitle = 'Elastic constants';
        yAxisLabel = [yTitle ' (GPa)'];
        for idx=1:nObjects
            idxObject = objectList(idx);
            objType = size(MEAM_Object{idxObject,1},2);
            lattice = MEAM_Object{idxObject,5};
            runData = MEAM_Object{idxObject,6};
            switch objType
                case 1
                    if ~isfield(runData.SingleElement, 'C'), continue, end
                    for row=1:6
                        for col=1:6
                            if runData.SingleElement.C(row,col)==0, continue, end
                            tLabel = sprintf('C%d%d', row, col);
                            if find(strcmpi(zString, tLabel)), continue, end
                            zString{end+1} = tLabel;
                        end
                    end
                    
                case 2
                    if ~isfield(runData.Alloy, 'C'), continue, end
                    for row=1:6
                        for col=1:6
                            for i=1:size(lattice,2)
                                for j=1:objType
                                    for k=1:objType % 1:(j-1)
                                        if k>size(runData.Alloy.BLattices,3), continue, end
                                        BLattices = runData.Alloy.BLattices{i, j, k};
                                        if isempty(BLattices), continue, end

                                        if runData.Alloy.C(row, col, i, j, k)==0.0, continue, end
                                        tLabel = sprintf('C%d%d', row, col);
                                        if find(strcmpi(zString, tLabel)), continue, end
                                        zString{end+1} = tLabel;
                                    end
                                end
                            end
                        end
                    end
            end
        end
        
    case {'Substitution'}
        yTitle = 'Replacement substitution energy';
        yAxisLabel = [yTitle ' (mJ/m^2)'];
        isY_many = false;

    case {'ISubstitution'}
        yTitle = 'Foreign element interstitial energy';
        yAxisLabel = [yTitle ' (mJ/m^2)'];
        isY_many = false;

    case {'E0'}
        yTitle = yValue;
        yAxisLabel = [yTitle ' (eV)'];
        isY_many = false;

    case {'H'}
        yTitle = 'Heat of formation energy';
        yAxisLabel = [yTitle ' (eV)'];
        isY_many = false;

    case {'Evac'}
        yTitle = 'Vacancy energy';
        yAxisLabel = [yTitle ' (eV)'];
        isY_many = false;

    case {'alpha'}
        yAxisLabel = yValue;
        yTitle = yValue;
        isY_many = false;

    case {'Nx'}
        yAxisLabel = yValue;
        yTitle = yValue;
        isY_many = false;

end
% zString


switch xValue

    case {'Substitution'}
        xTitle = 'Replacement substitution energy';
        xAxisLabel = [xTitle ' (mJ/m^2)'];

    case {'ISubstitution'}
        xTitle = 'Foreign element interstitial energy';
        xAxisLabel = [xTitle ' (mJ/m^2)'];

    case {'Einterstitials'}
        xTitle = 'Interstitial energy ';
        xAxisLabel = [xTitle ' (eV)'];

    case {'ESurfaces'}
        xTitle = 'Surface energy';
        xAxisLabel = [xTitle ' (mJ/m^2)'];

    case {'C'}
        xTitle = 'Elastic constants';
        xAxisLabel = [xTitle ' (GPa)'];

    case {'E0'}
        xTitle = xValue;
        xAxisLabel = [xTitle ' (eV)'];
        isX_many = false;

    case {'H'}
        xTitle = 'Heat of formation energy';
        xAxisLabel = [xTitle ' (eV)'];
        isX_many = false;

    case {'Evac'}
        xTitle = 'Vacancy energy';
        xAxisLabel = [xTitle ' (eV)'];
        isX_many = false;

    case {'alpha'}
        xAxisLabel = xValue;
        xTitle = xValue;
        isX_many = false;

    case {'Nx'}
        xAxisLabel = xValue;
        xTitle = xValue;
        isX_many = false;
end


if isX_many && isY_many
    display(['Invalid selection "' yValue ' vs.' xValue '"'])
    return
end

if ~(isX_many || isY_many) % 2D plot is sufficient
    min_x = Inf;
    max_x = -Inf;
    min_y = Inf;
    max_y = -Inf;
    symIdx = 1;
    tLegend = {};

    for idx=1:nObjects
        idxObject = objectList(idx);
        objType = size(MEAM_Object{idxObject,1},2);
        tag = MEAM_Object{idxObject,2};
        lattice = MEAM_Object{idxObject,5};
        runData = MEAM_Object{idxObject,6};
        colorTag = strrep(tag, '+', '_');
        
        switch objType
            
            case 1 % an element
                
                % elements are included?
                if ~strcmpi(get(handles.Plot_Include_elements, 'checked'), 'on')
                    continue
                end
                
                if ~( strcmpi(get(handles.(['Plot_Result_' xValue]), 'checked'), 'on') && ...
                        strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                    continue
                end

                if ~( isfield(runData.SingleElement, xValue) && ...
                        isfield(runData.SingleElement, yValue) )
                    continue
                end

                cIdx = colorIdx.(colorTag).(yValue);
                xval = runData.SingleElement.(xValue);
                yval = runData.SingleElement.(yValue);
                tLegend{end+1} = tag;

                plot(xval, yval, ...
                    'Linestyle', 'none', 'Color', c(cIdx,:), ...
                    'Marker', s{symIdx}, 'MarkerFaceColor', c(cIdx,:))
                %text(xval, yval, sprintf('   %s (%g, %g)', tag, xval, yval), ...
                %    'FontSize', fontSizes(1))
                hold on

                if writePlotData
                    fprintf(fid, '%s, %g, %g\n', tag, xval, yval);
                end
                if xval>max_x, max_x = xval; end
                if xval<min_x, min_x= xval; end
                if yval>max_y, max_y = yval; end
                if yval<min_y, min_y= yval; end
                
            case 2 % a binary
                
                % binaries are included in plot ?
                if ~strcmpi(get(handles.Plot_Include_binaries, 'checked'), 'on')
                    continue
                end
                
                % yValue, xValue included in plot?
                if ~( strcmpi(get(handles.(['Plot_Result_' xValue]), 'checked'), 'on') && ...
                        strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                    continue
                end
                
                % Alloy.BLattices structure available ?
                if ~isfield(runData, 'Alloy') || ~isfield(runData.Alloy, 'BLattices')
                    continue
                end
                
                % is yValue, xValue calculated ?
                if ~( isfield(runData.Alloy, xValue) && isfield(runData.Alloy, yValue) )
                    continue
                end
                
                
                nLat = 0;
                for i=1:size(lattice,2)
                    for j=1:objType
                        for k=1:objType % 1:(j-1)
                            if k>size(runData.Alloy.BLattices,3), continue, end
                            
                            BLattices = runData.Alloy.BLattices{i, j, k};
                            if isempty(BLattices), continue, end
                            latticeName = strrep(BLattices, '-', '_');
                            cIdx = colorIdx.(latticeName).(yValue);
                            tLegend{end+1} = BLattices;
                            nLat = nLat + 1;
                            
                            xval = runData.Alloy.(xValue)(i, j, k);
                            yval = runData.Alloy.(yValue)(i, j, k);
                            
                            plot(xval, yval, ...
                                'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                'Marker', s{nLat}, 'MarkerFaceColor', c(cIdx,:))
                            %text(xval, yval, ...
                            %    ['   ' BLattices ], ...
                            %    'FontSize', fontSizes(1))
                            hold on
                            
                            if writePlotData
                                fprintf(fid, '%s, %g\n', BLattices, xval, yval);
                            end
                            if xval>max_x, max_x = xval; end
                            if xval<min_x, min_x= xval; end
                            if yval>max_y, max_y = yval; end
                            if yval<min_y, min_y= yval; end
                            
                        end
                    end
                end
                
        end
        
    end
    
    if any(isinf([min_x,max_x,min_y,max_y])) 
        plot(0,0)
        text(0, 0, [yValue ' vs. ' xValue ' - not a valid plot'],  ...
            'HorizontalAlignment', 'center') %, 'FontSize', fontSizes(1))
        
    else
        ext_x = 0.1*(max_x - min_x);
        ext_y = 0.1*(max_y - min_y);
        xlim([floor(min_x-ext_x), ceil(max_x+ext_x)])
        ylim([floor(min_y-ext_y), ceil(max_y+ext_y)])
    end
    
    % font sizes [points, legend, axis, title]
    if ~isempty(tLegend)
        legend(tLegend, 'Interpreter', 'none', 'Location', 'Best', 'FontSize', fontSizes(2))
    end
    xlabel(xAxisLabel, 'FontSize', fontSizes(3))
    ylabel(yAxisLabel, 'FontSize', fontSizes(3))
    title([yTitle ' vs. ' xTitle], 'FontSize', fontSizes(4))
    
    hold off

    
else % 3D plot necessary
    
    min_x = Inf;
    max_x = -Inf;
    min_y = Inf;
    max_y = -Inf;
    min_z = Inf;
    max_z = -Inf;
    tLegend = {};

    hFig = figure(1);
    set(hFig, 'Name', [yTitle ' vs. ' xTitle], 'NumberTitle', 'Off')

    for idx=1:nObjects
        idxObject = objectList(idx);
        objType = size(MEAM_Object{idxObject,1},2);
        tag = MEAM_Object{idxObject,2};
        lattice = MEAM_Object{idxObject,5};
        runData = MEAM_Object{idxObject,6};
        colorTag = strrep(tag, '+', '_');

        switch objType

            case 1 % an element

                % elements are included?
                if ~strcmpi(get(handles.Plot_Include_elements, 'checked'), 'on')
                    continue
                end

                if ~strcmpi(get(handles.(['Plot_Result_' xValue]), 'checked'), 'on') 
                    continue
                end

                if ~( isfield(runData.SingleElement, xValue) && ...
                        isfield(runData.SingleElement, yValue) )
                    continue
                end


                switch yValue

                    case {'rose_E', 'E1', 'E2', 'E3'}

                        % Rose curve to be plotted?
                        if strcmpi(get(handles.Plot_Result_rose_E, 'checked'), 'on') && ...
                                isfield(runData.SingleElement, 'rose_E')
                            cIdx = colorIdx.(colorTag).rose_E;
                            tLegend{end+1} = [tag ': Rose'];
                            zval = runData.SingleElement.rose_E;
                            yval = runData.SingleElement.rose_a;
                            xval = repmat(runData.SingleElement.(xValue), size(yval) );
                            if writePlotData
                                fprintf(fid, '\n#\n# %s: %s, %s, %s\n#\n', tag, xValue, 'rose_a', yValue);
                                fprintf(fid, '%g, %g, %g\n', [xval; yval; zval]);
                            end

                            plot3(xval, yval, zval, ...
                                'Linestyle', '-', 'Color', c(cIdx,:), 'LineWidth', 2)
                            hold on

                            tmp = max(xval);
                            if tmp>max_x, max_x = tmp; end
                            tmp = min(xval);
                            if tmp<min_x, min_x= tmp; end
                            tmp = max(yval);
                            if tmp>max_y, max_y = tmp; end
                            tmp = min(yval);
                            if tmp<min_y, min_y= tmp; end
                            tmp = max(zval);
                            if tmp>max_z, max_z = tmp; end
                            tmp = min(zval);
                            if tmp<min_z, min_z= tmp; end
                        end


                        % E1 curve to be plotted?
                        if strcmpi(get(handles.Plot_Result_E1, 'checked'), 'on') && ...
                                isfield(runData.SingleElement, 'E1')
                            cIdx = colorIdx.(colorTag).E1;
                            tLegend{end+1} = [tag ': E1 (' handles.lattices{lattice(1)} ')'];
                            zval = runData.SingleElement.E1';
                            yval = runData.SingleElement.a1;
                            xval = repmat(runData.SingleElement.(xValue), size(yval) );
                            if writePlotData
                                fprintf(fid, '\n#\n# %s: %s, a1, E1 (%s)\n#\n', tag, xValue, handles.lattices{lattice(1)});
                                fprintf(fid, '%g, %g, %g\n', [xval; yval; zval]);
                            end

                            plot3(xval, yval, zval, ...
                                'Linestyle', '-', 'Color', c(cIdx,:),  'Linewidth', 3)
                            hold on

                            tmp = max(xval);
                            if tmp>max_x, max_x = tmp; end
                            tmp = min(xval);
                            if tmp<min_x, min_x= tmp; end
                            tmp = max(yval);
                            if tmp>max_y, max_y = tmp; end
                            tmp = min(yval);
                            if tmp<min_y, min_y= tmp; end
                            tmp = max(zval);
                            if tmp>max_z, max_z = tmp; end
                            tmp = min(zval);
                            if tmp<min_z, min_z= tmp; end

                        end

                        % E2 curve to be plotted?
                        if strcmpi(get(handles.Plot_Result_E2, 'checked'), 'on') && ...
                                isfield(runData.SingleElement, 'E2')
                            cIdx = colorIdx.(colorTag).E2;
                            tLegend{end+1} = [tag ': E2 (' handles.lattices{lattice(2)} ')'];
                            zval = runData.SingleElement.E2';
                            yval = runData.SingleElement.a2;
                            xval = repmat(runData.SingleElement.(xValue), size(yval) );
                            if writePlotData
                                fprintf(fid, '\n#\n# %s: %s, a2, E2 (%s)\n#\n', tag, xValue, handles.lattices{lattice(2)});
                                fprintf(fid, '%g, %g, %g\n', [xval; yval; zval]);
                            end

                            plot3(xval, yval, zval, ...
                                'Linestyle', '-', 'Color', c(cIdx,:),  'Linewidth', 3)
                            hold on

                            tmp = max(xval);
                            if tmp>max_x, max_x = tmp; end
                            tmp = min(xval);
                            if tmp<min_x, min_x= tmp; end
                            tmp = max(yval);
                            if tmp>max_y, max_y = tmp; end
                            tmp = min(yval);
                            if tmp<min_y, min_y= tmp; end
                            tmp = max(zval);
                            if tmp>max_z, max_z = tmp; end
                            tmp = min(zval);
                            if tmp<min_z, min_z= tmp; end

                        end

                        % E3 curve to be plotted?
                        if strcmpi(get(handles.Plot_Result_E3, 'checked'), 'on') && ...
                                isfield(runData.SingleElement, 'E3')
                            cIdx = colorIdx.(colorTag).E3;
                            tLegend{end+1} = [tag ': E3 (' handles.lattices{lattice(3)} ')'];
                            zval = runData.SingleElement.E3';
                            yval = runData.SingleElement.a3;
                            xval = repmat(runData.SingleElement.(xValue), size(yval) );
                            if writePlotData
                                fprintf(fid, '\n#\n# %s: %s, a3, E3 (%s)\n#\n', tag, xValue, handles.lattices{lattice(3)});
                                fprintf(fid, '%g, %g, %g\n', [xval; yval; zval]);
                            end

                            plot3(xval, yval, zval, ...
                                'Linestyle', '-', 'Color', c(cIdx,:),  'Linewidth', 3)
                            hold on

                            tmp = max(xval);
                            if tmp>max_x, max_x = tmp; end
                            tmp = min(xval);
                            if tmp<min_x, min_x= tmp; end
                            tmp = max(yval);
                            if tmp>max_y, max_y = tmp; end
                            tmp = min(yval);
                            if tmp<min_y, min_y= tmp; end
                            tmp = max(zval);
                            if tmp>max_z, max_z = tmp; end
                            tmp = min(zval);
                            if tmp<min_z, min_z= tmp; end

                        end


                    case 'GSFEs'

                        % GSFE curves to be plotted?
                        if ~( isfield(runData.SingleElement, yValue) && ...
                                strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                            continue
                        end

                        nValues = size(runData.SingleElement.GSFEs,1);
                        for i=1:nValues
                            cIdx = colorIdx.(colorTag).(['GSFEs_' num2str(i)]);

                            tLegend{end+1} = [tag ': ' runData.SingleElement.planes{i}];
                            zval = runData.SingleElement.GSFEs(i,:);
                            yval = runData.SingleElement.shift(i,:);
                            xval = repmat(runData.SingleElement.(xValue), size(yval) );
                            if writePlotData
                                fprintf(fid, '\n#\n# %s: %s, shift, GSFE (%s)\n#\n', tag, xValue, runData.SingleElement.planes{i});
                                fprintf(fid, '%g, %g, %g\n', [xval; yval; zval]);
                            end

                            plot3(xval, yval, zval, ...
                                'Linestyle', '-', 'Color', c(cIdx,:), 'Linewidth',3)
                            hold on

                            tmp = max(xval);
                            if tmp>max_x, max_x = tmp; end
                            tmp = min(xval);
                            if tmp<min_x, min_x= tmp; end
                            tmp = max(yval);
                            if tmp>max_y, max_y = tmp; end
                            tmp = min(yval);
                            if tmp<min_y, min_y= tmp; end
                            tmp = max(zval);
                            if tmp>max_z, max_z = tmp; end
                            tmp = min(zval);
                            if tmp<min_z, min_z= tmp; end

                        end

                        
                    case {'Einterstitials'}
                        
                        if ~( isfield(runData.SingleElement, yValue) && ...
                                strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                            continue
                        end
                        
                        cIdx = colorIdx.(colorTag).(yValue);
                        nValues = length(runData.SingleElement.interstials);
                        for i=1:nValues
                            xval = runData.SingleElement.(xValue);
                            yval = runData.SingleElement.Einterstitials(i);
                            zval = find(strcmpi(zString, runData.SingleElement.interstials{i}));
                            
                            if writePlotData
                                fprintf(fid, '\n#\n# %s: %s, %s\n#\n', tag, xValue, runData.SingleElement.interstials{i});
                                fprintf(fid, '%g, %g\n', xval, yval);
                            end

                            plot3(xval, yval, zval, ...
                                'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                'Marker', s{i}, 'MarkerFaceColor', c(cIdx,:))
                            text(xval, yval, zval, ['   ' tag ], 'FontSize', fontSizes(1))
                            hold on

                            tmp = max(xval);
                            if tmp>max_x, max_x = tmp; end
                            tmp = min(xval);
                            if tmp<min_x, min_x= tmp; end
                            tmp = max(yval);
                            if tmp>max_y, max_y = tmp; end
                            tmp = min(yval);
                            if tmp<min_y, min_y= tmp; end
                            tmp = max(zval);
                            if tmp>max_z, max_z = tmp; end
                            tmp = min(zval);
                            if tmp<min_z, min_z= tmp; end
                            
                        end

                        case {'ESurfaces'}
                            if ~( isfield(runData.SingleElement, yValue) && ...
                                    strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                                continue
                            end
                            cIdx = colorIdx.(colorTag).(yValue);

                            nValues = length(runData.SingleElement.Surfaces);
                            for i=1:nValues

                                xval = runData.SingleElement.(xValue);
                                yval = runData.SingleElement.ESurfaces(i);
                                zval = find(strcmpi(zString, runData.SingleElement.Surfaces{i}));

                                if writePlotData
                                    fprintf(fid, '\n#\n# %s: %s, %s\n#\n', tag, xValue, runData.SingleElement.Surfaces{i});
                                    fprintf(fid, '%g, %g\n', xval, yval);
                                end

                                plot3(xval, yval, zval, ...
                                    'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                    'Marker', s{i}, 'MarkerFaceColor', c(cIdx,:))
                                text(xval, yval, zval, ['   ' tag ], 'FontSize', fontSizes(1))
                                hold on

                                tmp = max(xval);
                                if tmp>max_x, max_x = tmp; end
                                tmp = min(xval);
                                if tmp<min_x, min_x= tmp; end
                                tmp = max(yval);
                                if tmp>max_y, max_y = tmp; end
                                tmp = min(yval);
                                if tmp<min_y, min_y= tmp; end
                                tmp = max(zval);
                                if tmp>max_z, max_z = tmp; end
                                tmp = min(zval);
                                if tmp<min_z, min_z= tmp; end
                                
                            end


                    case {'C'}
                        if ~( isfield(runData.SingleElement, yValue) && ...
                                strcmpi(get(handles.(['Plot_Result_' yValue]), 'checked'), 'on') )
                            continue
                        end
                        cIdx = colorIdx.(colorTag).(yValue);
                        
                        for row=1:6
                            for col=1:6
                                if runData.SingleElement.C(row,col)==0, continue, end
                                tLabel = sprintf('C%d%d', row, col);
                                
                                xval = runData.SingleElement.(xValue);
                                yval = runData.SingleElement.C(row,col);
                                zval = find(strcmpi(zString, tLabel));

                                if writePlotData
                                    fprintf(fid, '\n#\n# %s: %s, %s\n#\n', tag, xValue, tLabel);
                                    fprintf(fid, '%g, %g\n', xval, yval);
                                end

                                hPt = plot3(xval, yval, zval, ...
                                    'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                    'Marker', s{zval}, 'MarkerFaceColor', c(cIdx,:));
                                % text(xval, yval, zval, ['   ' tag ], 'FontSize', fontSizes(1))
                                if find(strcmpi(tLegend, tag))
                                    set(get(get(hPt,'Annotation'),'LegendInformation'), ...
                                        'IconDisplayStyle', 'off');
                                else
                                    tLegend{end+1} = tag;
                                end
                                hold on

                                tmp = max(xval);
                                if tmp>max_x, max_x = tmp; end
                                tmp = min(xval);
                                if tmp<min_x, min_x= tmp; end
                                tmp = max(yval);
                                if tmp>max_y, max_y = tmp; end
                                tmp = min(yval);
                                if tmp<min_y, min_y= tmp; end
                                tmp = max(zval);
                                if tmp>max_z, max_z = tmp; end
                                tmp = min(zval);
                                if tmp<min_z, min_z= tmp; end

                            end
                        end
                end


            case 2 % a binary

                % binaries are included in plot ?
                 if ~strcmpi(get(handles.Plot_Include_binaries, 'checked'), 'on')
                    continue
                 end

                 % xValue is included in plot ?
                if ~strcmpi(get(handles.(['Plot_Result_' xValue]), 'checked'), 'on')
                    continue
                end

                % Alloy sctructure available ?
                if ~isfield(runData, 'Alloy')
                    continue
                end

                switch yValue

                    case 'E1'

                        % is yValue calculated ?
                        if ~isfield(runData.Alloy, 'BLattices') || ~isfield(runData.Alloy, yValue)
                            continue
                        end

                        for i=1:size(lattice,2)
                            for j=1:objType
                                for k=1:objType % 1:(j-1)
                                    if k>size(runData.Alloy.BLattices,3), continue, end

                                    BLattices = runData.Alloy.BLattices{i, j, k};
                                    if isempty(BLattices), continue, end
                                    latticeName = strrep(BLattices, '-', '_');
                                    cIdx = colorIdx.(latticeName).(yValue);

                                    tLegend{end+1} = [BLattices ': E1'];

                                    zval = runData.Alloy.E1(:, i, j, k);
                                    yval = runData.Alloy.a1(:, i, j, k);
                                    xval = repmat(runData.Alloy.(xValue)(i,j,k), size(yval) );
                                    if writePlotData
                                        fprintf(fid, '\n#\n# %s: %s, a1, E1 (%s)\n#\n', tag, xValue, BLattices);
                                        fprintf(fid, '%g, %g, %g\n', [xval; yval; zval]);
                                    end

                                    plot3(xval, yval, zval, ...
                                        'Linestyle', '-', 'Color', c(cIdx,:),  'Linewidth', 3)
                                    hold on

                                    tmp = max(xval);
                                    if tmp>max_x, max_x = tmp; end
                                    tmp = min(xval);
                                    if tmp<min_x, min_x= tmp; end
                                    tmp = max(yval);
                                    if tmp>max_y, max_y = tmp; end
                                    tmp = min(yval);
                                    if tmp<min_y, min_y= tmp; end
                                    tmp = max(zval);
                                    if tmp>max_z, max_z = tmp; end
                                    tmp = min(zval);
                                    if tmp<min_z, min_z= tmp; end
                                    
                                end
                            end
                        end
                        
                        
                        
                    case {'C'}
                        
                        % is yValue calculated ?
                        if ~isfield(runData.Alloy, 'BLattices') || ~isfield(runData.Alloy, yValue)
                            continue
                        end
                        
                        for i=1:size(lattice,2)
                            for j=1:objType
                                for k=1:objType % 1:(j-1)
                                    if k>size(runData.Alloy.BLattices,3), continue, end
                                    
                                    BLattices = runData.Alloy.BLattices{i, j, k};
                                    if isempty(BLattices), continue, end
                                    latticeName = strrep(BLattices, '-', '_');
                                    cIdx = colorIdx.(latticeName).(yValue);
                                    
                                    for row=1:6
                                        for col=1:6
                                            yval = runData.Alloy.C(row, col, i, j, k);
                                            if yval==0.0, continue, end
                                            
                                            
                                            xval = runData.Alloy.(xValue)(i, j, k);
                                            tLabel = sprintf('C%d%d', row, col);
                                            zval = find(strcmpi(zString, tLabel));
                                            
                                            if writePlotData
                                                fprintf(fid, '\n#\n# %s: %s, %s\n#\n', tag, xValue, tLabel);
                                                fprintf(fid, '%g, %g\n', xval, yval);
                                            end
                                            
                                            hPt = plot3(xval, yval, zval, ...
                                                'Linestyle', 'none', 'Color', c(cIdx,:), ...
                                                'Marker', s{zval}, 'MarkerFaceColor', c(cIdx,:));
                                            % text(xval, yval, zval, ['   ' tag ], 'FontSize', fontSizes(1))
                                            if find(strcmpi(tLegend, BLattices))
                                                set(get(get(hPt,'Annotation'),'LegendInformation'), ...
                                                    'IconDisplayStyle', 'off');
                                            else
                                                tLegend{end+1} = BLattices;
                                            end
                                            hold on
                                            
                                            tmp = max(xval);
                                            if tmp>max_x, max_x = tmp; end
                                            tmp = min(xval);
                                            if tmp<min_x, min_x= tmp; end
                                            tmp = max(yval);
                                            if tmp>max_y, max_y = tmp; end
                                            tmp = min(yval);
                                            if tmp<min_y, min_y= tmp; end
                                            tmp = max(zval);
                                            if tmp>max_z, max_z = tmp; end
                                            tmp = min(zval);
                                            if tmp<min_z, min_z= tmp; end
                                            
                                        end
                                    end
                                end
                            end
                        end
                        
 
                end

        end

    end

    if any(isinf([min_x,max_x,min_y,max_y,min_z,max_z]))
        plot(0,0)
        text(0, 0, [yValue ' vs. ' xValue ' - not a valid combination'],  ...
            'HorizontalAlignment', 'center') %, 'FontSize', fontSizes(1))

    else
        ext_x = 0.1*(max_x - min_x);
        if ext_x==0
            ext_x = 0.1*max_x;
        end
        %xlim([floor(min_x-ext_x), ceil(max_x+ext_x)])
        xlim(sort([min_x-ext_x, max_x+ext_x]))
        ext_y = 0.1*(max_y - min_y);
        %ylim([floor(min_y-ext_y), ceil(max_y+ext_y)])
        ylim([min_y-ext_y, max_y+ext_y])
        ext_z = 0.1*(max_z - min_z);
        zlim([floor(min_z-ext_z), ceil(max_z+ext_z)])        
    end

    if ~isempty(zString)
        set(gca, 'ZTick', min_z:max_z)
        set(gca, 'ZTickLabel', zString, 'FontSize', fontSizes(3))
    end
    % font sizes [points, legend, axis, title]
    xlabel(xAxisLabel, 'FontSize', fontSizes(3))
    ylabel(yAxisLabel, 'FontSize', fontSizes(3))
    zlabel(zAxisLabel, 'FontSize', fontSizes(3))
    title([yTitle ' vs. ' xTitle], 'FontSize', fontSizes(4))
    if ~isempty(tLegend)
        legend(tLegend, 'Interpreter', 'none', 'Location', 'Best', 'FontSize', fontSizes(2))
    end
    grid on
    hold off
   
end


if writePlotData
    fclose(fid);
end

