function MEAM_Object = MPC_Run_LAMMPS( handles, libFile, paramFile, ...
    MEAM_Object, objectList, calculationList, traceOn )
% run LAMMPS for each (object+calculation) combination; re-use results of
% single-element calculations in binary and ternary calculations; re-use
% results of binary calculations in ternary calculations

elementList = sprintf('%s ', MEAM_Object{1:max(MEAM_Object{end,1}),2});
elementList = strtrim( elementList ); % remove leading/trailing spaces
numObjects = length(objectList);

% loop over objects in list
for idxList=1:numObjects

    idxObject = objectList(idxList);
    elementIdx = MEAM_Object{idxObject,1};
    objectSize = size(elementIdx,2);
    tag = MEAM_Object{idxObject,2};
    
    SEflags = 0;
    calculateFlag = calculationList{idxList}{1}>0;
    for i=1:length(calculationList{idxList}{1})
        SEflags = SEflags + 2^(i-1)*calculateFlag(i);
    end
    Bflags = 0;
    calculateFlag = calculationList{idxList}{2}>0;
    for i=1:length(calculationList{idxList}{2})
        if calculationList{idxList}{2}(i)>0
            Bflags = Bflags + 2^(i-1)*calculateFlag(i);
        end
    end
    Tflags = 0;
    calculateFlag = calculationList{idxList}{3}>0;
    for i=1:length(calculationList{idxList}{3})
        Tflags = Tflags + 2^(i-1)*calculateFlag(i);
    end
    
    lattice1 = cell(size(elementIdx));
    lattice2 = cell(size(elementIdx));
    lattice3 = cell(size(elementIdx));
    alat  = zeros(size(elementIdx));
    E0    = zeros(size(elementIdx));
    alpha = zeros(size(elementIdx));

    SESet = zeros(1,objectSize);
    SERun = zeros(1,objectSize);
    BSet = zeros(objectSize,objectSize);
    BRun = zeros(objectSize,objectSize);
    TSet = zeros(objectSize,objectSize,objectSize);
    TRun = zeros(objectSize,objectSize,objectSize);
 
    jdx = 0;
    for idx=elementIdx
        jdx = jdx + 1;
        lattice1{jdx} = handles.lattices{MEAM_Object{idx,5}(1)};
        lattice2{jdx} = handles.lattices{MEAM_Object{idx,5}(2)};
        lattice3{jdx} = handles.lattices{MEAM_Object{idx,5}(3)};
        alat(jdx) = MEAM_Object{idx,4}.alat;
        E0(jdx) = MEAM_Object{idx,4}.esub;
        alpha(jdx) = MEAM_Object{idx,4}.alpha;
    end
%     display(lattice1)
%     display(lattice2)
%     display(lattice3)
%     display(alat)
%     display(E0)
%     display(alpha)

    dataold = MEAM_Object{idxObject,6};

    switch objectSize
        case 1
            if traceOn
                display([tag ' Single Element calculations ' num2str(SEflags)])
            end
            BLattices = {};
            SESet(:) = SEflags;
            SERun(:) = SEflags;

        case 2
            if traceOn
                display([tag ' Binary calculations ' num2str(Bflags)])
            end
            BLattices = handles.BLattices(MEAM_Object{idxObject,5})';
            SESet(:) = SEflags;
            BSet(:) = Bflags;
            BRun(:) = Bflags;
        
        case 3
            if traceOn
                display([tag ' Ternary calculation ' num2str(Tflags)])
            end
            BLattices = {};        
            SESet(:) = SEflags;
            BSet(:) = Bflags;
            TSet(:) = Tflags;
            TRun(:) = Tflags;
    end

%     % run LAMMPS
%     h = msgbox(['Please wait until ' mesg ...
%         ' for ' tag ' completes'], ['LAMMPS_Run(' tag ')'], 'warn', 'modal');
    MEAM_Object{idxObject,6} = lammps_MPCv2(handles.LAMMPS_ptr, ...
        libFile, paramFile, elementList, elementIdx, ...
        lattice1, lattice2, lattice3, alat, E0, alpha, BLattices, dataold, ...
        'SingleElementSetupflags', SESet, ...
        'SingleElementRunflags', SERun, ...
        'BinarySetupflags', BSet, ...
        'BinaryRunflags', BRun, ...
        'TernarySetupflags', TSet, ...
        'TernaryRunflags', TRun);
%     if ishandle(h), delete(h); end
    %display(['Returned values for ' tag])
    %Show_Run_Results(MEAM_Object{idxObject,6})

    switch objectSize
        
        case 1 % copy SingleElement results to applicable binaries, ternaries
            elt1Idx = MEAM_Object{idxObject,1}(1);
            for i=idxList+1:numObjects
                jdx = objectList(i);
                pos1 = find(MEAM_Object{jdx,1}==elt1Idx,1);
                if isempty(pos1), continue, end
                if traceOn
                    display(['  - used as element ' num2str(pos1) ' in ' MEAM_Object{jdx,2}])
                end
                copyList = {'E1', 'a1', 'box', 'E0'};
                if ~isfield(MEAM_Object{jdx,6}, 'SingleElement')
                    for j=1:length(MEAM_Object{jdx,1})
                        for k=1:length(copyList)
                            MEAM_Object{jdx,6}.SingleElement(j).(copyList{k}) = 0;
                        end
                    end
                end
                for k=1:length(copyList)
                    MEAM_Object{jdx,6}.SingleElement(pos1).(copyList{k}) = ...
                        MEAM_Object{idxObject,6}.SingleElement.(copyList{k});
                end 
            end
            
        case 2 % copy binary Alloy results to ternaries

            % Substitution energy
            if isfield(MEAM_Object{idxObject,6}.Alloy, 'SubsLabel')
                for j=1:size(MEAM_Object{idxObject,1},2)
                    elt1Idx = MEAM_Object{idxObject,1}(j);
                    for k=1:size(MEAM_Object{idxObject,1},2)
                        elt2Idx = MEAM_Object{idxObject,1}(k);
                        if isempty(MEAM_Object{idxObject,6}.Alloy.SubsLabel{j,k}), continue, end
                        
                        for i=idxList+1:numObjects
                            jdx = objectList(i);
                            pos1 = find(MEAM_Object{jdx,1}==elt1Idx,1);
                            pos2 = find(MEAM_Object{jdx,1}==elt2Idx,1);
                            if isempty(pos1) || isempty(pos2), continue, end
                            if traceOn
                                display(sprintf('  - used in Substitution(%d,%d) of %s', pos1, pos2, MEAM_Object{jdx,2}))
                            end
                            if ~isfield(MEAM_Object{jdx,6}, 'Alloy')
                                MEAM_Object{jdx,6}.Alloy.temp = 0;
                            end
                            MEAM_Object{jdx,6}.Alloy.SubsLabel{pos1,pos2} = MEAM_Object{idxObject,6}.Alloy.SubsLabel{j,k};
                            MEAM_Object{jdx,6}.Alloy.Substitution(pos1,pos2) = MEAM_Object{idxObject,6}.Alloy.Substitution(j,k);
                        end
                        
                    end
                end
            end
            
            % ISubstitution energy
            if isfield(MEAM_Object{idxObject,6}.Alloy, 'ISubsLabel')
                for j=1:size(MEAM_Object{idxObject,1},2)
                    elt1Idx = MEAM_Object{idxObject,1}(j);
                    for k=1:size(MEAM_Object{idxObject,1},2)
                        elt2Idx = MEAM_Object{idxObject,1}(k);
                        if isempty(MEAM_Object{idxObject,6}.Alloy.ISubsLabel{j,k}), continue, end
                        
                        for i=idxList+1:numObjects
                            jdx = objectList(i);
                            pos1 = find(MEAM_Object{jdx,1}==elt1Idx,1);
                            pos2 = find(MEAM_Object{jdx,1}==elt2Idx,1);
                            if isempty(pos1) || isempty(pos2), continue, end
                            if traceOn
                                display(sprintf('  - used in ISubstitution(%d,%d) of %s', pos1, pos2, MEAM_Object{jdx,2}))
                            end
                            if ~isfield(MEAM_Object{jdx,6}, 'Alloy')
                                MEAM_Object{jdx,6}.Alloy.temp = 0;
                            end
                            MEAM_Object{jdx,6}.Alloy.ISubsLabel{elt1Idx,elt2Idx} = MEAM_Object{idxObject,6}.Alloy.ISubsLabel{j,k};
                            MEAM_Object{jdx,6}.Alloy.ISubstitution(elt1Idx,elt2Idx) = MEAM_Object{idxObject,6}.Alloy.ISubstitution(j,k);
                        end
                        
                    end
                end
            end
            
    end
    
end


function Show_Run_Results(runData)
% fieldnames in structure returned by LAMMPS run

f1List = fieldnames(runData);
for i=1:length(f1List)
    if isstruct( runData.(f1List{i}) )
        len = size(runData.(f1List{i}),2 );
        f2List = fieldnames(runData.(f1List{i}));
        level3 = false;
        for j=1:length(f2List)
            if isstruct( runData.(f1List{i})(len).(f2List{j}) )
                f3List = fieldnames(runData.(f1List{i})(len).(f2List{j}));
                display([' - ' f1List{i} '.' f2List{j} ':' sprintf(' %s ', f3List{:})])
                level3 = true;
            end
        end
        if ~level3
            display([' - ' f1List{i} ':' sprintf(' %s ', f2List{:})])
        end
    end
end
