function data=lammps_MPCv2(ptr,meam_lib,meam_param,element_list,type,lattice1,lattice2,lattice3,alat,E0,alpha,BLattices,dataold,varargin)
    %gather options
    Options=initiateOptions(size(type,2));
    if ~isempty(varargin)
         Options=processOptions(Options,varargin);
    end
    if ~isstruct(dataold)
        dataold.SingleElement(1).temp=0;
    end
    %setup initial guess values
    if ~isfield(dataold.SingleElement(1),'E0')
        for i=1:size(lattice1,2)
            dataold.SingleElement(i).E0=-E0(i);
        end
    end
    if ~isfield(dataold.SingleElement(1),'box')
        for i=1:size(lattice1,2)
            dataold.SingleElement(i).box=alat(i);
        end
    end
    if ~isfield(dataold.SingleElement(1),'alpha')
        for i=1:size(lattice1,2)
            dataold.SingleElement(i).alpha=alpha(i);
        end
    end
    %prepare directory
    if ~isdir('atomic_data')
        mkdir('atomic_data');
    end

% Single element calculations
    for i=1:size(lattice1,2)
%         display(sprintf('Single element calculations: %d of %s', type(i), element_list))
        data.SingleElement(i)=lammps_SingleElement(ptr,lattice1{i},lattice2{i},lattice3{i},alat(i),meam_lib,meam_param,element_list,type(i),dataold.SingleElement(i),Options,i);
    end
% Alloy calculations
    if isfield(dataold,'Alloy')
%         display('Alloy calculations')
        data.Alloy=dataold.Alloy;
    end
    data=lammps_Alloy(ptr,lattice1,BLattices,alat,type,meam_lib,meam_param,element_list,data,Options);
%     if isfield(data,'Alloy')
%         fieldnames(data.Alloy)
%     end
    

end

function data=lammps_SingleElement(ptr,lattice1,lattice2,lattice3,alat,meam_lib,meam_param,element_list,type,data,Options,i)
    Elements = regexp(element_list,' ','split');
    ntypes = length(Elements);
    p=getLatticeConversion(lattice1);
    r=alat/p;
    
    SetupMatrix=zeros(1,8);
    RunMatrix=SetupMatrix;
    Setup=Options.SESetupflags(i);
    Run=Options.SERunflags(i);
    for j=7:-1:0
        SetupMatrix(j+1)=floor(Setup/2^j)>0;
        RunMatrix(j+1)=floor(Run/2^j)>0;
        Setup=mod(Setup,2^j);
        Run=mod(Run,2^j);
    end
    If=Options.Interstitialflags(i);
    Sf=Options.Surfaceflags(i);
    Gf=Options.GSFEflags(i);
    for j=5:-1:0
        SOptions.GSFEflags(j+1)=floor(Gf/2^j);
        Gf=mod(Gf,2^j);
        if j==5
            continue;
        end
        SOptions.Interstitialflags(j+1)=floor(If/2^j);
        SOptions.Surfaceflags(j+1)=floor(Sf/2^j);
        If=mod(If,2^j);
        Sf=mod(Sf,2^j);
    end
    SOptions.InterstitialSize=Options.InterstitialSize(i);
    SOptions.VacancySize=Options.VacancySize(i);
    SOptions.GSFEDistance=Options.GSFEDistance(i);
    SOptions.SurfaceDistance=Options.SurfaceDistance(i);
    SOptions.EVmin=Options.EVmin(i);
    SOptions.EVmax=Options.EVmax(i);
    SOptions.CustomElasticConstants=Options.ElasticConstants{i};
    SOptions.GSFErelax=Options.GSFErelax(i);
    % Generate rose curve data.
    [data.rose_a, data.rose_E]=rose_Curve(lattice1,81,SOptions,alat,data.alpha,data.E0);
    if all(SetupMatrix==0) && all(RunMatrix==0)
        return % only if nothing else is being run        
    end
% Write primary EV atomic data - Only has to be done once
    %Write atomic position files:
    if SetupMatrix(1)
    write_atom_positions_EV(lattice1,r,type,ntypes,Elements);
    end

% Calculate primary EV curve - must be done at least once before the rest of the calculations
    %Primary EV curve
    if RunMatrix(1)
    [data.E1, data.a1, data.box, data.E0]=lammps_EV(ptr,lattice1,81,meam_lib,meam_param,element_list,type,SOptions);
    
%     display(' - [a1,E1], box, E0')

    %box and E0 are needed from the primary EV results for the
    %rest of the calculations. 
    end
    % Write all the rest of the single element atomic data - Only has to be done once
    if SetupMatrix(2)
    write_atom_positions_EV(lattice2,r,type,ntypes,Elements);
    end
    if SetupMatrix(3)
    write_atom_positions_EV(lattice3,r,type,ntypes,Elements);
    end
    if SetupMatrix(4)
    write_atom_positions_Elastic(lattice1,data.box,type,ntypes,Elements);
    end
    if SetupMatrix(5)
    write_atom_positions_Vacancy(lattice1,data.box,type,ntypes,Elements,SOptions);
    end
    if SetupMatrix(6)
    write_atom_positions_Interstitials(lattice1,data.box,type,ntypes,Elements,SOptions);
    end
    if SetupMatrix(7)
    write_atom_positions_Surfaces(lattice1,data.box,type,ntypes,Elements,SOptions);
    end
    if SetupMatrix(8)
    write_atom_positions_GSFE(lattice1,data.box,type,ntypes,Elements,SOptions);
    end
% Calculate the rest of the single element stuff
    %Secondary EV curve
    if RunMatrix(2)
    [data.E2, data.a2]=lammps_EV(ptr,lattice2,81,meam_lib,meam_param,element_list,type,SOptions);
%     display(' - [a2,E2]')
    end
    %Secondary2 EV curve
    if RunMatrix(3)
    [data.E3, data.a3]=lammps_EV(ptr,lattice3,81,meam_lib,meam_param,element_list,type,SOptions);
%     display(' - [a3,E3]')
    end
    %Elastic constants. 3 for cubic lattices, 5 to 21 for others.
    if RunMatrix(4)
    data.C=lammps_Elastic(ptr,lattice1,meam_lib,meam_param,element_list,type,SOptions);
%     display(' - C(:,:)')
    end
    %returns a 6x6 matrix. Only needed values are non-zero. Indices
    %correspond to name: C(1,1) is C11, C(4,4) is C44, etc.

    %Vacancy energy
    if RunMatrix(5)
        if size(data.box)<3
            warning('Running vacancy before EV curve is not recommended');
        end
    data.Evac=lammps_Vacancy(ptr,meam_lib,meam_param,element_list,type,data.E0);
%     display(' - Evac')
    end
    %Interstitial Energy. Various types for different lattices.
    if RunMatrix(6)
        if size(data.box)<3
            warning('Running interstitials before EV curve is not recommended');
        end
    [data.Einterstitials, data.interstials]=lammps_Interstitial(ptr,lattice1,meam_lib,meam_param,element_list,type,data.E0,SOptions);
%     display(' - Einterstitials')
    end
    %Surface energies. Various types for different lattices.
    if RunMatrix(7)
        if size(data.box)<3
            warning('Running surfaces before EV curve is not recommended');
        end        
    [data.ESurfaces, data.Surfaces]=lammps_Surface(ptr,lattice1,meam_lib,meam_param,element_list,type,data.E0,SOptions);
%     display(' - ESurfaces')
    end
    %GSFE. Two curves for FCC, one for BCC. Five for HCP. See
    %getGSFE_Plane_And_Direction for list of curve names.
    if RunMatrix(8)
        if size(data.box)<3
            warning('Running GSFE before EV curve is not recommended');
        end        
    [data.GSFEs, data.shift, data.planes]=lammps_GSFE(ptr,lattice1,81,meam_lib,meam_param,element_list,type,data.E0,SOptions);
%     display(' - GSFEs')
    end
%     cd ..
%     lammps_command(ptr,'shell cd ..');
end

function data=lammps_Alloy(ptr,lattice1,BLattices,alat,types,meam_lib,meam_param,element_list,data,Options)
    SingleElement=data.SingleElement;
    % preliminary calculations
    
    Elements = regexp(element_list,' ','split');
    ntypes = size(types,2);
    ntypes1 = size(Elements,2);
    p = zeros(1,ntypes);
    for i=1:ntypes
        p(i)=getLatticeConversion(lattice1{i});
    end
    r=alat./p;
    if isfield(data,'Alloy')
        alloy = data.Alloy;
    else
        alloy.temp=0;
    end
    if ~isfield(alloy,'box')
        for i=1:size(BLattices,2)
            for j=1:ntypes
                for k=1:(j-1)
                    alloy.box{i,j,k} = getLatticeConversion(BLattices{i})*mean([alat(j) alat(k)]);
                end
            end
        end
    end
    if ~isfield(alloy,'Substitution')
        alloy.Substitution = zeros(ntypes,ntypes);
    end
    SetupMatrix=zeros(ntypes,ntypes,4);
    RunMatrix=SetupMatrix;
    for i=1:ntypes
        for j=1:ntypes
            Setup = Options.BinarySetupflags(i,j);
            Run= Options.BinaryRunflags(i,j);
            for k=3:-1:0
                SetupMatrix(i,j,k+1)=floor(Setup/2^k);
                RunMatrix(i,j,k+1)=floor(Run/2^k);
                Setup=mod(Setup,2^k);
                Run=mod(Run,2^k);
            end
        end
    end
    %write binary EV curve data all combinations of two elements
    for i=1:size(BLattices,2)
        for j=1:ntypes
            for k=1:(j-1)
                if SetupMatrix(j,k,1)
                    write_atom_positions_EV(BLattices{i},r([j k]),types([j k]),ntypes1,Elements);
                    %display(sprintf('write_atom_positions_EV(%s), r([%d, %d]) ...', BLattices{i}, j, k))
                end
                if ~strncmp(BLattices{i},'B',1) && ~strcmp(BLattices{i},'L12') && SetupMatrix(k,j,1)
                    write_atom_positions_EV(BLattices{i},r([k j]),types([k j]),ntypes1,Elements);
                    %display(sprintf('write_atom_positions_EV(%s), r([%d, %d]) ...', BLattices{i}, k, j))
                end
                if (strncmp(BLattices{i},'B',1) || strcmp(BLattices{i},'L12')) && SetupMatrix(k,j,1) && ~SetupMatrix(j,k,1)
                    write_atom_positions_EV(BLattices{i},r([j k]),types([j k]),ntypes1,Elements);
                    %display(sprintf('write_atom_positions_EV(%s), r([%d, %d]) ...', BLattices{i}, j, k))
                end
            end
        end
    end
    
    %Run binary EV curves for all combinations of two elements
    for j=1:ntypes
        for k=1:(j-1)
            BLattices1=BLattices;
            if ~isempty(Options.BLEVflags{j,k})
                BLattices1=regexp(BLEVflags,',','split');
            end
            for i=1:size(BLattices1,2)
                if RunMatrix(j,k,1) || ((strncmp(BLattices1{i},'B',1) || strcmp(BLattices1{i},'L12')) && RunMatrix(k,j,1))
                    tName = [BLattices1{i} '-' Elements{types(j)} '-' Elements{types(k)}];
                    if size(SingleElement(j).box) < 3 
                        warning([tName ': Running binary EV curves without single element ' Elements{types(j)} ' EV curve is not recommended']);
                    elseif size(SingleElement(k).box) < 3
                        warning([tName ': Running binary EV curves without single element ' Elements{types(k)} ' EV curve is not recommended']);
                    end
                    
                    EVOptions.EVmax=Options.BEVmax(j,k);
                    EVOptions.EVmin=Options.BEVmin(j,k);
                    [alloy.E1(:,i,j,k), alloy.a1(:,i,j,k), alloy.box{i,j,k}, alloy.E0(i,j,k)]=lammps_EV(ptr,BLattices1{i},81,meam_lib,meam_param,element_list,types([j k]),EVOptions);
                    alloy.BLattices{i,j,k}=[BLattices1{i} '-' Elements{types(j)} '-' Elements{types(k)}];
                    [~,type]=generateAlloyUnitCell(BLattices1{i});
                    alloy.H(i,j,k) = (alloy.E0(i,j,k)*size(type,2)-SingleElement(j).E0*sum(type==1)-SingleElement(k).E0*sum(type==2))/size(type,2);
                    alloy.Nx(i,j,k) = sum(type==1)/size(type,2);

%                     display( sprintf('A. %s: E0, box, [E1,a1], H, Nx', alloy.BLattices{i,j,k}) )
                    
                    if strncmp(BLattices1{i},'B',1) || strcmp(BLattices1{i},'L12')
                        alloy.E1(:,i,k,j) = alloy.E1(:,i,j,k);
                        alloy.a1(:,i,k,j) = alloy.a1(:,i,j,k);
                        alloy.H(i,k,j) = alloy.H(i,j,k);
                        alloy.Nx(i,k,j) = alloy.Nx(i,j,k);
                    end
                end
            end
            BLattices1=BLattices;
            if ~isempty(Options.BLEVflags{k,j})
                BLattices1=regexp(BLEVflags,',','split');
            end
            for i=1:size(BLattices1,2)
                if ~strncmp(BLattices1{i},'B',1) && ~strcmp(BLattices1{i},'L12') && RunMatrix(k,j,1)
                    if size(SingleElement(j).box) < 3 | size(SingleElement(k).box) < 3
                        warning('Running binary EV curves without single element EV curves is not recommended');
                    end
                    
                    EVOptions.EVmax=Options.BEVmax(k,j);
                    EVOptions.EVmin=Options.BEVmin(k,j);
                    [alloy.E1(:,i,k,j), alloy.a1(:,i,k,j), alloy.box{i,k,j}, alloy.E0(i,k,j)]=lammps_EV(ptr,BLattices1{i},81,meam_lib,meam_param,element_list,types([k j]),EVOptions);
                    alloy.BLattices{i,k,j}=[BLattices1{i} '-' Elements{types(k)} '-' Elements{types(j)}];
                    [~,type]=generateAlloyUnitCell(BLattices1{i});
                    alloy.H(i,k,j) = (alloy.E0(i,k,j)*size(type,2)-SingleElement(k).E0*sum(type==1)-SingleElement(j).E0*sum(type==2))/size(type,2);
                    alloy.Nx(i,k,j) = sum(type==2)/size(type,2);
                    
%                     display( sprintf('B. %s: E0, box, [E1,a1], H, Nx', alloy.BLattices{i,k,j}) )
                    
                end
            end
        end
    end
    
    %Write all the other alloy files
    for j=1:ntypes
        for k=1:(j-1)
            %write elastic calculation data for all combinations of two
            %elements
            for i=1:size(BLattices,2)
                if SetupMatrix(j,k,2) || ((strncmp(BLattices{i},'B',1) || strcmp(BLattices{i},'L12')) && SetupMatrix(k,j,2))
                %if size(alloy.box{i,j,k}) < 3
                %    warning('Binary elastic data should not be produced without finding binary EV curves first');
                %end
                write_atom_positions_Elastic(BLattices{i},alloy.box{i,j,k},types([j k]),ntypes1,Elements);
                end
                if ~strncmp(BLattices{i},'B',1) && ~strcmp(BLattices{i},'L12') && SetupMatrix(k,j,2)
                %if size(alloy.box{i,j,k}) < 3
                %    warning('Binary elastic data should not be produced without finding binary EV curves first');
                %end
                write_atom_positions_Elastic(BLattices{i},alloy.box{i,j,k},types([k j]),ntypes1,Elements);
                end
            end
            %write substitution calculation data for all combinations of two
            %elements
            if SetupMatrix(j,k,3)
            write_atom_positions_Substitution(lattice1{j},SingleElement(j).box,types([j k]),ntypes1,Elements);
            end
            if SetupMatrix(k,j,3)
            write_atom_positions_Substitution(lattice1{k},SingleElement(k).box,types([k j]),ntypes1,Elements);
            end
            if SetupMatrix(j,k,4)
            write_atom_positions_InterstitialSubstitution(lattice1{j},SingleElement(j).box,types([j k]),ntypes1,Elements);
            end
            if SetupMatrix(k,j,4)
            write_atom_positions_InterstitialSubstitution(lattice1{k},SingleElement(k).box,types([k j]),ntypes1,Elements);
            end
        end
    end
    %write binding energy data for binary and ternary substitutions of all
    %elements
    for i=1:ntypes
        for j=1:ntypes
            if i==j
                continue;
            end
            for k=1:j
                if k==i || ~(Options.TernarySetupflags(i,j,k) || Options.TernarySetupflags(i,k,j))
                    continue;
                end
                write_atom_positions_BindingEnergy(lattice1{i},SingleElement(i).box,[types(i) types(j) types(k)],ntypes1,Elements);
            end
        end
    end
    
    % Calculate all the rest of the alloy results
    for j=1:ntypes
        for k=1:(j-1)
            BLattices1=BLattices;
            if ~isempty(Options.BLElasticflags{j,k})
                BLattices1=regexp(BLEVflags,',','split');
            end
            %calculate elastic calculation data for 1st and 2nd element
            for i=1:size(BLattices1,2)
                if RunMatrix(j,k,2) || ((strncmp(BLattices1{i},'B',1) || strcmp(BLattices1{i},'L12')) && RunMatrix(k,j,2))
                ElOptions.CustomElasticConstants=Options.BElasticConstants{j,k};
                alloy.C(:,:,i,j,k)=lammps_Elastic(ptr,BLattices1{i},meam_lib,meam_param,element_list,types([j k]),ElOptions);
                alloy.C(:,:,i,k,j)=alloy.C(:,:,i,j,k);
                
%                 display( sprintf('C. %s: C(:,:,%d,%d,%d)', alloy.BLattices{i,j,k}, i, j, k) )
                    
                end
            end
            BLattices1=BLattices;
            if ~isempty(Options.BLElasticflags{k,j})
                BLattices1=regexp(BLEVflags,',','split');
            end
            for i=1:size(BLattices1,2)
                if ~strncmp(BLattices1{i},'B',1) && ~strcmp(BLattices1{i},'L12') && RunMatrix(k,j,2)
                ElOptions.CustomElasticConstants=Options.BElasticConstants{k,j};
                alloy.C(:,:,i,k,j)=lammps_Elastic(ptr,BLattices1{i},meam_lib,meam_param,element_list,types([k j]),ElOptions);
                
%                 display( sprintf('D. %s: C(:,:,%d,%d,%d)', alloy.BLattices{i,k,j}, i, k, j) )
                    
                end
            end
            %substitution
            if RunMatrix(j,k,3)
            if size(SingleElement(j).box) < 3 | size(SingleElement(k).box) < 3
                 warning('Running substitution calculations before single element EV curves is not recommended');
            end
            alloy.Substitution(j,k)=lammps_Substitution(ptr,meam_lib,meam_param,element_list,types([j k]),[SingleElement(j).E0 SingleElement(k).E0]);
            alloy.SubsLabel{j,k}=['Substitution: ' Elements{types(k)} ' into ' Elements{types(j)}];
                    
%             display( sprintf('E. %s', alloy.SubsLabel{j,k}) )
                    
            end
            if RunMatrix(k,j,3)
            if size(SingleElement(j).box) < 3 | size(SingleElement(k).box) < 3
                 warning('Running substitution calculations before single element EV curves is not recommended');
            end    
            alloy.Substitution(k,j)=lammps_Substitution(ptr,meam_lib,meam_param,element_list,types([k j]),[SingleElement(k).E0 SingleElement(j).E0]);
            alloy.SubsLabel{k,j}=['Substitution: ' Elements{types(j)} ' into ' Elements{types(k)}];
                    
%             display( sprintf('F. %s', alloy.SubsLabel{k,j}) )
                    
            end
            if RunMatrix(j,k,4)
            %interstitialsubstitution
            if size(SingleElement(j).box) < 3 | size(SingleElement(k).box) < 3
                 warning('Running substitution calculations before single element EV curves is not recommended');
            end
            alloy.ISubstitution(j,k)=lammps_InterstitialSubstitution(ptr,meam_lib,meam_param,element_list,types([j k]),[SingleElement(j).E0 SingleElement(k).E0]);
            alloy.ISubsLabel{j,k}=['ISubstitution: ' Elements{types(k)} ' into ' Elements{types(j)}];
                    
%             display( sprintf('G. %s', alloy.ISubsLabel{j, k}) )
                    
            end
            if RunMatrix(k,j,4)
            if size(SingleElement(j).box) < 3 | size(SingleElement(k).box) < 3
                 warning('Running substitution calculations before single element EV curves is not recommended');
            end
            alloy.ISubstitution(k,j)=lammps_InterstitialSubstitution(ptr,meam_lib,meam_param,element_list,types([k j]),[SingleElement(k).E0 SingleElement(j).E0]);
            alloy.ISubsLabel{k,j}=['ISubstitution: ' Elements{types(j)} ' into ' Elements{types(k)}];
                    
%             display( sprintf('H. %s', alloy.ISubsLabel{k,j}) )
                    
            end
        end
    end
    %run binding energy calculations
    for i=1:ntypes
        for j=1:ntypes
            if i==j
                continue;
            end
            for k=1:j
                if k==i || ~(Options.TernaryRunflags(i,j,k) || Options.TernaryRunflags(i,k,j))
                    continue;
                end
                if size(SingleElement(i).box) < 3 | size(SingleElement(j).box) < 3 | size(SingleElement(k).box) < 3
                    warning('Running binding energy calculations before single element EV curves is not recommended');
                end
                if alloy.Substitution(i,j)==0 || alloy.Substitution(i,k)==0
                    error('Cannot run binding energy calculations before running substitution calculations');
                end
                alloy.BindingEnergy(j,k,i)=lammps_BindingEnergy(ptr,meam_lib,meam_param,element_list,[types(i) types(j) types(k)],[SingleElement(i).E0 SingleElement(j).E0 SingleElement(k).E0],[alloy.Substitution(i,j) alloy.Substitution(i,k)]);
                alloy.BindingLabel{j,k,i}=['Binding: ' Elements{types(k)} ' and ' Elements{types(j)} ' in ' Elements{types(i)}];
                alloy.BindingEnergy(k,j,i)=alloy.BindingEnergy(j,k,i);
                alloy.BindingLabel{k,j,i}=alloy.BindingLabel{j,k,i};
                    
%                 display( sprintf('I. %s', alloy.BindingLabel{j,k,i}) )
                                    
            end
        end
    end
    if any(any(any(RunMatrix))) || isfield(alloy, 'BindingEnergy')
        data.Alloy=alloy;
        %display('any(any(any(RunMatrix)))')
        %fieldnames(alloy)
    end
end
% Rose curve
function [a, E] = rose_Curve(lattice1,steps,Options,alat,alpha,E0)
    dmin=Options.EVmin;
    dmax=Options.EVmax;
    amin = alat*dmin;
    amax = alat*dmax;
    a = amin + (0:(steps-1))*(amax-amin)/(steps-1);
    p=getLatticeConversion(lattice1);
    r = a/p;
    astar = alpha * (r / alat * p - 1.0);
    E = E0 * (1.0 + astar).*exp(-astar);
end
% Functions to write atomic data
function write_atom_positions_EV(lattice,r,type,ntypes,Elements)
    c=getLatticeConversion(lattice);
    alat = c*mean(r);
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/EV_' lattice '' typestr '.atoms.data'];
    [unitcell, types, atoms] = generateAlloyUnitCell(lattice);
    type1=zeros(1,size(types,2));
    for i=1:size(type,2)
        type1 = (types==i)*type(i)+type1;
    end
    [atoms1, box, tilt] = triclinic2lammps(atoms*alat,unitcell*alat);
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    exportlammpsdata(data_file,Box,atoms1,type1,ntypes);
end

function write_atom_positions_Elastic(lattice,box,type,ntypes,Elements)
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Elastic_' lattice '' typestr '.atoms.data'];
    [unitcell, types, atoms] = generateAlloyUnitCell(lattice);
    if size(box)>1
        atoms = (box'*(unitcell'^-1*atoms'))'/box(1,1);
        unitcell = box/box(1,1);
    end
    type1=zeros(1,size(types,2));
    for i=1:size(type,2)
        type1 = (types==i)*type(i)+type1;
    end
    [atoms1, box, tilt] = triclinic2lammps(atoms*box(1,1),unitcell*box(1,1));
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    exportlammpsdata(data_file,Box,atoms1,type1,ntypes);
end

function write_atom_positions_Vacancy(lattice,cell,type,ntypes,Elements,Options)
    %generate atomic position file:
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Vacancy' typestr '.atoms.data'];
    dist = Options.VacancySize*ones(1,3);% box dimensions in angstroms
    [unitcell, types, atoms] = generateAlloyUnitCell(lattice);
    if size(cell)>1
        atoms = (cell'*(unitcell'^-1*atoms'))'/cell(1,1);
        unitcell = cell/cell(1,1);
    end
    type1=zeros(1,size(types,2));
    for i=1:size(type,2)
        type1 = (types==i)*type(i)+type1;
    end
    [atoms1, box, tilt] = triclinic2lammps(atoms*cell(1,1),unitcell*cell(1,1));
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    dims = ceil(dist./diag(Box)');
    atoms2=simpleMakeLattice(Box',size(atoms1,1),atoms1,dims);
    Box(:,1) = Box(:,1)*dims(1);Box(:,2)=Box(:,2)*dims(2);Box(:,3)=Box(:,3)*dims(3);
    atoms2=atoms2(2:end,:);
    type1 = repmat(type1,1,dims(1)*dims(2)*dims(3));
    type1 = type1(:,2:end);
    exportlammpsdata(data_file,Box,atoms2,type1,ntypes);
end

function write_atom_positions_Interstitials(lattice,cell,type,ntypes,Elements,Options)
    % there are several relevant types of interstitials for each lattice,
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    dist = Options.InterstitialSize*ones(1,3);% box dimensions in angstroms
    [unitcell, types, atoms] = generateAlloyUnitCell(lattice);
    if size(cell)>1
        atoms = (cell'*(unitcell'^-1*atoms'))'/cell(1,1);
        unitcell = cell/cell(1,1);
    end
    type1=zeros(1,size(types,2));
    for i=1:size(type,2)
        type1 = (types==i)*type(i)+type1;
    end
    clear types
    [offsets, types] = getInterstitial_Offset(lattice,unitcell);
    [atoms1, box, tilt, R] = triclinic2lammps(atoms*cell(1,1),unitcell*cell(1,1));
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    dims = ceil(dist./diag(Box)');
    for i=1:size(offsets,1)
        data_file = ['atomic_data/Interstitial_' types{i} '' typestr '.atoms.data'];
        atoms2=simpleMakeLattice(Box',size(atoms1,1),atoms1,dims);
        Box1(:,1) = Box(:,1)*dims(1);Box1(:,2)=Box(:,2)*dims(2);Box1(:,3)=Box(:,3)*dims(3);
        atoms2(end+1,:)=(R*offsets(i,:)'*cell(1,1))';
        type2 = repmat(type1,1,dims(1)*dims(2)*dims(3));
        type2(1,end+1)=type2(1);
        exportlammpsdata(data_file,Box1,atoms2,type2,ntypes);
    end
end

function write_atom_positions_Surfaces(lattice,cell,type,ntypes,Elements,Options)
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    [xplanes, yplanes] = getSurface_Planes(lattice);
    for i=1:size(xplanes,1)
        xplane=xplanes(i,:);
        yplane=yplanes(i,:);
        surf = ['[' num2str(xplane(1)) '_' num2str(xplane(2)) '_' num2str(xplane(3)) ']x[' num2str(yplane(1)) '_' num2str(yplane(2)) '_' num2str(yplane(3)) ']'];
        data_file=['atomic_data/Surface_' surf  '' typestr '.atoms.data'];
        [unitcell, types, atoms] = generateSurfaceCell(lattice,xplane,yplane,Options.SurfaceDistance,cell);
        type1=zeros(1,size(types,2));
        for j=1:size(type,2)
            type1 = (types==j)*type(j)+type1;
        end
        [atoms1, box, tilt] = triclinic2lammps(atoms*cell(1,1),unitcell*cell(1,1));
        tilt = adjusttilt(box,tilt,[1 1 0]);
        Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
        Box(3,:)=Box(3,:)*2;
        exportlammpsdata(data_file,Box,atoms1,type1,ntypes);
    end
end

function write_atom_positions_GSFE(lattice,cell,type,ntypes,Elements,Options)
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    [xplanes, yplanes] = getGSFE_Plane_And_Direction(lattice);
    for i=1:size(xplanes,1)
        xplane=xplanes(i,:);
        yplane=yplanes(i,:);
        %plane = num2str(abs(integerconvertv(cross(xplane,yplane),.0001)));
        %plane=strrep(plane,'  ','');
        direction=[num2str(xplane(1)) num2str(xplane(2)) num2str(xplane(3))];
        direction2=[num2str(yplane(1)) num2str(yplane(2)) num2str(yplane(3))];
        folder=['atomic_data/GSFE_[' direction ']x[' direction2 ']'];
        data_file=[folder '' typestr '.atoms.data'];
        [unitcell, types, atoms] = generateSurfaceCell(lattice,xplane,yplane,Options.GSFEDistance,cell);
        type1=zeros(1,size(types,2));
        for j=1:size(type,2)
            type1 = (types==j)*type(j)+type1;
        end
        [atoms1, box, tilt] = triclinic2lammps(atoms*cell(1,1),unitcell*cell(1,1));
        %tilt = adjusttilt(box,tilt,[1 1 1]);
        Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
        exportlammpsdata(data_file,Box,atoms1,type1,ntypes);
    end
end

function write_atom_positions_Substitution(lattice,cell,type,ntypes,Elements)
    %generate atomic position file:
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Substitution' typestr '.atoms.data'];
    dist = [10 10 10];% box dimensions in angstroms
    [unitcell, types, atoms] = generateAlloyUnitCell(lattice);
    if size(cell)>1
        atoms = (cell'*(unitcell'^-1*atoms'))'/cell(1,1);
        unitcell = cell/cell(1,1);
    end
    type1=zeros(1,size(types,2));
    for i=1:size(type,2)
        type1 = (types==i)*type(i)+type1;
    end
    [atoms1, box, tilt] = triclinic2lammps(atoms*cell(1,1),unitcell*cell(1,1));
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    dims = ceil(dist./diag(Box)');
    atoms2=simpleMakeLattice(Box',size(atoms1,1),atoms1,dims);
    Box(:,1) = Box(:,1)*dims(1);Box(:,2)=Box(:,2)*dims(2);Box(:,3)=Box(:,3)*dims(3);
    type1 = repmat(type1,1,dims(1)*dims(2)*dims(3));
    type1(1)=type(2);
    exportlammpsdata(data_file,Box,atoms2,type1,ntypes);
end

function write_atom_positions_InterstitialSubstitution(lattice,cell,type,ntypes,Elements)
    % there are several relevant types of interstitials for each lattice,
    % for now we do tetrahedral substitution only
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    dist = [12 12 12];% box dimensions in angstroms
    [unitcell, types, atoms] = generateAlloyUnitCell(lattice);
    if size(cell)>1
        atoms = (cell'*(unitcell'^-1*atoms'))'/cell(1,1);
        unitcell = cell/cell(1,1);
    end
    type1=zeros(1,size(types,2));
    for i=1:size(type,2)
        type1 = (types==i)*type(i)+type1;
    end
    clear types
    [offsets, types] = getInterstitial_Offset(lattice,unitcell);
    [atoms1, box, tilt, R] = triclinic2lammps(atoms*cell(1,1),unitcell*cell(1,1));
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    dims = ceil(dist./diag(Box)');
    for i=1:1%only tetrahedral
        data_file = ['atomic_data/Interstitial_Substitution' typestr '.atoms.data'];
        atoms2=simpleMakeLattice(Box',size(atoms1,1),atoms1,dims);
        Box(:,1) = Box(:,1)*dims(1);Box(:,2)=Box(:,2)*dims(2);Box(:,3)=Box(:,3)*dims(3);
        atoms2(end+1,:)=(R*offsets(i,:)'*cell(1,1))';
        type2 = repmat(type1,1,dims(1)*dims(2)*dims(3));
        type2(1,end+1)=type(2);
        exportlammpsdata(data_file,Box,atoms2,type2,ntypes);
    end
end

function write_atom_positions_BindingEnergy(lattice,cell,type,ntypes,Elements)
    %generate atomic position file:
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Binding_Energy' typestr '.atoms.data'];
    dist = [12 12 12];% box dimensions in angstroms
    [unitcell, types, atoms] = generateAlloyUnitCell(lattice);
    if size(cell)>1
        atoms = (cell'*(unitcell'^-1*atoms'))'/cell(1,1);
        unitcell = cell/cell(1,1);
    end
    type1=zeros(1,size(types,2));
    for i=1:size(type,2)
        type1 = (types==i)*type(i)+type1;
    end
    [atoms1, box, tilt] = triclinic2lammps(atoms*cell(1,1),unitcell*cell(1,1));
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    dims = ceil(dist./diag(Box)');
    atoms2=simpleMakeLattice(Box',size(atoms1,1),atoms1,dims);
    Box(:,1) = Box(:,1)*dims(1);Box(:,2)=Box(:,2)*dims(2);Box(:,3)=Box(:,3)*dims(3);
    type1 = repmat(type1,1,dims(1)*dims(2)*dims(3));
    type1(1)=type(2);
    [~,~,I]=mindist2_c(atoms2,atoms2(1,:));
    type1(I)=type(3);
    exportlammpsdata(data_file,Box,atoms2,type1,ntypes);
end

% Functions to run lammps calculations
function [E, a, box, E0]=lammps_EV(ptr,lattice,steps,meam_lib,meam_param,element_list,type,Options)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    dmin=Options.EVmin;
    dmax=Options.EVmax;
    data_file=['atomic_data/EV_' lattice '' typestr '.atoms.data'];  
    lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
    lammps_command(ptr,'fix 0 all box/relax aniso 0.0');
    lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
    box(3,3)=lammps_extract_global(ptr,'boxzhi')-lammps_extract_global(ptr,'boxzlo');
    box(2,2)=lammps_extract_global(ptr,'boxyhi')-lammps_extract_global(ptr,'boxylo');
    box(1,1)=lammps_extract_global(ptr,'boxxhi')-lammps_extract_global(ptr,'boxxlo');
    maxlp=box(1,1)*dmax;
    minlp=box(1,1)*dmin;
    step = (maxlp-minlp)/(steps-1)/minlp;
    lammps_command(ptr,'unfix 0');
    lammps_command(ptr,'variable tmp equal xy');
    lammps_command(ptr,'variable xy1 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal xz');
    lammps_command(ptr,'variable xz1 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal yz');
    lammps_command(ptr,'variable yz1 equal ${tmp}');
    box(2,1)=lammps_extract_variable(ptr,'xy1','null',0);
    box(3,1)=lammps_extract_variable(ptr,'xz1','null',0);
    box(3,2)=lammps_extract_variable(ptr,'yz1','null',0);
    xdel = step*minlp;
    xydel = step*box(2,1)*dmin;
    xzdel = step*box(3,1)*dmin;
    ydel = step*box(2,2)*dmin;
    yzdel = step*box(3,2)*dmin;
    zdel = step*box(3,3)*dmin;
    lammps_command(ptr,['fix 0 all deform 1 x scale ' num2str(dmin) ' y scale ' num2str(dmin) ' z scale ' num2str(dmin) ' xy final ' num2str(box(2,1)*dmin,10) ' xz final ' num2str(box(3,1)*dmin,10) ' yz final ' num2str(box(3,2)*dmin,10) ' units box']);
    lammps_command(ptr,'run 1');
    E0 = lammps_extract_variable(ptr,'energy','null',0)/lammps_natoms(ptr);
    lammps_command(ptr,'unfix 0');
    lammps_command(ptr,['fix 1 all deform 1 x delta 0  ' num2str(xdel,10) ' y delta 0  ' num2str(ydel,10) ' z delta 0 ' num2str(zdel,10) ' xy delta ' num2str(xydel,10) ' xz delta ' num2str(xzdel,10) ' yz delta ' num2str(yzdel,10) ' units box']);
    E = zeros(steps,1);
    for i=1:steps
        lammps_command(ptr,'run 1');
        E(i) = lammps_extract_variable(ptr,'energy','null',0);
    end
    E=E/lammps_natoms(ptr);
    lammps_command(ptr,'print STOPPED');
    a = minlp+xdel*((1:steps)-1);
end


function CC=lammps_Elastic(ptr,lattice,meam_lib,meam_param,elementlist,type,Options)
    Elements = regexp(elementlist,' ','split');
    Cs=getElastic_Constants(lattice);
    if ~isempty(Options.CustomElasticConstants)
        Cs=Options.CustomElasticConstants;
    end
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Elastic_' lattice '' typestr '.atoms.data'];
    do = regexp(Cs, ',','split');
    C = zeros(6,6);
    for i=1:length(do)
        eval([do{i}(1) '(' do{i}(2) ',' do{i}(3) ')=1;']);
        eval([do{i}(1) '(' do{i}(3) ',' do{i}(2) ')=1;']);
    end
    lammps_Initiate(ptr,['' meam_lib],['' meam_param],elementlist,data_file);
    lammps_command(ptr,'thermo		1');
    lammps_command(ptr,'variable cfac equal 1.0e-4');
    lammps_command(ptr,'variable up equal 1.0e-6');
    lammps_command(ptr,'thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol');
    lammps_command(ptr,'thermo_modify norm no');
    lammps_command(ptr,'fix 3 all box/relax aniso 0.0');
    lammps_command(ptr,'minimize 1e-8 1e-9 5000 10000');

    lammps_command(ptr,'variable tmp equal pxx');
    lammps_command(ptr,'variable pxx0 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal pyy');
    lammps_command(ptr,'variable pyy0 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal pzz');
    lammps_command(ptr,'variable pzz0 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal pyz');
    lammps_command(ptr,'variable pyz0 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal pxz');
    lammps_command(ptr,'variable pxz0 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal pxy');
    lammps_command(ptr,'variable pxy0 equal ${tmp}');

    lammps_command(ptr,'variable tmp equal lx');
    lammps_command(ptr,'variable lx0 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal ly');
    lammps_command(ptr,'variable ly0 equal ${tmp}');
    lammps_command(ptr,'variable tmp equal lz');
    lammps_command(ptr,'variable lz0 equal ${tmp}');

    lammps_command(ptr,'variable d1 equal -(v_pxx1-${pxx0})/(v_delta/v_len0)*${cfac}');
    lammps_command(ptr,'variable d2 equal -(v_pyy1-${pyy0})/(v_delta/v_len0)*${cfac}');
    lammps_command(ptr,'variable d3 equal -(v_pzz1-${pzz0})/(v_delta/v_len0)*${cfac}');
    lammps_command(ptr,'variable d4 equal -(v_pyz1-${pyz0})/(v_delta/v_len0)*${cfac}');
    lammps_command(ptr,'variable d5 equal -(v_pxz1-${pxz0})/(v_delta/v_len0)*${cfac}');
    lammps_command(ptr,'variable d6 equal -(v_pxy1-${pxy0})/(v_delta/v_len0)*${cfac}');

    lammps_command(ptr,'unfix 3');
    lammps_command(ptr,'write_restart restart.equil');
    dir = {'x','y','z','yz','xz','xy'};
    shear = {'0','0','0','','',''};
    D=C;
    for i=1:6
        if ~C(i,i) && any(C(i,:))
            for j=1:6
                if C(i,j) && ~C(j,j)
                        D(j,j)=1;
                end
            end
        end
    end
    for i=1:6
        if D(i,i)
            lammps_command(ptr,['variable dir equal ' num2str(i) '']);
            lammps_command(ptr,['variable len0 equal ${l' dir{i}(end) '0}']);
%            lammps_command(ptr,'clear');
            %lammps_command(ptr,'box tilt large');
%            lammps_command(ptr,'read_restart restart.equil');
%            lammps_command(ptr,'pair_style meam');
%            lammps_command(ptr,['pair_coeff * * ' meam_lib ' ' elementlist ' ' meam_param ' ' elementlist '']);
            lammps_Initiate(ptr,['' meam_lib],['' meam_param],elementlist,data_file);

            lammps_command(ptr,'variable delta equal -${up}*${len0}');
            lammps_command(ptr,['change_box all ' dir{i} ' delta ' shear{i} ' ${delta} units box']);
            lammps_command(ptr,'thermo		1');
            lammps_command(ptr,'thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol');
            lammps_command(ptr,'thermo_modify norm no');
            lammps_command(ptr,'minimize 1e-8 1e-9 5000 10000');
            lammps_command(ptr,'variable tmp equal pxx');
            lammps_command(ptr,'variable pxx1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pyy');
            lammps_command(ptr,'variable pyy1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pzz');
            lammps_command(ptr,'variable pzz1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pxy');
            lammps_command(ptr,'variable pxy1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pxz');
            lammps_command(ptr,'variable pxz1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pyz');
            lammps_command(ptr,'variable pyz1 equal ${tmp}');

            lammps_command(ptr,'variable C1neg equal ${d1}');
            lammps_command(ptr,'variable C2neg equal ${d2}');
            lammps_command(ptr,'variable C3neg equal ${d3}');
            lammps_command(ptr,'variable C4neg equal ${d4}');
            lammps_command(ptr,'variable C5neg equal ${d5}');
            lammps_command(ptr,'variable C6neg equal ${d6}');

%            lammps_command(ptr,'clear');
            %lammps_command(ptr,'box tilt large');
%            lammps_command(ptr,'read_restart restart.equil');
%            lammps_command(ptr,'pair_style meam');
%            lammps_command(ptr,['pair_coeff * * ' meam_lib ' ' elementlist ' ' meam_param ' ' elementlist ]);
            lammps_Initiate(ptr,['' meam_lib],['' meam_param],elementlist,data_file);

            lammps_command(ptr,'variable delta equal ${up}*${len0}');
            lammps_command(ptr,['change_box all ' dir{i} ' delta ' shear{i} ' ${delta} units box']);
            lammps_command(ptr,'thermo		1');
            lammps_command(ptr,'thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol');
            lammps_command(ptr,'thermo_modify norm no');
            lammps_command(ptr,'minimize 1e-8 1e-9 5000 10000');
            lammps_command(ptr,'variable tmp equal pe');
            lammps_command(ptr,'variable e1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal press');
            lammps_command(ptr,'variable p1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pxx');
            lammps_command(ptr,'variable pxx1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pyy');
            lammps_command(ptr,'variable pyy1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pzz');
            lammps_command(ptr,'variable pzz1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pxy');
            lammps_command(ptr,'variable pxy1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pxz');
            lammps_command(ptr,'variable pxz1 equal ${tmp}');
            lammps_command(ptr,'variable tmp equal pyz');
            lammps_command(ptr,'variable pyz1 equal ${tmp}');

            lammps_command(ptr,'variable C1pos equal ${d1}');
            lammps_command(ptr,'variable C2pos equal ${d2}');
            lammps_command(ptr,'variable C3pos equal ${d3}');
            lammps_command(ptr,'variable C4pos equal ${d4}');
            lammps_command(ptr,'variable C5pos equal ${d5}');
            lammps_command(ptr,'variable C6pos equal ${d6}');

            lammps_command(ptr,'variable C1${dir} equal 0.5*(${C1neg}+${C1pos})');
            lammps_command(ptr,'variable C2${dir} equal 0.5*(${C2neg}+${C2pos})');
            lammps_command(ptr,'variable C3${dir} equal 0.5*(${C3neg}+${C3pos})');
            lammps_command(ptr,'variable C4${dir} equal 0.5*(${C4neg}+${C4pos})');
            lammps_command(ptr,'variable C5${dir} equal 0.5*(${C5neg}+${C5pos})');
            lammps_command(ptr,'variable C6${dir} equal 0.5*(${C6neg}+${C6pos})');
            lammps_command(ptr,'variable C${dir}1 equal 0.5*(${C1neg}+${C1pos})');
            lammps_command(ptr,'variable C${dir}2 equal 0.5*(${C2neg}+${C2pos})');
            lammps_command(ptr,'variable C${dir}3 equal 0.5*(${C3neg}+${C3pos})');
            lammps_command(ptr,'variable C${dir}4 equal 0.5*(${C4neg}+${C4pos})');
            lammps_command(ptr,'variable C${dir}5 equal 0.5*(${C5neg}+${C5pos})');
            lammps_command(ptr,'variable C${dir}6 equal 0.5*(${C6neg}+${C6pos})');
            lammps_command(ptr,'variable dir delete');
        end
    end
    CC = zeros(6,6);
    for i=1:6
        for j=1:i
            if C(j,i)
                CC(j,i)=lammps_extract_variable(ptr,['C' num2str(j) num2str(i) ],'null',0);
            end
        end
    end
    lammps_command(ptr,'print "ALL DONE"');
end

function E=lammps_Vacancy(ptr,meam_lib,meam_param,element_list,type,Esub)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Vacancy' typestr '.atoms.data'];
    lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
    lammps_command(ptr,'fix 1 all box/relax aniso 0.0');
    lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
    E = lammps_extract_variable(ptr,'energy','null',0);
    lammps_command(ptr,'print STOPPED');
    E = E-lammps_natoms(ptr)*Esub;
end

function [E types]=lammps_Interstitial(ptr,lattice,meam_lib,meam_param,element_list,type,Esub,Options)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    [~, types] = getInterstitial_Offset(lattice,[1 0 0;0 1 0;0 0 1]);
    E = zeros(size(types,2),1);
    for i=1:size(types,2)
        if ~Options.Interstitialflags(i)
            continue;
        end
        data_file=['atomic_data/Interstitial_' types{i} '' typestr '.atoms.data'];
        lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
        lammps_command(ptr,'fix 1 all box/relax aniso 0.0');
        lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
        E(i) = lammps_extract_variable(ptr,'energy','null',0);
        lammps_command(ptr,'print STOPPED');
        E(i) = E(i)-lammps_natoms(ptr)*Esub;
    end
end

function [E surface]=lammps_Surface(ptr,lattice,meam_lib,meam_param,element_list,type,Esub,Options)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    [xplanes yplanes] = getSurface_Planes(lattice);
    E=zeros(size(xplanes,1),1);
    for i=1:size(xplanes,1)
        if ~Options.Surfaceflags(i)
            continue;
        end
        xplane=xplanes(i,:);
        yplane=yplanes(i,:);
        surf = ['[' num2str(xplane(1)) '_' num2str(xplane(2)) '_' num2str(xplane(3)) ']x[' num2str(yplane(1)) '_' num2str(yplane(2)) '_' num2str(yplane(3)) ']'];
        data_file=['atomic_data/Surface_' surf '' typestr '.atoms.data'];
        lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
        lammps_command(ptr,'fix 1 all box/relax aniso 0.0');
        lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
        lammps_command(ptr,'variable area equal lx*ly');
        E(i)=(lammps_extract_variable(ptr,'energy','null',0)-lammps_natoms(ptr)*Esub)/lammps_extract_variable(ptr,'area','null',0)/2*16021.77;
        surface{i}=surf;
    end
end

function [E shift planes]=lammps_GSFE(ptr,lattice,steps,meam_lib,meam_param,element_list,type,Esub,Options)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    [xplanes yplanes] = getGSFE_Plane_And_Direction(lattice);
    E=zeros(size(xplanes,1),steps);
    shift=zeros(size(xplanes,1),steps);
    planes = cell(1,size(xplanes,1));
    for i=1:size(xplanes,1)
        if ~Options.GSFEflags(i)
            continue;
        end
        xplane=xplanes(i,:);
        yplane=yplanes(i,:);
        %plane = num2str(abs(integerconvertv(cross(xplane,yplane),.0001)));
        %plane=strrep(plane,'  ','');
        direction=[num2str(xplane(1)) num2str(xplane(2)) num2str(xplane(3))];
        direction2=[num2str(yplane(1)) num2str(yplane(2)) num2str(yplane(3))];
        folder=['[' direction ']x[' direction2 ']'];
        planes{i} = folder;
        data_file=['atomic_data/GSFE_' folder '' typestr '.atoms.data'];
        lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
        lammps_command(ptr,'variable zhalf equal (zhi+zlo)/2');
        lammps_command(ptr,'variable z34 equal (3*zhi+zlo)/4');
        lammps_command(ptr,'variable z14 equal (zhi+3*zlo)/4');
        lammps_command(ptr,'variable area equal lx*ly');
        lammps_command(ptr,'region upper block INF INF INF INF ${zhalf} INF units box');
        lammps_command(ptr,'group upper region upper');
        lammps_command(ptr,'region middle block INF INF INF INF ${z14} ${z34} units box');
        lammps_command(ptr,'group middle region middle');
        lammps_command(ptr,'fix 2 all box/relax aniso 0.0');
        lammps_command(ptr,'compute eng middle pe/atom');
        lammps_command(ptr,'compute drat middle reduce sum c_eng');
        lammps_command(ptr,'thermo 1');
        lammps_command(ptr,'thermo_style custom etotal c_drat');
        lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
        step = lammps_extract_global(ptr,'boxxhi')/(steps-1);
        if Options.GSFErelax
            lammps_command(ptr,'fix 3 all setforce 0.0 0.0 NULL');
        end
        for j=1:steps
            lammps_command(ptr,'run 1');
            lammps_command(ptr,['displace_atoms upper move ' num2str(step) ' 0.0 0.0 units box']);
            if Options.GSFErelax
                lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
            end
            Ev = lammps_extract_compute(ptr,'eng',1,1);
            E(i,j)=(sum(Ev)-sum(abs(Ev)>0.0000001)*Esub)/lammps_extract_variable(ptr,'area','null',0)*16021.77;
            shift(i,j)=step*(j-1);
        end
    end
end

function SubstitutionEnergy=lammps_Substitution(ptr,meam_lib,meam_param,element_list,type,Esubs)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Substitution' typestr '.atoms.data'];
    lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
    lammps_command(ptr,'fix 1 all box/relax aniso 0.0');
    lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
    E = lammps_extract_variable(ptr,'energy','null',0);
    lammps_command(ptr,'print STOPPED');
    SubstitutionEnergy = E-(lammps_natoms(ptr)-1)*Esubs(1)-Esubs(2);
end

function InterstititalSubstitutionEnergy=lammps_InterstitialSubstitution(ptr,meam_lib,meam_param,element_list,type,Esubs)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Interstitial_Substitution' typestr '.atoms.data'];
    lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
    lammps_command(ptr,'fix 1 all box/relax aniso 0.0');
    lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
    E = lammps_extract_variable(ptr,'energy','null',0);
    lammps_command(ptr,'print STOPPED');
    InterstititalSubstitutionEnergy = E-(lammps_natoms(ptr)-1)*Esubs(1)-Esubs(2);
end

function BindingEnergy=lammps_BindingEnergy(ptr,meam_lib,meam_param,element_list,type,Esubs,Substitution)
    Elements = regexp(element_list,' ','split');
    typestr = '';
    for i=1:size(type,2)
        typestr=[typestr '_' Elements{type(i)}];
    end
    data_file=['atomic_data/Binding_Energy' typestr '.atoms.data'];
    lammps_Initiate(ptr,['' meam_lib],['' meam_param],element_list,data_file);
    lammps_command(ptr,'fix 1 all box/relax aniso 0.0');
    lammps_command(ptr,'minimize 1.0e-8 1.0e-8 10000 10000');
    E = lammps_extract_variable(ptr,'energy','null',0);
    lammps_command(ptr,'print STOPPED');
    BindingEnergy = E-(lammps_natoms(ptr)-2)*Esubs(1)-Esubs(2)-Esubs(3)-Substitution(1)-Substitution(2);
end
% Functions that depend on the lattice
% If you add a new lattice, add support for it here.
function Cs = getElastic_Constants(lattice)
    switch lattice
        %isotropic: all cubics
        case {'FCC','BCC','DC','SC','B1','B2','B3','C1','C2','C3','L12','D03'}
            Cs='C11,C12,C44';
        case 'HCP'
            Cs='C11,C12,C13,C33,C44';
        %the following three are for reference
        case 'Anisotropic'
            Cs='C11,C12,C13,C14,C15,C16,C22,C23,C24,C25,C26,C33,C34,C35,C36,C44,C45,C46,C55,C56,C66';
        case 'Orthotropic'
            Cs='C11,C12,C13,C22,C23,C33,C44,C55,C66';
        case 'TransverseIsotropic'
            Cs='C11,C12,C13,C33,C44';
        otherwise
        %Default to isotropic
            Cs='C11,C12,C44';
    end
end

function [offset types] = getInterstitial_Offset(lattice,unitcell)
    %select types:
    switch lattice
        case 'FCC'
            types = {'Tetrahedral','Octahedral','[100]dumbbell'};
        case 'BCC'
            types = {'Tetrahedral','Octahedral','[100]dumbbell','[110]dumbbell','[111]dumbbell'};
        case 'HCP'
            types = {'Tetrahedral','Octahedral','[0001]dumbbell'};
        case 'DC'
            types = {'[110]dumbbell'};
        otherwise 
            types = {'Tetrahedral'};
    end
    %get interstitial positions
    offset = zeros(size(types,2),3);
    for i=1:size(types,2)
        type = types{i};
        switch type
            case 'Tetrahedral'
                if strcmp(lattice,'FCC')
                    offset(i,:) = (unitcell'*[0.25 0.5 0.5]')';
                elseif strcmp(lattice,'BCC')
                    offset(i,:) = (unitcell'*[0.5 0 0.25]')';
                elseif strcmp(lattice,'HCP')
                    offset(i,:) = (unitcell'*[0 0 2/3]')';
                elseif strcmp(lattice,'DC')
                    offset(i,:) = (unitcell'*[-1/2 1 1]')';
                end
            case 'Octahedral'
                if strcmp(lattice,'FCC')
                    offset(i,:) = (unitcell'*[0.5 1 1]')';
                elseif strcmp(lattice,'BCC')
                    offset(i,:) = (unitcell'*[0.5 0 0.5]')';
                elseif strcmp(lattice,'HCP')
                    offset(i,:) = (unitcell'*[1/3 1/3 3/4]')';
                elseif strcmp(lattice,'SC')
                    offset = (unitcell'*[0.5 0.5 0.5]')';
                end
            case '[100]dumbbell'
                offset(i,:) = 2*unitcell(1,:)/3;
            case '[110]dumbbell'
                offset(i,:) = 2*(unitcell(1,:)+unitcell(2,:))/3;
            case '[111]dumbbell'
                offset(i,:) = 2*(unitcell(1,:)+unitcell(2,:)+unitcell(3,:))/3;
            case '[0001]dumbbell'
                offset(i,:) = 2*unitcell(3,:)/3;
            otherwise
                error('Unsupported interstitial type');
        end
    end
end

function [xplanes yplanes] = getSurface_Planes(lattice)
% xplanes and yplanes are in unitcell coordinates, not cartesian.
    switch lattice
        case 'FCC'
            %(111),(110),(100)
            xplanes = [0 0 1;1 0 2;1 0 2];
            yplanes = [0 1 0;0 1 0;1 0 0];
        case 'BCC'
            %(100),(110),(111)
            xplanes = [1 0 0;1 0 0;1 1 0];
            yplanes = [0 1 0;0.5 0.5 0.5;0 1 1];
        case 'DC'
            xplanes = [1 0 0;0 0 1;0 1 0;];
            yplanes = [0 1 0;-1 2 0;0 0 1];
        case 'HCP'
            xplanes = [1 0 0;1 0 0;1 0 0;0 0 1;1 0 1];
            yplanes = [-1 2 0;0 0 1;-1 2 1;-1 2 0;-1 2 0];
        otherwise
            xplanes = [0 1 0];
            yplanes = [0 0 1];
    end
end

function [xdirs yplanes] = getGSFE_Plane_And_Direction(lattice)
% xplanes and yplanes are in unitcell coordinates, not cartesian.
    switch lattice
        case 'FCC'
            %[110](111), [121](111)
            xdirs = [0 0 1;0 2 1];
            yplanes = [0 1 0;0 0 -1];
        case 'BCC'
            %[111](110)
            xdirs = [0.5 0.5 0.5];
            yplanes = [1 0 0];
        case 'DC'
            xdirs = [0 0 1];
            yplanes = [0 1 0];
        case 'HCP'
            %basal a, prismatic a, pyramidal a, 2nd order pyramidal c+a, basal
            %Shockley, 1st order pyramidal c+a
            xdirs = [1 0 0;1 0 0;1 0 0;-1 0 1;1 1 0;0 1 1]; 
            yplanes = [-1 2 0;0 0 1;-1 2 2;1 -2 0;-1 0 0;-1 0 0];
        otherwise
            xdirs = [0 1 0];
            yplanes = [0 0 1];
    end
end

function Conversion = getLatticeConversion(lattice)
    %add more lattices later
    if strcmp(lattice,'FCC')
        Conversion = 4/sqrt(2);
    elseif strcmp(lattice,'BCC')
        Conversion = 4/sqrt(3);
    elseif strcmp(lattice,'DC')
        Conversion = 8/sqrt(3);
    elseif strcmp(lattice,'HCP')
        Conversion = 2;
    elseif strcmp(lattice,'SC')
        Conversion = 2;
    else
        Conversion = computeConversion(lattice);
    end
end

function [unitcell type offset] = generateAlloyUnitCell(lattice)
switch lattice
    case {'FCC','A1'}
        unitcell = [1 0 0;0.5 0.5 0;-0.5 0 0.5];
        type = 1;
        offset = [0 0 0];
    case {'BCC','A2'}
        unitcell = [1 0 0;0 1 0;0 0 1];
        type = [1 1];
        offset = [0 0 0;0.5 0.5 0.5];
    case {'HCP','A3'}
        %ideal c/a ratio
        unitcell = [1 0 0;0.5 sqrt(3)/2 0;0 0 sqrt(8/3)];
        type = [1 1];
        offset = [0 0 0;1 1/sqrt(3) sqrt(2/3)];
    case {'SC'}
        unitcell = [1 0 0;0 1 0;0 0 1];
        type = 1;
        offset = [0 0 0];
    case {'DC','A4'}
        unitcell = [1 0 0;0.5 0.5 0;0.5 0 0.5];
        type = 2;
        offset = [0 0 0;1.25 0.25 0.25];
    case 'B1'
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  2  2  2  2];
		offset = [				0 0       0.5;
				0       0.5 0;
				0.5 0 0;
				0.5       0.5       0.5;
				0 0 0;
				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0];
    case 'B2'
        % equivalent to single element BCC
        unitcell = [1 0 0;0 1 0;0 0 1];
        type = [1 2];
        offset = [0 0 0;0.5 0.5 0.5];
    case 'B3'
        % equivalent to single element DC
        unitcell = [1 0 0;0.5 0.5 0;0.5 0 0.5];
        type = [1 2];
        offset = [0 0 0;1.25 0.25 0.25];
    case 'B4'
		unitcell = [1 0 0;-0.5     0.8660254038 0;0 0      8.158995816];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2];
		offset = [				-5e-06     0.5773531559      1.427824268;
				-5e-06     0.5773531559      3.875523013;
				-4.999999999e-06     0.5773531559      6.323221757;
				0 0     0.6119246862;
				0 0      3.059623431;
				0     0.5773444957      1.427824268;
				0 0      4.691422594;
				0     0.5773444957      3.875523013;
				0 0      7.139121339;
				0     0.5773444957      6.323221757;
				5e-06     0.5773531559      1.427824268;
				5e-06     0.5773531559      3.875523013;
				5.000000001e-06     0.5773531559      6.323221757;
				0.499995     0.2886722478      2.243723849;
				0.499995     0.2886722478      5.507322176;
				0.499995     0.2886722478      7.955020921;
				0.5     0.2886809081      2.243723849;
				0.5     0.2886809081      5.507322176;
				0.5     0.2886809081      7.955020921;
				0.500005     0.2886722478      2.243723849;
				0.500005     0.2886722478      5.507322176;
				0.500005     0.2886722478      7.955020921;
				-5e-06     0.5773531559     0.8158995816;
				-5e-06     0.5773531559      3.263598326;
				-4.999999999e-06     0.5773531559      5.711297071;
				0 0 0;
				0 0      2.447698745;
				0     0.5773444957     0.8158995816;
				0 0      4.079497908;
				0     0.5773444957      3.263598326;
				0 0      6.527196653;
				0     0.5773444957      5.711297071;
				5e-06     0.5773531559     0.8158995816;
				5e-06     0.5773531559      3.263598326;
				5.000000001e-06     0.5773531559      5.711297071;
				0.499995     0.2886722478      1.631799163;
				0.499995     0.2886722478       4.89539749;
				0.499995     0.2886722478      7.343096234;
				0.5     0.2886809081      1.631799163;
				0.5     0.2886809081       4.89539749;
				0.5     0.2886809081      7.343096234;
				0.500005     0.2886722478      1.631799163;
				0.500005     0.2886722478       4.89539749;
				0.500005     0.2886722478      7.343096234];  
    case 'Bf'
		unitcell = [1 0 0;0      2.646682385 0;0 0     0.9875378915];
		type = [1  1  1  1  2  2  2  2];
		offset = [				0      1.151306837     0.2468844729;
				0      1.495375547     0.7406534187;
				0.5      0.172034355     0.7406534187;
				0.5       2.47464803     0.2468844729;
				0     0.3824456046     0.2468844729;
				0       2.26423678     0.7406534187;
				0.5      1.705786797     0.2468844729;
				0.5     0.9408955877     0.7406534187];     
    case 'C1'
        unitcell = [1 0 0;0 1 0;0 0 1];
        type = [1 1 1 1 2 2 2 2 2 2 2 2];
        offset = [0 0 0;0.5 0 0;0 0.5 0;0 0 0.5;0.25 0.25 0.25;0.25 0.25 0.75;0.25 0.75 0.25;0.25 0.75 0.75;0.75 0.25 0.25;0.75 0.25 0.75; 0.75 0.75 0.25; 0.75 0.75 0.75];
    case 'C2'
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  2  2  2  2  2  2  2  2];
		offset = [				0 0 0;
				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0;
				0.162     0.838     0.338;
				0.162     0.662     0.838;
				0.338     0.162     0.838;
				0.338     0.338     0.338;
				0.662     0.662     0.662;
				0.662     0.838     0.162;
				0.838     0.162     0.662;
				0.838     0.338     0.162];
    case 'C3'
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  2  2];
		offset = [				0.25      0.25      0.25;
				0.25      0.75      0.75;
				0.75      0.25      0.75;
				0.75      0.75      0.25;
				0 0 0;
				0.5       0.5       0.5];
    case 'C4'
		unitcell = [1 0 0;0 1 0;0 0      0.644005052];
		type = [1  1  1  1  2  2];
		offset = [				0.19504   0.80496      0.322002526;
				0.30496   0.30496 0;
				0.69504   0.69504 0;
				0.80496   0.19504      0.322002526;
				0 0 0;
				0.5       0.5      0.322002526];        
    case 'C8'
		unitcell = [1 0 0;-0.5     0.8660254038 0;0 0      1.099552482];
		type = [1  1  1  1  1  1  2  2  2];
		offset = [				-0.21995     0.6348832235     0.9685957811;
				0.06015     0.3581015045     0.6020782872;
				0.1598     0.7390660796     0.2355607933;
				0.28005     0.2311421803     0.1309567006;
				0.56015     0.5079238993     0.4974741945;
				0.6598     0.1269593242     0.8639916884;
				-0.23485     0.4067721322     0.7330349878;
				0.26515     0.4592532716     0.3665174939;
				0.4697 0 0];        
    case 'C9'
		unitcell = [1 0 0;0 1 0;0 0      1.392340648];
		type = [1  1  1  1  1  1  1  1  2  2  2  2];
		offset = [				0.1044    0.2392      1.143529374;
				0.2392    0.1044     0.2488112738;
				0.2608    0.6044    0.09927388821;
				0.3956    0.7392     0.5968964358;
				0.6044    0.2608       1.29306676;
				0.7392    0.3956     0.7954442122;
				0.7608    0.8956     0.9449815978;
				0.8956    0.7608     0.4473590502;
				0.19972   0.80028      0.348085162;
				0.30028   0.30028 0;
				0.69972   0.69972      0.696170324;
				0.80028   0.19972      1.044255486];        
    case 'C11b'
		unitcell = [1 0 0;0 1 0;0 0 2.4565625];
		type = [1  1  2  2  2  2];
		offset = [				0 0 0;
				0.5       0.5       1.22828125;
				0 0     0.8180353125;
				0 0      1.638527188;
				0.5       0.5     0.4102459375;
				0.5       0.5      2.046316562];    
    case 'C15'
        
    case 'C16'
		unitcell = [1 0 0;0 1 0;0 0     0.8038569309];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2];
		offset = [				0.1581    0.3419     0.4019284655;
				0.1581    0.6581 0;
				0.1581    0.3419     0.4019284655;
				0.1581    0.6581 0;
				0.3419    0.1581 0;
				0.3419    0.1581 0;
				0.3419    0.8419     0.4019284655;
				0.6581    0.1581     0.4019284655;
				0.6581    0.1581     0.4019284655;
				0.6581    0.8419 0;
				0.8419    0.3419 0;
				0.8419    0.6581     0.4019284655;
				0 0     0.2009642327;
				0 0     0.6028926982;
				0.5       0.5     0.2009642327;
				0.5       0.5     0.6028926982];        
    case 'C21'
		unitcell = [1 0 0;0     0.5939611947 0;0 0     0.5600610421];
		type = [1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  3  3  3  3  3  3  3  3];
		offset = [				0.0095    0.08855961413     0.1027712012;
				0.0095     0.2084209832     0.3828017223;
				0.4905     0.3855402115     0.1027712012;
				0.4905     0.5054015806     0.3828017223;
				0.5095    0.08855961413     0.1772593198;
				0.5095     0.2084209832     0.4572898409;
				0.9905     0.3855402115     0.1772593198;
				0.9905     0.5054015806     0.4572898409;
				0.2314    0.06592969261     0.3005287552;
				0.2314     0.2310509047    0.02049823414;
				0.2686     0.3629102899     0.3005287552;
				0.2686     0.5280315021    0.02049823414;
				0.7314    0.06592969261     0.5395628079;
				0.7314     0.2310509047     0.2595322869;
				0.7686     0.3629102899     0.5395628079;
				0.7686     0.5280315021     0.2595322869;
				0.1289    0.05773302812     0.4832206671;
				0.1289     0.2392475692     0.2031901461;
				0.3711     0.3547136255     0.4832206671;
				0.3711     0.5362281666     0.2031901461;
				0.6289    0.05773302812      0.356870896;
				0.6289     0.2392475692    0.07684037497;
				0.8711     0.3547136255      0.356870896;
				0.8711     0.5362281666    0.07684037497];        
    case 'L12'
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  2];
		offset = [				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0;
				0 0 0];     
    case 'L10'
		unitcell = [1 0 0;0 1 0;0 0       1.36588948];
		type = [1  2];
		offset = [				0 0 0;
				0.5       0.5     0.6829447402];
    case 'D03'
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2];
		offset = [				0 0       0.5;
				0       0.5 0;
				0.25      0.25      0.25;
				0.25      0.25      0.75;
				0.25      0.75      0.25;
				0.25      0.75      0.75;
				0.5 0 0;
				0.5       0.5       0.5;
				0.75      0.25      0.25;
				0.75      0.25      0.75;
				0.75      0.75      0.25;
				0.75      0.75      0.75;
				0 0 0;
				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0];    
    case 'D024'
		unitcell = [1 0 0;-0.5     0.8660254038 0;0 0      1.629513344];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2];
		offset = [				-0.250035     0.7216849497      1.222135008;
				-0.250015     0.7217195908      1.222135008;
				-0.25     0.4330127019 0;
				-0.25     0.4330127019     0.8147566719;
				-2e-05     0.2886462671      1.222135008;
				2e-05     0.2886462671      1.222135008;
				0.249965      0.144340454     0.4073783359;
				0.249985      0.144305813     0.4073783359;
				0.25     0.4330127019 0;
				0.25     0.4330127019     0.8147566719;
				0.250015     0.7217195908      1.222135008;
				0.250035     0.7216849497      1.222135008;
				0.49998     0.5773791367     0.4073783359;
				0.5 0 0;
				0.5 0     0.8147566719;
				0.50002     0.5773791367     0.4073783359;
				0.750015      0.144305813     0.4073783359;
				0.750035      0.144340454     0.4073783359;
				-5e-06     0.5773531559     0.4073783359;
				0 0 0;
				0 0     0.8147566719;
				0     0.5773444957     0.4073783359;
				5e-06     0.5773531559     0.4073783359;
				0.499995     0.2886722478      1.222135008;
				0.5     0.2886809081      1.222135008;
				0.500005     0.2886722478      1.222135008];       
    case 'D8b'
		unitcell = [1 0 0;0 1 0;0 0     0.5163636364];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2];
		offset = [				0 0 0;
				0.0368    0.6316     0.2581818182;
				0.0653    0.7376 0;
				0.1019    0.8981     0.2581818182;
				0.1316    0.4632 0;
				0.1823    0.1823     0.1303301818;
				0.1823    0.1823     0.3860334545;
				0.2376    0.4347     0.2581818182;
				0.2624    0.9347 0;
				0.3177    0.6823     0.1278516364;
				0.3177    0.6823  0.388512;
				0.3684    0.9632     0.2581818182;
				0.3981    0.3981 0;
				0.4347    0.2376     0.2581818182;
				0.4632    0.1316 0;
				0.5       0.5     0.2581818182;
				0.5368    0.8684 0;
				0.5653    0.7624     0.2581818182;
				0.6019    0.6019 0;
				0.6316    0.0368     0.2581818182;
				0.6823    0.3177     0.1278516364;
				0.6823    0.3177  0.388512;
				0.7376    0.0653 0;
				0.7624    0.5653     0.2581818182;
				0.8177    0.8177     0.1303301818;
				0.8177    0.8177     0.3860334545;
				0.8684    0.5368 0;
				0.8981    0.1019     0.2581818182;
				0.9347    0.2624 0;
				0.9632    0.3684     0.2581818182;
				0 0 0;
				0.0368    0.6316     0.2581818182;
				0.0653    0.7376 0;
				0.1019    0.8981     0.2581818182;
				0.1316    0.4632 0;
				0.1823    0.1823     0.1303301818;
				0.1823    0.1823     0.3860334545;
				0.2376    0.4347     0.2581818182;
				0.2624    0.9347 0;
				0.3177    0.6823     0.1278516364;
				0.3177    0.6823  0.388512;
				0.3684    0.9632     0.2581818182;
				0.3981    0.3981 0;
				0.4347    0.2376     0.2581818182;
				0.4632    0.1316 0;
				0.5       0.5     0.2581818182;
				0.5368    0.8684 0;
				0.5653    0.7624     0.2581818182;
				0.6019    0.6019 0;
				0.6316    0.0368     0.2581818182;
				0.6823    0.3177     0.1278516364;
				0.6823    0.3177  0.388512;
				0.7376    0.0653 0;
				0.7624    0.5653     0.2581818182;
				0.8177    0.8177     0.1303301818;
				0.8177    0.8177     0.3860334545;
				0.8684    0.5368 0;
				0.8981    0.1019     0.2581818182;
				0.9347    0.2624 0;
				0.9632    0.3684     0.2581818182];             
    otherwise
        error('Unsupported lattice.');
end
end

% Miscellaneous functions
function Options=initiateOptions(ntypes)
    for j=1:ntypes
        Options.SERunflags(j)=255;
        Options.SESetupflags(j)=255;
        for k=1:ntypes
            Options.BinaryRunflags(j,k)=15;
            Options.BinarySetupflags(j,k)=15;
            for l=1:ntypes
                Options.TernaryRunflags(j,k,l)=1;
                Options.TernarySetupflags(j,k,l)=1;
            end
            Options.BLEVflags{j,k}='';
            Options.BLElasticflags{j,k}='';
            Options.BEVmin(j,k)=0.7;
            Options.BEVmax(j,k)=1.25;
            Options.BElasticConstants{j,k}='';
        end
        Options.EVmin(j)=0.7;
        Options.EVmax(j)=1.25;
        Options.ElasticConstants{j}='';
        Options.Interstitialflags(j)=31;
        Options.Surfaceflags(j)=31;
        Options.GSFEflags(j)=63;
        Options.VacancySize(j)=8;
        Options.InterstitialSize(j)=8;
        Options.SurfaceDistance(j)=10;
        Options.GSFEDistance(j)=10;
        Options.GSFErelax(j)=0;
    end
end
function Options=processOptions(Options,cell)
    i=1;
    while i<size(cell,2)
        switch cell{i}
            case {'SingleElementRunflags'}
                Options.SERunflags=cell{i+1};
                i=i+2;
            case {'SingleElementSetupflags'}
                Options.SESetupflags=cell{i+1};
                i=i+2;
            case {'BinaryRunflags'}
                Options.BinaryRunflags=cell{i+1};
                i=i+2;
            case {'BinarySetupflags'}
                Options.BinarySetupflags=cell{i+1};
                i=i+2;
            case {'TernaryRunflags'}
                Options.TernaryRunflags=cell{i+1};
                i=i+2;
            case {'TernarySetupflags'}
                Options.TernarySetupflags=cell{i+1};
                i=i+2;                
            case {'Interstitialflags'}
                Options.Interstitialflag(cell{i+1})=cell{i+2};
                i=i+3;
            case {'Surfaceflags'}
                Options.Surfaceflag(cell{i+1})=cell{i+2};
                i=i+3;
            case {'GSFEflags'}
                Options.GSFEflag(cell{i+1})=cell{i+2};
                i=i+3;
            case {'BinaryLatticeEVflags'}
                Options.BLEVflags{cell{i+1},cell{i+2}}=cell{i+3};
                i=i+4;
            case {'BinaryLatticeElasticflags'}
                Options.BLElasticflags{cell{i+1},cell{i+2}}=cell{i+3};
                i=i+4;
            case {'CustomEVrange'}
                Options.EVmin(cell{i+1})=cell{i+2};
                Options.EVmax(cell{i+1})=cell{i+3};
                i=i+4;
            case {'CustomElasticConstants'}
                Options.ElasticConstants{cell{i+1}}=cell{i+2};
                i=i+3;
            case {'CustomBEVrange'}
                Options.BEVmin(cell{i+1},cell{i+2})=cell{i+3};
                Options.BEVmax(cell{i+1},cell{i+2})=cell{i+4};
                i=i+5;
            case {'CustomBElasticConstants'}
                Options.BElasticConstants{cell{i+1},cell{i+2}}=cell{i+3};
                i=i+4;
            case {'CustomVacancySize'}
                Options.VacancySize(cell{i+1})=cell{i+2};
                i=i+3;
            case {'CustomInterstitialSize'}
                Options.InterstitialSize(cell{i+1})=cell{i+2};
                i=i+3;
            case {'CustomSurfaceDistance'}
                Options.SurfaceDistance(cell{i+1})=cell{i+2};
                i=i+3;
            case {'CustomGSFEDistance'}
                Options.GSFEDistance(cell{i+1})=cell{i+2};
                i=i+3;
            case {'GSFErelax'}
                Options.GSFErelax(cell{i+1})=cell{i+2};
                i=i+3;
            otherwise
                error('Could not understand custom argument');
        end
    end
end

function lammps_Initiate(ptr,meam_lib,meam_param,elementlist,data_file)
    lammps_command(ptr,'clear');
    lammps_command(ptr,'units metal');
    lammps_command(ptr,'boundary p p p');
    lammps_command(ptr,'atom_style atomic');
    lammps_command(ptr,'box tilt large');
    lammps_command(ptr,['read_data ' data_file]);
    lammps_command(ptr,'pair_style meam');
    lammps_command(ptr,['pair_coeff * * ' meam_lib ' ' elementlist ' ' meam_param ' ' elementlist ]);
    lammps_command(ptr,'delete_atoms overlap 0.0001 all all');
    lammps_command(ptr,'neighbor 0.3 nsq');
    lammps_command(ptr,'neigh_modify delay 1');
    lammps_command(ptr,'variable energy equal etotal');
    lammps_command(ptr,'thermo 1');
    lammps_command(ptr,'thermo_style custom step etotal temp');
    lammps_command(ptr,'thermo_modify format int %5d');
    lammps_command(ptr,'thermo_modify format 2 %28.16f');
    lammps_command(ptr,'thermo_modify format float %15.4f');
end

function [unitcell type1 offset] = generateSurfaceCell(lattice,xplane,yplane,dist,box)
    [unitcell type offset] = generateAlloyUnitCell(lattice);
   if size(box)>1
        offset = (box'*(unitcell'^-1*offset'))'/box(1,1);
        unitcell = box/box(1,1);
    end
    xplane = (unitcell'*xplane')';
    yplane = (unitcell'*yplane')';
    zvec = cross(xplane,yplane)/norm(cross(xplane,yplane))*dist;
    if det([xplane; yplane; unitcell(1,:)])~=0
        zplane = unitcell(1,:);
    elseif det([xplane; yplane; unitcell(2,:)])~=0
        zplane = unitcell(2,:);
    else
        zplane = unitcell(3,:);
    end
    zplane = zplane * abs((zvec*zvec')/ (zplane*zvec'));
    zplane = (unitcell'*ceil(unitcell'^-1*zplane'))';
    dims = ceil(range([(unitcell'^-1)*xplane' (unitcell'^-1)*yplane' (unitcell'^-1)*zplane'],2));
    dims = dims+1;
    atoms = simpleMakeLattice(unitcell,size(offset,1),offset,dims);
    type1 = repmat(type,1,dims(1)*dims(2)*dims(3));
    atoms = bsxfun(@plus,atoms,(unitcell'*min([(unitcell'^-1)*xplane' (unitcell'^-1)*yplane' (unitcell'^-1)*zplane'],[],2))');
    atoms = atoms+.00000001;
    [atoms I] = clip(atoms*0.9999999999999,xplane,yplane,zplane);
    type1=type1(I);
    offset = bsxfun(@plus,atoms,unitcell(1,:)/2+unitcell(2,:)/2+unitcell(3,:)/2);
    unitcell = [xplane; yplane; zplane];
    if det(unitcell)<0
        unitcell = [xplane; yplane; -zplane];
    end
    offset = unique((unitcell'^-1*offset')','rows');
    offset = offset-floor(offset);
    offset = (unitcell'*offset')';
end

function [Conversion Cmax Cmin] = computeConversion(lattice)
    [unitcell,~,atoms] = generateAlloyUnitCell(lattice);
    dims = [3 3 3];
    atoms1 = simpleMakeLattice(unitcell,size(atoms,1),atoms,dims);
    atoms1 = bsxfun(@plus,atoms1,-unitcell(1,:)-unitcell(2,:)-unitcell(3,:));
    nndist = zeros(size(atoms,1),1);
    for i=1:size(atoms,1)
        [~,nndist(i)] = mindist2_c(atoms1,atoms(i,:));
    end
    Conversion = 2/mean(nndist);
    Cmax = 2/max(nndist);
    Cmin = 2/min(nndist);
end

function atoms = simpleMakeLattice(unitcell,p,offset,dims)
    atoms = zeros(dims(1)*dims(2)*dims(3)*p,3);
    for i=1:dims(1)
        for j=1:dims(2)
            for k=1:dims(3)
                for l=1:p
                    atoms((l-1)*dims(1)*dims(2)*dims(3)+(i-1)*dims(2)*dims(3)+(j-1)*dims(3)+k,:) = (i-1)*unitcell(1,:)+(j-1)*unitcell(2,:)+(k-1)*unitcell(3,:) + offset(l,:);
                end
            end
        end
    end
end

function [x, box, tilt, R] = triclinic2lammps(X,Box)
    % Lammps only takes very specific non-orthogonal box types. We do a
    % rotation into the required form.
    % Adapted from http://lammps.sandia.gov/doc/Section_howto.html#howto_12
    xlo = 0;
    ylo = 0;
    zlo = 0;
    xhi = norm(Box(1,:));
    xy = dot(Box(2,:),Box(1,:)/norm(Box(1,:)));
    yhi = norm(cross(Box(1,:)/norm(Box(1,:)),Box(2,:)));
    xz = dot(Box(3,:),Box(1,:)/norm(Box(1,:)));
    yz = (dot(Box(3,:),Box(2,:))-xy*xz)/yhi;
    zhi = sqrt(dot(Box(3,:),Box(3,:))-xz^2-yz^2);
    box = [xlo xhi;ylo yhi;zlo zhi];
    tilt = [xy xz yz];
    R = [xhi xy xz;0 yhi yz;0 0 zhi]*[cross(Box(2,:),Box(3,:));cross(Box(3,:),Box(1,:));cross(Box(1,:),Box(2,:))]/det([xhi xy xz;0 yhi yz;0 0 zhi]);
    x = (R*X')';
end


function tilt = adjusttilt(box,tilt,varargin)
    % lammps requires boxes not to be extremely tilted when periodic.
    % If boxes become extremely tilted during simulations, they are
    % automatically remapped. Free surface dimensions are not affected by this.
    % Boxes defined via Read_data must already satisfy the tilt conditions
    % which this function ensures.
    % http://lammps.sandia.gov/doc/Section_howto.html#howto_12
    if isempty(varargin)
        periodicity = [1 1 1];
    elseif length(varargin)==1
        periodicity = varargin{1};
    elseif length(varargin)==3
        periodicity(1)=varargin{1};
        periodicity(2)=varargin{2};
        periodicity(3)=varargin{3};
    else
        error('wrong number of input arguments');
    end
    j = [1 1 2];
    for i=1:3
        if periodicity(j(i))
            if tilt(i)*2 > box(j(i),2)-box(j(i),1)
                tilt(i) = tilt(i)-box(j(i),2)+box(j(i),1);
            elseif tilt(i)*2 < box(j(i),1)-box(j(i),1)
                tilt(i) = tilt(i)+box(j(i),2)-box(j(i),1);
            end
        end
    end
end

function exportlammpsdata(filename,T,atoms,types,ntypes)
    % Completely general function to output any atomic data to lammps data
    % format. T a 3x3 matrix of the box edges. atoms is a nx3 matrix of
    % atomic coordinates. types is a nx1 vector of the integer atomic
    % types.
    fid = fopen(filename,'w');
    fprintf(fid,'# Comments empty\n');
    fprintf(fid,'%d atoms\n',size(atoms,1));
    fprintf(fid,[num2str(ntypes) ' atom types\n']);
    T(1,1)=T(1,1)+1e-8;
    T(2,2)=T(2,2)+1e-8;
    fprintf(fid,'0 %14.8f xlo xhi\n',T(1,1));
    fprintf(fid,'0 %14.8f ylo yhi\n',T(2,2));
    fprintf(fid,'0 %14.8f zlo zhi\n\n',T(3,3));
    fprintf(fid,'%14.8f %14.8f %14.8f xy xz yz\n',[T(1,2) T(1,3) T(2,3)]);
    fprintf(fid,'Atoms\n\n');
    fprintf(fid,'%d %d %14.8f %14.8f %14.8f\n',[(1:size(atoms,1));types; atoms']);
    fclose(fid);
end

function [c I] = clip(ID,xvec,yvec,zvec)
    % clips atoms in ID to lie inside a parallelpiped defined by basis vectors
    % xvec, yvec, and zvec.
    S = [xvec' yvec' zvec'];
    c = (S^-1*ID')';
    I=1:size(ID,1);
    I = I(all(c < 1 & c >= 0,2));
    c = c(all(c < 1 & c >= 0,2),:);
    c = (S*c')';
end

function [Min m1 I] = mindist_c(ID,point)
% Min Distance
% Returns vector M with a scalar value m1 representing the coordinates of
% the nearest atom and the distance to the point. 
% Syntax [Vector magnitude Index] = mindist_c(ID,point);
mag = sqrt(dot(bsxfun(@minus,ID,point),bsxfun(@minus,ID,point),2));
[m1 I] = min(mag);
Min = [ID(I,1) ID(I,2) ID(I,3)]-point;
end

function [Min2 m2 I2] = mindist2_c(ID,point)
% 2nd Min Distance
% Returns vector M with a scalar value m2 representing the coordinates of
% the second nearest atom.
[~,~,I]=mindist_c(ID,point);
ID1=ID([1:I-1 I+1:end],:);
[Min2 m2 I2]=mindist_c(ID1,point);
if I2 >= I
    I2 = I2+1;
end
end



function x = integerconvertv(v,tol)
    for i=1:size(v,1)
        [x(i,1) x(i,2) x(i,3)] = integerconvert(v(i,1),v(i,2),v(i,3),tol);
    end
end

function [x, y, z] = integerconvert(a1, a2, a3, tol)
    % Convert three real numbers to a set of integers which are 
    % proportional to them, with a certain tolerance.
    loopend = 1;
    t = 0;
    n = 1;
    if a1 ~= 0
        k = a1;
    elseif a2 ~= 0
        k = a2;
    else k = a3;
    end      
    tend = 10/tol;
    while t < tend && loopend == 1;

        r = n/abs(k);
        a2m = a2 * r;
        a3m = a3 * r;
        a2i = round(a2m);
        a3i = round(a3m);
        a2err=abs(a2m-a2i);
        a3err=abs(a3m-a3i);
            if a2err <= tol && a3err <= tol
                loopend = 0;
            else
                n = n + 1;
                t = t + 1;
            end
    end

    x = round(r * a1);
    y = round(r * a2);
    z = round(r * a3);
end
