function [H V0 EsAl EsFe C11mC12 C44 B]=lammps_alloy()
    %Formation Energy and cell volume for given binary lattice:
    lattice='B1';
    minlatticeparameter=2.5; %Angstroms
    maxlatticeparameter=5.5;% At some point I'll write an automated min-max bound determining function.
    steps=81;
    data_file='atoms.data';
    meam_lib='library.meam';
    meam_param='AlSiMgCuFe.meam';
    elementlist = 'AlS SiS MgS CuS FeS';%names from meam.lib file. All elements in meam.param must be included.
    elements = regexp(elementlist,' ','split');
    element1_type=1; % define the first element used in this calculation
    element2_type=5; % define the second element used in this calculation
    element1=elements{element1_type};
    element2=elements{element2_type};
    EsubAl=3.353;%get from meam.lib file
    EsubFe=4.28 ;%get from meam.lib file
    ntypes = length(elements);
    [atoms,type,Box]=write_atom_position_file(lattice,minlatticeparameter,data_file,element1_type,element2_type,ntypes);
    natoms = size(atoms,1);
    step = (maxlatticeparameter-minlatticeparameter)/(steps-1);
    E =run_lammps(meam_lib,meam_param,elementlist,data_file,Box,step/minlatticeparameter,steps);
    a = minlatticeparameter+step*((1:steps)-1);
    [E0 I]=min(E);
    H=(min(E)+sum(type==element1_type)*EsubAl+sum(type==element2_type)*EsubFe)/natoms; %Heat of formation for given lattice. Fit to DFT explicitly for lattice(1,5) in param file: delta(1,5). Calibrated for other lattices.
    V0 = a(I)^3/natoms; % Per-atom volume for given lattce. Fit to DFT data explicitly for lattice(1,5) in param file: re(1,5). Calibrated for other lattices.
    
    % Elastic Constants B, (C11-C12)/2, C44 for given lattice
    Cs = 'C11,C12,C44';
    [atoms,type,Box]=write_atom_position_file(lattice,a(I),data_file,element1_type,element2_type,ntypes);
    writelammpsinputElastic('lammpsElastic.in',data_file,meam_lib,meam_param,elementlist,Cs)
    [C11 C12 C44] = runlammps2();
    B = C11/3/(1-2*C12/C11);
    C11mC12 = (C11-C12)/2;
    
    %Substitutional energy for Fe atom in Al matrix
    lattice='FCC'; % lat from lib.meam for Al
    latticeparameter=4.05; %alat from lib.meam
    [atoms,type,Box]=write_atom_position_file2(lattice,latticeparameter,data_file,element1_type,element2_type,ntypes);
    Es1 =run_lammps(meam_lib,meam_param,elementlist,data_file,Box,0,1);
    EsAl = Es1+EsubAl*(size(atoms,1)-1)+EsubFe;
    
    %Substitutional energy for Al atom in Fe matrix
    lattice='BCC'; % lat from lib.meam for Fe
    latticeparameter=2.851; %alat from lib.meam
    [atoms,type,Box]=write_atom_position_file2(lattice,latticeparameter,data_file,element2_type,element1_type,ntypes);
    Es2 =run_lammps(meam_lib,meam_param,elementlist,data_file,Box,0,1);
    EsFe = Es2+EsubFe*(size(atoms,1)-1)+EsubAl;
end

function E =run_lammps(meam_lib,meam_param,elementlist,data_file,Box,step,steps)
    ptr=lammps_open();
    lammps_command(ptr,'units metal');
    lammps_command(ptr,'boundary p p p');
    lammps_command(ptr,'atom_style atomic');
    lammps_command(ptr,['read_data ' data_file]);
    lammps_command(ptr,'pair_style meam');
    lammps_command(ptr,['pair_coeff * * ' meam_lib ' ' elementlist ' ' meam_param ' ' elementlist ]);
    lammps_command(ptr,'neighbor 0.3 nsq');
    lammps_command(ptr,'neigh_modify delay 1');
    lammps_command(ptr,'variable energy equal etotal');
    xdel = step*Box(1,1);
    xydel = step*Box(1,2);
    xzdel = step*Box(1,3);
    ydel = step*Box(2,2);
    yzdel = step*Box(2,3);
    zdel = step*Box(3,3);
    lammps_command(ptr,['fix 1 all deform 1 x delta 0  ' num2str(xdel,10) ' y delta 0  ' num2str(ydel,10) ' z delta 0 ' num2str(zdel,10) ' xy delta ' num2str(xydel,10) ' xz delta ' num2str(xzdel,10) ' yz delta ' num2str(yzdel,10) ' units box']);
    E = zeros(steps,1);
    for i=1:steps
        lammps_command(ptr,'run 1');
        lammps_command(ptr,'minimize 1e-8 1e-9 100 200');
        E(i) = lammps_extract_variable(ptr,'energy','null',0);
    end
    lammps_command(ptr,'print STOPPED');
    lammps_close(ptr);
end

function [C11 C12 C44] = runlammps2()
    ptr=lammps_open();
    lammps_file(ptr,'lammpsElastic.in');
    C11 = lammps_extract_variable(ptr,'C11','null',0);
    C12 = lammps_extract_variable(ptr,'C12','null',0);
    C44 = lammps_extract_variable(ptr,'C44','null',0);
    lammps_close(ptr);
end

function [atoms1 type Box] = write_atom_position_file(lattice,latticeparameter,filename,type1,type2,ntypes)
    % write_atom_position_file(lattice,latticeparameter,filename,type).
    % lattice is FCC, BCC, HCP, SC, or DC. latticeparameter is the
    % traditional unitcell parameter. filename is where atom data is
    % exported in lammps data format. type is the atom type for defining
    % different elements and depends on the Meam library file and the
    % lammps input file.
    [unitcell, type, atoms] = generateAlloyUnitCell(lattice);
    type11=(type==1);
    type22=(type==2);
    type= type11*type1+type22*type2;
    [atoms1 box tilt] = triclinic2lammps(atoms*latticeparameter,unitcell*latticeparameter);
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    exportlammpsdata(filename,Box,atoms1,type,ntypes);
end

function [atoms1 type Box] = write_atom_position_file2(lattice,latticeparameter,filename,type1,type2,ntypes)
    % write_atom_position_file(lattice,latticeparameter,filename,type).
    % lattice is FCC, BCC, HCP, SC, or DC. latticeparameter is the
    % traditional unitcell parameter. filename is where atom data is
    % exported in lammps data format. type is the atom type for defining
    % different elements and depends on the Meam library file and the
    % lammps input file.
    [unitcell, type, atoms] = generateAlloyUnitCell(lattice);
    type11=(type==1);
    type22=(type==2);
    type= type11*type1+type22*type2;
    [atoms1 box tilt] = triclinic2lammps(atoms*latticeparameter,unitcell*latticeparameter);
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    dims = [5 5 5];
    atoms1=simpleMakeLattice(Box',size(atoms1,1),atoms1,dims);
    Box(1,:) = Box(1,:)*dims(1);Box(2,:)=Box(2,:)*dims(2);Box(3,:)=Box(3,:)*dims(3);
    type = repmat(type,1,dims(1)*dims(2)*dims(3));
    type(1)=type2;
    exportlammpsdata(filename,Box,atoms1,type,ntypes);
end

function [unitcell type offset] = generateAlloyUnitCell(lattice)
    if strcmp(lattice,'FCC') || strcmp(lattice,'A1')
        unitcell = [1 0 0;0.5 0.5 0;-0.5 0 0.5];
        type = 1;
        offset = [0 0 0];
    elseif strcmp(lattice,'BCC') || strcmp(lattice,'A2')
        unitcell = [1 0 0;0 1 0;0.5 0.5 0.5];
        type = 1;
        offset = [0 0 0];
    elseif strcmp(lattice,'HCP') || strcmp(lattice,'A3')
        %ideal c/a ratio
        unitcell = [1 0 0;0.5 sqrt(3)/2 0;0 0 sqrt(8/3)];
        type = [1 1];
        offset = [0 0 0;1 1/sqrt(3) sqrt(2/3)];
    elseif strcmp(lattice,'SC')
        unitcell = [1 0 0;0 1 0;0 0 1];
        p = 1;
        offset = [0 0 0];
    elseif strcmp(lattice,'DC') || strcmp(lattice,'A4')
        unitcell = [1 0 0;0.5 0.5 0;0.5 0 0.5];
        p = 2;
        offset = [0 0 0;1.25 0.25 0.25];
    elseif strcmp(lattice,'B1')
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  2  2  2  2];
		offset = [				0 0       0.5;
				0       0.5 0;
				0.5 0 0;
				0.5       0.5       0.5;
				0 0 0;
				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0];
    elseif strcmp(lattice,'B2')
        % equivalent to single element BCC
        unitcell = [1 0 0;0 1 0;0 0 1];
        type = [1 2];
        offset = [0 0 0;0.5 0.5 0.5];
    elseif strcmp(lattice,'B3')
        % equivalent to single element DC
        unitcell = [1 0 0;0.5 0.5 0;0.5 0 0.5];
        type = [1 2];
        offset = [0 0 0;1.25 0.25 0.25];
    elseif strcmp(lattice,'B4')
		unitcell = [1 0 0;-0.5     0.8660254038 0;0 0      8.158995816];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2];
		offset = [				-5e-06     0.5773531559      1.427824268;
				-5e-06     0.5773531559      3.875523013;
				-4.999999999e-06     0.5773531559      6.323221757;
				0 0     0.6119246862;
				0 0      3.059623431;
				0     0.5773444957      1.427824268;
				0 0      4.691422594;
				0     0.5773444957      3.875523013;
				0 0      7.139121339;
				0     0.5773444957      6.323221757;
				5e-06     0.5773531559      1.427824268;
				5e-06     0.5773531559      3.875523013;
				5.000000001e-06     0.5773531559      6.323221757;
				0.499995     0.2886722478      2.243723849;
				0.499995     0.2886722478      5.507322176;
				0.499995     0.2886722478      7.955020921;
				0.5     0.2886809081      2.243723849;
				0.5     0.2886809081      5.507322176;
				0.5     0.2886809081      7.955020921;
				0.500005     0.2886722478      2.243723849;
				0.500005     0.2886722478      5.507322176;
				0.500005     0.2886722478      7.955020921;
				-5e-06     0.5773531559     0.8158995816;
				-5e-06     0.5773531559      3.263598326;
				-4.999999999e-06     0.5773531559      5.711297071;
				0 0 0;
				0 0      2.447698745;
				0     0.5773444957     0.8158995816;
				0 0      4.079497908;
				0     0.5773444957      3.263598326;
				0 0      6.527196653;
				0     0.5773444957      5.711297071;
				5e-06     0.5773531559     0.8158995816;
				5e-06     0.5773531559      3.263598326;
				5.000000001e-06     0.5773531559      5.711297071;
				0.499995     0.2886722478      1.631799163;
				0.499995     0.2886722478       4.89539749;
				0.499995     0.2886722478      7.343096234;
				0.5     0.2886809081      1.631799163;
				0.5     0.2886809081       4.89539749;
				0.5     0.2886809081      7.343096234;
				0.500005     0.2886722478      1.631799163;
				0.500005     0.2886722478       4.89539749;
				0.500005     0.2886722478      7.343096234];  
    elseif strcmp(lattice,'Bf')
		unitcell = [1 0 0;0      2.646682385 0;0 0     0.9875378915];
		type = [1  1  1  1  2  2  2  2];
		offset = [				0      1.151306837     0.2468844729;
				0      1.495375547     0.7406534187;
				0.5      0.172034355     0.7406534187;
				0.5       2.47464803     0.2468844729;
				0     0.3824456046     0.2468844729;
				0       2.26423678     0.7406534187;
				0.5      1.705786797     0.2468844729;
				0.5     0.9408955877     0.7406534187];     
    elseif strcmp(lattice,'C1')
        unitcell = [1 0 0;0 1 0;0 0 1];
        type = [1 1 1 1 2 2 2 2 2 2 2 2];
        offset = [0 0 0;0.5 0 0;0 0.5 0;0 0 0.5;0.25 0.25 0.25;0.25 0.25 0.75;0.25 0.75 0.25;0.25 0.75 0.75;0.75 0.25 0.25;0.75 0.25 0.75; 0.75 0.75 0.25; 0.75 0.75 0.75];
    elseif strcmp(lattice,'C2')
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  2  2  2  2  2  2  2  2];
		offset = [				0 0 0;
				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0;
				0.162     0.838     0.338;
				0.162     0.662     0.838;
				0.338     0.162     0.838;
				0.338     0.338     0.338;
				0.662     0.662     0.662;
				0.662     0.838     0.162;
				0.838     0.162     0.662;
				0.838     0.338     0.162];
    elseif strcmp(lattice,'C3')
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  2  2];
		offset = [				0.25      0.25      0.25;
				0.25      0.75      0.75;
				0.75      0.25      0.75;
				0.75      0.75      0.25;
				0 0 0;
				0.5       0.5       0.5];
    elseif strcmp(lattice,'C4')
		unitcell = [1 0 0;0 1 0;0 0      0.644005052];
		type = [1  1  1  1  2  2];
		offset = [				0.19504   0.80496      0.322002526;
				0.30496   0.30496 0;
				0.69504   0.69504 0;
				0.80496   0.19504      0.322002526;
				0 0 0;
				0.5       0.5      0.322002526];        
    elseif strcmp(lattice,'C8')
		unitcell = [1 0 0;-0.5     0.8660254038 0;0 0      1.099552482];
		type = [1  1  1  1  1  1  2  2  2];
		offset = [				-0.21995     0.6348832235     0.9685957811;
				0.06015     0.3581015045     0.6020782872;
				0.1598     0.7390660796     0.2355607933;
				0.28005     0.2311421803     0.1309567006;
				0.56015     0.5079238993     0.4974741945;
				0.6598     0.1269593242     0.8639916884;
				-0.23485     0.4067721322     0.7330349878;
				0.26515     0.4592532716     0.3665174939;
				0.4697 0 0];        
    elseif strcmp(lattice,'C9')
		unitcell = [1 0 0;0 1 0;0 0      1.392340648];
		type = [1  1  1  1  1  1  1  1  2  2  2  2];
		offset = [				0.1044    0.2392      1.143529374;
				0.2392    0.1044     0.2488112738;
				0.2608    0.6044    0.09927388821;
				0.3956    0.7392     0.5968964358;
				0.6044    0.2608       1.29306676;
				0.7392    0.3956     0.7954442122;
				0.7608    0.8956     0.9449815978;
				0.8956    0.7608     0.4473590502;
				0.19972   0.80028      0.348085162;
				0.30028   0.30028 0;
				0.69972   0.69972      0.696170324;
				0.80028   0.19972      1.044255486];        
    elseif strcmp(lattice,'C11b')
		unitcell = [1 0 0;0 1 0;0 0 2.4565625];
		type = [1  1  2  2  2  2];
		offset = [				0 0 0;
				0.5       0.5       1.22828125;
				0 0     0.8180353125;
				0 0      1.638527188;
				0.5       0.5     0.4102459375;
				0.5       0.5      2.046316562];    
    elseif strcmp(lattice,'C15')
        
    elseif strcmp(lattice,'C16')
		unitcell = [1 0 0;0 1 0;0 0     0.8038569309];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2];
		offset = [				0.1581    0.3419     0.4019284655;
				0.1581    0.6581 0;
				0.1581    0.3419     0.4019284655;
				0.1581    0.6581 0;
				0.3419    0.1581 0;
				0.3419    0.1581 0;
				0.3419    0.8419     0.4019284655;
				0.6581    0.1581     0.4019284655;
				0.6581    0.1581     0.4019284655;
				0.6581    0.8419 0;
				0.8419    0.3419 0;
				0.8419    0.6581     0.4019284655;
				0 0     0.2009642327;
				0 0     0.6028926982;
				0.5       0.5     0.2009642327;
				0.5       0.5     0.6028926982];        
    elseif strcmp(lattice,'C21')
		unitcell = [1 0 0;0     0.5939611947 0;0 0     0.5600610421];
		type = [1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  3  3  3  3  3  3  3  3];
		offset = [				0.0095    0.08855961413     0.1027712012;
				0.0095     0.2084209832     0.3828017223;
				0.4905     0.3855402115     0.1027712012;
				0.4905     0.5054015806     0.3828017223;
				0.5095    0.08855961413     0.1772593198;
				0.5095     0.2084209832     0.4572898409;
				0.9905     0.3855402115     0.1772593198;
				0.9905     0.5054015806     0.4572898409;
				0.2314    0.06592969261     0.3005287552;
				0.2314     0.2310509047    0.02049823414;
				0.2686     0.3629102899     0.3005287552;
				0.2686     0.5280315021    0.02049823414;
				0.7314    0.06592969261     0.5395628079;
				0.7314     0.2310509047     0.2595322869;
				0.7686     0.3629102899     0.5395628079;
				0.7686     0.5280315021     0.2595322869;
				0.1289    0.05773302812     0.4832206671;
				0.1289     0.2392475692     0.2031901461;
				0.3711     0.3547136255     0.4832206671;
				0.3711     0.5362281666     0.2031901461;
				0.6289    0.05773302812      0.356870896;
				0.6289     0.2392475692    0.07684037497;
				0.8711     0.3547136255      0.356870896;
				0.8711     0.5362281666    0.07684037497];        
    elseif strcmp(lattice,'L12')
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  2];
		offset = [				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0;
				0 0 0];     
    elseif strcmp(lattice,'L10')
		unitcell = [1 0 0;0 1 0;0 0       1.36588948];
		type = [1  2];
		offset = [				0 0 0;
				0.5       0.5     0.6829447402];
    elseif strcmp(lattice,'D03')
		unitcell = [1 0 0;0 1 0;0 0 1];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2];
		offset = [				0 0       0.5;
				0       0.5 0;
				0.25      0.25      0.25;
				0.25      0.25      0.75;
				0.25      0.75      0.25;
				0.25      0.75      0.75;
				0.5 0 0;
				0.5       0.5       0.5;
				0.75      0.25      0.25;
				0.75      0.25      0.75;
				0.75      0.75      0.25;
				0.75      0.75      0.75;
				0 0 0;
				0       0.5       0.5;
				0.5 0       0.5;
				0.5       0.5 0];    
    elseif strcmp(lattice,'D024')
		unitcell = [1 0 0;-0.5     0.8660254038 0;0 0      1.629513344];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2];
		offset = [				-0.250035     0.7216849497      1.222135008;
				-0.250015     0.7217195908      1.222135008;
				-0.25     0.4330127019 0;
				-0.25     0.4330127019     0.8147566719;
				-2e-05     0.2886462671      1.222135008;
				2e-05     0.2886462671      1.222135008;
				0.249965      0.144340454     0.4073783359;
				0.249985      0.144305813     0.4073783359;
				0.25     0.4330127019 0;
				0.25     0.4330127019     0.8147566719;
				0.250015     0.7217195908      1.222135008;
				0.250035     0.7216849497      1.222135008;
				0.49998     0.5773791367     0.4073783359;
				0.5 0 0;
				0.5 0     0.8147566719;
				0.50002     0.5773791367     0.4073783359;
				0.750015      0.144305813     0.4073783359;
				0.750035      0.144340454     0.4073783359;
				-5e-06     0.5773531559     0.4073783359;
				0 0 0;
				0 0     0.8147566719;
				0     0.5773444957     0.4073783359;
				5e-06     0.5773531559     0.4073783359;
				0.499995     0.2886722478      1.222135008;
				0.5     0.2886809081      1.222135008;
				0.500005     0.2886722478      1.222135008];       
    elseif strcmp(lattice,'D8b')
		unitcell = [1 0 0;0 1 0;0 0     0.5163636364];
		type = [1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2  2];
		offset = [				0 0 0;
				0.0368    0.6316     0.2581818182;
				0.0653    0.7376 0;
				0.1019    0.8981     0.2581818182;
				0.1316    0.4632 0;
				0.1823    0.1823     0.1303301818;
				0.1823    0.1823     0.3860334545;
				0.2376    0.4347     0.2581818182;
				0.2624    0.9347 0;
				0.3177    0.6823     0.1278516364;
				0.3177    0.6823  0.388512;
				0.3684    0.9632     0.2581818182;
				0.3981    0.3981 0;
				0.4347    0.2376     0.2581818182;
				0.4632    0.1316 0;
				0.5       0.5     0.2581818182;
				0.5368    0.8684 0;
				0.5653    0.7624     0.2581818182;
				0.6019    0.6019 0;
				0.6316    0.0368     0.2581818182;
				0.6823    0.3177     0.1278516364;
				0.6823    0.3177  0.388512;
				0.7376    0.0653 0;
				0.7624    0.5653     0.2581818182;
				0.8177    0.8177     0.1303301818;
				0.8177    0.8177     0.3860334545;
				0.8684    0.5368 0;
				0.8981    0.1019     0.2581818182;
				0.9347    0.2624 0;
				0.9632    0.3684     0.2581818182;
				0 0 0;
				0.0368    0.6316     0.2581818182;
				0.0653    0.7376 0;
				0.1019    0.8981     0.2581818182;
				0.1316    0.4632 0;
				0.1823    0.1823     0.1303301818;
				0.1823    0.1823     0.3860334545;
				0.2376    0.4347     0.2581818182;
				0.2624    0.9347 0;
				0.3177    0.6823     0.1278516364;
				0.3177    0.6823  0.388512;
				0.3684    0.9632     0.2581818182;
				0.3981    0.3981 0;
				0.4347    0.2376     0.2581818182;
				0.4632    0.1316 0;
				0.5       0.5     0.2581818182;
				0.5368    0.8684 0;
				0.5653    0.7624     0.2581818182;
				0.6019    0.6019 0;
				0.6316    0.0368     0.2581818182;
				0.6823    0.3177     0.1278516364;
				0.6823    0.3177  0.388512;
				0.7376    0.0653 0;
				0.7624    0.5653     0.2581818182;
				0.8177    0.8177     0.1303301818;
				0.8177    0.8177     0.3860334545;
				0.8684    0.5368 0;
				0.8981    0.1019     0.2581818182;
				0.9347    0.2624 0;
				0.9632    0.3684     0.2581818182];             
    else
        error('Unsupported lattice.');
    end
end

function atoms = simpleMakeLattice(unitcell,p,offset,dims)
    atoms = zeros(dims(1)*dims(2)*dims(3)*p,3);
    for l=1:p
        for i=1:dims(1)
            for j=1:dims(2)
                for k=1:dims(3)
                    atoms((l-1)*dims(1)*dims(2)*dims(3)+(i-1)*dims(2)*dims(3)+(j-1)*dims(3)+k,:) = (i-1)*unitcell(1,:)+(j-1)*unitcell(2,:)+(k-1)*unitcell(3,:) + offset(l,:);
                end
            end
        end
    end
end

function [x, box, tilt, R] = triclinic2lammps(X,Box)
    % Lammps only takes very specific non-orthogonal box types. We do a
    % rotation into the required form.
    % Adapted from http://lammps.sandia.gov/doc/Section_howto.html#howto_12
    xlo = 0;
    ylo = 0;
    zlo = 0;
    xhi = norm(Box(1,:));
    xy = dot(Box(2,:),Box(1,:)/norm(Box(1,:)));
    yhi = norm(cross(Box(1,:)/norm(Box(1,:)),Box(2,:)));
    xz = dot(Box(3,:),Box(1,:)/norm(Box(1,:)));
    yz = (dot(Box(3,:),Box(2,:))-xy*xz)/yhi;
    zhi = sqrt(dot(Box(3,:),Box(3,:))-xz^2-yz^2);
    box = [xlo xhi;ylo yhi;zlo zhi];
    tilt = [xy xz yz];
    R = [xhi xy xz;0 yhi yz;0 0 zhi]*[cross(Box(2,:),Box(3,:));cross(Box(3,:),Box(1,:));cross(Box(1,:),Box(2,:))]/det([xhi xy xz;0 yhi yz;0 0 zhi]);
    x = (R*X')';
end


function tilt = adjusttilt(box,tilt,varargin)
    % lammps requires boxes not to be extremely tilted when periodic.
    % If boxes become extremely tilted during simulations, they are
    % automatically remapped. Free surface dimensions are not affected by this.
    % Boxes defined via Read_data must already satisfy the tilt conditions
    % which this function ensures.
    % http://lammps.sandia.gov/doc/Section_howto.html#howto_12
    if isempty(varargin)
        periodicity = [1 1 1];
    elseif length(varargin)==1
        periodicity = varargin{1};
    elseif length(varargin)==3
        periodicity(1)=varargin{1};
        periodicity(2)=varargin{2};
        periodicity(3)=varargin{3};
    else
        error('wrong number of input arguments');
    end
    j = [1 1 2];
    for i=1:3
        if periodicity(j(i))
            if tilt(i)*2 > box(j(i),2)-box(j(i),1)
                tilt(i) = tilt(i)-box(j(i),2)+box(j(i),1);
            elseif tilt(i)*2 < box(j(i),1)-box(j(i),1)
                tilt(i) = tilt(i)+box(j(i),2)-box(j(i),1);
            end
        end
    end
end

function exportlammpsdata(filename,T,atoms,types,ntypes)
    % Completely general function to output any atomic data to lammps data
    % format. T a 3x3 matrix of the box edges. atoms is a nx3 matrix of
    % atomic coordinates. types is a nx1 vector of the integer atomic
    % types.
    fid = fopen(filename,'w');
    fprintf(fid,'# Comments empty\n');
    fprintf(fid,'%d atoms\n',size(atoms,1));
    fprintf(fid,[num2str(ntypes) ' atom types\n']);
    fprintf(fid,'0 %14.8f xlo xhi\n',T(1,1));
    fprintf(fid,'0 %14.8f ylo yhi\n',T(2,2));
    fprintf(fid,'0 %14.8f zlo zhi\n\n',T(3,3));
    fprintf(fid,'%14.8f %14.8f %14.8f xy xz yz\n',[T(1,2) T(1,3) T(2,3)]);
    fprintf(fid,'Atoms\n\n');
    fprintf(fid,'%d %d %14.8f %14.8f %14.8f\n',[(1:size(atoms,1));types; atoms']);
    fclose(fid);
end

function writelammpsinputElastic(outfilename,datafilename,potfilename,parameterfilename,elementlist,Cs)
    % Syntax and documentation for lammps input at:
    % http://lammps.sandia.gov/doc/Section_commands.html
    % Elastic routine adapted from http://lammps.sandia.gov/doc/Section_example.html
    do = regexp(Cs, ',','split');
    C = zeros(6,6);
    for i=1:length(do)
        eval([do{i}(1) '(' do{i}(2) ',' do{i}(3) ')=1;']);
        eval([do{i}(1) '(' do{i}(3) ',' do{i}(2) ')=1;']);
    end
    fileID=fopen(outfilename,'w');
    fprintf(fileID,'#Lammps input script autogenerated by MATLAB writelammpsinputElastic\n');
    fprintf(fileID,'#Author: Christopher Barrett, 11/7/14.\n');
    fprintf(fileID,'units metal\n');
    fprintf(fileID,'variable cfac equal 1.0e-4\n');
    fprintf(fileID,'variable cunits string GPa\n');
    fprintf(fileID,'variable up equal 1.0e-6\n');
    fprintf(fileID,'boundary p p p\n');
    fprintf(fileID,'atom_style atomic\n');
    fprintf(fileID,['read_data ' datafilename '\n']);
    fprintf(fileID,'pair_style meam\n');
    fprintf(fileID,['pair_coeff * * ' potfilename ' ' elementlist ' ' parameterfilename ' ' elementlist '\n']);
    fprintf(fileID,'neighbor 0.3 nsq\n');
    fprintf(fileID,'neigh_modify delay 1\n');
    fprintf(fileID,'thermo		1\n');
    fprintf(fileID,'thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol\n');
    fprintf(fileID,'thermo_modify norm no\n');
    fprintf(fileID,'fix 3 all box/relax aniso 0.0\n');
    fprintf(fileID,'minimize 1e-8 1e-9 5000 10000\n\n');

    fprintf(fileID,'variable tmp equal pxx\n');
    fprintf(fileID,'variable pxx0 equal ${tmp}\n');
    fprintf(fileID,'variable tmp equal pyy\n');
    fprintf(fileID,'variable pyy0 equal ${tmp}\n');
    fprintf(fileID,'variable tmp equal pzz\n');
    fprintf(fileID,'variable pzz0 equal ${tmp}\n');
    fprintf(fileID,'variable tmp equal pyz\n');
    fprintf(fileID,'variable pyz0 equal ${tmp}\n');
    fprintf(fileID,'variable tmp equal pxz\n');
    fprintf(fileID,'variable pxz0 equal ${tmp}\n');
    fprintf(fileID,'variable tmp equal pxy\n');
    fprintf(fileID,'variable pxy0 equal ${tmp}\n\n');

    fprintf(fileID,'variable tmp equal lx\n');
    fprintf(fileID,'variable lx0 equal ${tmp}\n');
    fprintf(fileID,'variable tmp equal ly\n');
    fprintf(fileID,'variable ly0 equal ${tmp}\n');
    fprintf(fileID,'variable tmp equal lz\n');
    fprintf(fileID,'variable lz0 equal ${tmp}\n\n');

    fprintf(fileID,'variable d1 equal -(v_pxx1-${pxx0})/(v_delta/v_len0)*${cfac}\n');
    fprintf(fileID,'variable d2 equal -(v_pyy1-${pyy0})/(v_delta/v_len0)*${cfac}\n');
    fprintf(fileID,'variable d3 equal -(v_pzz1-${pzz0})/(v_delta/v_len0)*${cfac}\n');
    fprintf(fileID,'variable d4 equal -(v_pyz1-${pyz0})/(v_delta/v_len0)*${cfac}\n');
    fprintf(fileID,'variable d5 equal -(v_pxz1-${pxz0})/(v_delta/v_len0)*${cfac}\n');
    fprintf(fileID,'variable d6 equal -(v_pxy1-${pxy0})/(v_delta/v_len0)*${cfac}\n\n');

    fprintf(fileID,'unfix 3\n');
    fprintf(fileID,'write_restart restart.equil\n');
    dir = {'x','y','z','yz','xz','xy'};
    shear = {'0','0','0','','',''};
    D=C;
    for i=1:6
        if ~C(i,i) && any(C(i,:))
            for j=1:6
                if C(i,j) && ~C(j,j)
                        D(j,j)=1;
                end
            end
        end
    end
    for i=1:6
        if D(i,i)
            fprintf(fileID,['variable dir equal ' num2str(i) '\n']);
            fprintf(fileID,['variable len0 equal ${l' dir{i}(end) '0}\n']);
            fprintf(fileID,'clear\n');
            fprintf(fileID,'read_restart restart.equil\n');
            fprintf(fileID,'pair_style meam\n');
            fprintf(fileID,['pair_coeff * * ' potfilename ' ' elementlist ' ' parameterfilename ' ' elementlist '\n']);
            fprintf(fileID,'variable delta equal -${up}*${len0}\n');
            fprintf(fileID,['change_box all ' dir{i} ' delta ' shear{i} ' ${delta} units box\n']);
            fprintf(fileID,'thermo		1\n');
            fprintf(fileID,'thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol\n');
            fprintf(fileID,'thermo_modify norm no\n');
            fprintf(fileID,'minimize 1e-8 1e-9 5000 10000\n\n');
            fprintf(fileID,'variable tmp equal pxx\n');
            fprintf(fileID,'variable pxx1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pyy\n');
            fprintf(fileID,'variable pyy1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pzz\n');
            fprintf(fileID,'variable pzz1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pxy\n');
            fprintf(fileID,'variable pxy1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pxz\n');
            fprintf(fileID,'variable pxz1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pyz\n');
            fprintf(fileID,'variable pyz1 equal ${tmp}\n');

            fprintf(fileID,'variable C1neg equal ${d1}\n');
            fprintf(fileID,'variable C2neg equal ${d2}\n');
            fprintf(fileID,'variable C3neg equal ${d3}\n');
            fprintf(fileID,'variable C4neg equal ${d4}\n');
            fprintf(fileID,'variable C5neg equal ${d5}\n');
            fprintf(fileID,'variable C6neg equal ${d6}\n');

            fprintf(fileID,'clear\n');
            fprintf(fileID,'read_restart restart.equil\n');
            fprintf(fileID,'pair_style meam\n');
            fprintf(fileID,['pair_coeff * * ' potfilename ' ' elementlist ' ' parameterfilename ' ' elementlist '\n']);

            fprintf(fileID,'variable delta equal ${up}*${len0}\n');
            fprintf(fileID,['change_box all ' dir{i} ' delta ' shear{i} ' ${delta} units box\n']);
            fprintf(fileID,'thermo		1\n');
            fprintf(fileID,'thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol\n');
            fprintf(fileID,'thermo_modify norm no\n');
            fprintf(fileID,'minimize 1e-8 1e-9 5000 10000\n\n');
            fprintf(fileID,'variable tmp equal pe\n');
            fprintf(fileID,'variable e1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal press\n');
            fprintf(fileID,'variable p1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pxx\n');
            fprintf(fileID,'variable pxx1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pyy\n');
            fprintf(fileID,'variable pyy1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pzz\n');
            fprintf(fileID,'variable pzz1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pxy\n');
            fprintf(fileID,'variable pxy1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pxz\n');
            fprintf(fileID,'variable pxz1 equal ${tmp}\n');
            fprintf(fileID,'variable tmp equal pyz\n');
            fprintf(fileID,'variable pyz1 equal ${tmp}\n');

            fprintf(fileID,'variable C1pos equal ${d1}\n');
            fprintf(fileID,'variable C2pos equal ${d2}\n');
            fprintf(fileID,'variable C3pos equal ${d3}\n');
            fprintf(fileID,'variable C4pos equal ${d4}\n');
            fprintf(fileID,'variable C5pos equal ${d5}\n');
            fprintf(fileID,'variable C6pos equal ${d6}\n');

            fprintf(fileID,'variable C1${dir} equal 0.5*(${C1neg}+${C1pos})\n');
            fprintf(fileID,'variable C2${dir} equal 0.5*(${C2neg}+${C2pos})\n');
            fprintf(fileID,'variable C3${dir} equal 0.5*(${C3neg}+${C3pos})\n');
            fprintf(fileID,'variable C4${dir} equal 0.5*(${C4neg}+${C4pos})\n');
            fprintf(fileID,'variable C5${dir} equal 0.5*(${C5neg}+${C5pos})\n');
            fprintf(fileID,'variable C6${dir} equal 0.5*(${C6neg}+${C6pos})\n');
            fprintf(fileID,'variable C${dir}1 equal 0.5*(${C1neg}+${C1pos})\n');
            fprintf(fileID,'variable C${dir}2 equal 0.5*(${C2neg}+${C2pos})\n');
            fprintf(fileID,'variable C${dir}3 equal 0.5*(${C3neg}+${C3pos})\n');
            fprintf(fileID,'variable C${dir}4 equal 0.5*(${C4neg}+${C4pos})\n');
            fprintf(fileID,'variable C${dir}5 equal 0.5*(${C5neg}+${C5pos})\n');
            fprintf(fileID,'variable C${dir}6 equal 0.5*(${C6neg}+${C6pos})\n');

            fprintf(fileID,'variable dir delete\n');
        end
    end
    for i=1:6
        for j=1:i
            if C(j,i)
                fprintf(fileID,['print "Elastic Constant C(' num2str(j) ',' num2str(i) ')=${C' num2str(j) num2str(i) '} ${cunits}"\n']);
            end
        end
    end
    fprintf(fileID,'print "ALL DONE"');
    fclose(fileID);
end

