function E=lammps_example()
    lattice='FCC';
    latticeparameter=4.05; %Angstroms
    data_file='atoms.data';
    %input_script='lammpsscript.in';
    meam_lib='Al-example.library.meam';
    meam_param='Al-example.parameter.meam';
    element_name='Al';
    %writelammpsinputcgmin(input_script,data_file,meam_lib,meam_param,element_name)
    write_atom_position_file(lattice,latticeparameter,data_file,1);
    E=run_lammps(meam_lib,meam_param,element_name,data_file);
end

function E=run_lammps(meam_lib,meam_param,element_name,data_file)
    ptr=lammps_open();
    lammps_command(ptr,'units metal');
    lammps_command(ptr,'boundary p p p');
    lammps_command(ptr,'atom_style atomic');
    lammps_command(ptr,['read_data ' data_file]);
    lammps_command(ptr,'pair_style meam');
    lammps_command(ptr,['pair_coeff ** ' meam_lib ' ' element_name ' ' meam_param ' ' element_name ' ']);
    lammps_command(ptr,'neighbor 0.3 nsq');
    lammps_command(ptr,'neigh_modify delay 1');
    lammps_command(ptr,'thermo 1');
    lammps_command(ptr,'thermo_style custom step etotal vol lx ly lz ke');
    lammps_command(ptr,'thermo_modify format int %5d');
    lammps_command(ptr,'thermo_modify format 2 %28.16f');
    lammps_command(ptr,'thermo_modify format 4 %28.16f');
    lammps_command(ptr,'thermo_modify format float %15.4f');
    lammps_command(ptr,'fix 1 all deform 1 x scale     2.0126582278481013 y scale     2.0126582278481013 z scale     2.0126582278481013');
    lammps_command(ptr,'units box');
    E = zeros(81,1);
    for i=1:81
        lammps_command(ptr,'run 1');
        E(i) = lammps_extract_compute(ptr,'eatoms',0,0);
    end
    lammps_command(ptr,'print STOPPED');
    lammps_close(ptr);
end

function writelammpsinputcgmin(outfilename,datafilename,potfilename,parameterfilename,elementlist)
    % Syntax and documentation for lammps input at:
    % http://lammps.sandia.gov/doc/Section_commands.html
    fileID=fopen(outfilename,'w');
    fprintf(fileID,'#Lammps input script autogenerated by MATLAB writelammpsinputcgmin\n');
    fprintf(fileID,'#Author: Christopher Barrett, 11/7/14.\n');
    fprintf(fileID,'units metal\n');
    fprintf(fileID,'boundary p p p\n');
    fprintf(fileID,'atom_style atomic\n');
    fprintf(fileID,['read_data ' datafilename '\n']);
    fprintf(fileID,'pair_style meam\n');
    fprintf(fileID,['pair_coeff * * ' potfilename ' ' elementlist ' ' parameterfilename ' ' elementlist '\n']);
    fprintf(fileID,'neighbor 0.3 nsq\n');
    fprintf(fileID,'neigh_modify delay 1\n');
    fprintf(fileID,'compute eng all pe/atom\n');
    fprintf(fileID,'compute eatoms all reduce sum c_eng\n');
    fprintf(fileID,'thermo 1\n');
    fprintf(fileID,'thermo_style custom step etotal vol lx ly lz ke\n');
    fprintf(fileID,'thermo_modify format int %%5d\n');
    fprintf(fileID,'thermo_modify format 2 %%28.16f\n');
    fprintf(fileID,'thermo_modify format float %%15.4f\n');
    fprintf(fileID,'minimize 1.0e-8 1.0e-9 5000 10000\n');
    fclose(fileID);
end

function write_atom_position_file(lattice,latticeparameter,filename,type)
    % write_atom_position_file(lattice,latticeparameter,filename,type).
    % lattice is FCC, BCC, HCP, SC, or DC. latticeparameter is the
    % traditional unitcell parameter. filename is where atom data is
    % exported in lammps data format. type is the atom type for defining
    % different elements and depends on the Meam library file and the
    % lammps input file.
    [unitcell, ~, atoms] = generatePrimitiveCell(lattice);
    [atoms1 box tilt] = triclinic2lammps(atoms*latticeparameter,unitcell*latticeparameter);
    tilt = adjusttilt(box,tilt,[1 1 1]);
    Box = [box(1,2) tilt(1) tilt(2);0 box(2,2) tilt(3);0 0 box(3,2)];
    exportlammpsdata(filename,Box,atoms1,type);
end

function [unitcell p offset] = generatePrimitiveCell(lattice)
% simple function which defines the PRIMITIVE unitcell for a given lattice
% Note the primitive cell is not usually the traditionally recognized one.
% This one is smaller, and therefore more efficient.
    if strcmp(lattice,'FCC')
        unitcell = [1 0 0;0.5 0.5 0;0.5 0 0.5];
        p = 1;
        offset = [0 0 0];
    elseif strcmp(lattice,'BCC')
        unitcell = [1 0 0;0 1 0;0.5 0.5 0.5];
        p = 1;
        offset = [0 0 0];
    elseif strcmp(lattice,'HCP')
        %ideal c/a ratio
        unitcell = [1 0 0;0.5 sqrt(3)/2 0;0 0 sqrt(8/3)];
        p = 2;
        offset = [0 0 0;1 1/sqrt(3) sqrt(2/3)];
    elseif strcmp(lattice,'SC')
        unitcell = [1 0 0;0 1 0;0 0 1];
        p = 1;
        offset = [0 0 0];
    elseif strcmp(lattice,'DC')
        unitcell = [1 0 0;0.5 0.5 0;0.5 0 0.5];
        p = 2;
        offset = [0 0 0;1.25 0.25 0.25];
    else
        error('Unsupported lattice.');
    end
end

function [x, box, tilt, R] = triclinic2lammps(X,Box)
    % Lammps only takes very specific non-orthogonal box types. We do a
    % rotation into the required form.
    % Adapted from http://lammps.sandia.gov/doc/Section_howto.html#howto_12
    xlo = 0;
    ylo = 0;
    zlo = 0;
    xhi = norm(Box(1,:));
    xy = dot(Box(2,:),Box(1,:)/norm(Box(1,:)));
    yhi = norm(cross(Box(1,:)/norm(Box(1,:)),Box(2,:)));
    xz = dot(Box(3,:),Box(1,:)/norm(Box(1,:)));
    yz = (dot(Box(3,:),Box(2,:))-xy*xz)/yhi;
    zhi = sqrt(dot(Box(3,:),Box(3,:))-xz^2-yz^2);
    box = [xlo xhi;ylo yhi;zlo zhi];
    tilt = [xy xz yz];
    R = [xhi xy xz;0 yhi yz;0 0 zhi]*[cross(Box(2,:),Box(3,:));cross(Box(3,:),Box(1,:));cross(Box(1,:),Box(2,:))]/det([xhi xy xz;0 yhi yz;0 0 zhi]);
    x = (R*X')';
end


function tilt = adjusttilt(box,tilt,varargin)
    % lammps requires boxes not to be extremely tilted when periodic.
    % If boxes become extremely tilted during simulations, they are
    % automatically remapped. Free surface dimensions are not affected by this.
    % Boxes defined via Read_data must already satisfy the tilt conditions
    % which this function ensures.
    % http://lammps.sandia.gov/doc/Section_howto.html#howto_12
    if isempty(varargin)
        periodicity = [1 1 1];
    elseif length(varargin)==1
        periodicity = varargin{1};
    elseif length(varargin)==3
        periodicity(1)=varargin{1};
        periodicity(2)=varargin{2};
        periodicity(3)=varargin{3};
    else
        error('wrong number of input arguments');
    end
    j = [1 1 2];
    for i=1:3
        if periodicity(j(i))
            if tilt(i)*2 > box(j(i),2)-box(j(i),1)
                tilt(i) = tilt(i)-box(j(i),2)+box(j(i),1);
            elseif tilt(i)*2 < box(j(i),1)-box(j(i),1)
                tilt(i) = tilt(i)+box(j(i),2)-box(j(i),1);
            end
        end
    end
end

function exportlammpsdata(filename,T,atoms,types)
    % Completely general function to output any atomic data to lammps data
    % format. T a 3x3 matrix of the box edges. atoms is a nx3 matrix of
    % atomic coordinates. types is a nx1 vector of the integer atomic
    % types.
    fid = fopen(filename,'w');
    fprintf(fid,'# Comments empty\n');
    fprintf(fid,'%d atoms\n',size(atoms,1));
    fprintf(fid,[num2str(max(types)) ' atom types\n']);
    fprintf(fid,'0 %14.8f xlo xhi\n',T(1,1));
    fprintf(fid,'0 %14.8f ylo yhi\n',T(2,2));
    fprintf(fid,'0 %14.8f zlo zhi\n\n',T(3,3));
    fprintf(fid,'%14.8f %14.8f %14.8f xy xz yz\n',[T(1,2) T(1,3) T(2,3)]);
    fprintf(fid,'Atoms\n\n');
    fprintf(fid,'%d %d %14.8f %14.8f %14.8f\n',[(1:size(atoms,1));types; atoms']);
    fclose(fid);
end
