#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
/* ----------------------------------------------------------------------
   extract an internal LAMMPS atom-based entity
   name = desired quantity, e.g. 'x', 'f', or 'mass'
   data=lammps_extract_atom(lmp_pointer,name);
------------------------------------------------------------------------- */

/*function prototype necessary to avoid truncation to 32 bits*/
void * lammps_extract_atom(void * lmp1,char * command);

void mexFunction( int nlhs, mxArray *plhs[],
int nrhs,  const mxArray *prhs[])
{
    double *lmp;
    char *command;
    long lmp1;
    int i,j;
    double *outaddr;
    int *outaddr1;
    double *data;
    int *data1;
    if (nlhs != 1)
    {
        mxErrMsgTxt("Out output argument expected.");
    }
    if (nrhs != 2 || !mxIsChar(prhs[1]))
    {
        mxErrMsgTxt("Invalid syntax. Should be like so: data=lammps_extract_atom(lmp_pointer,'data type').\n Valid data types: mass,id,type,mask,image,x,v,f,molecule,q,mu,omega,amgmom (note spelling),torque,radius,rmass,vfrac,s0.\n Not all data types are defined in all simulations. Requesting undefined data causes segfault.");
    }
    command=(char *)mxMalloc(mxGetN(prhs[1]) * sizeof(char));
    mxGetString(prhs[1],command,mxGetN(prhs[1])+1);
    lmp = mxGetPr(prhs[0]);
    lmp1 = (long)*lmp;
    if (strcmp(command,"x")==0 || strcmp(command,"f")==0 || strcmp(command,"v")==0 || strcmp(command,"mu")==0 || strcmp(command,"omega")==0 || strcmp(command,"amgmom")==0 || strcmp(command,"torque")==0)
    {
        double **data = (double **)lammps_extract_atom((void *)lmp1,command);
        if (data==NULL)
        {
            mxErrMsgTxt("Invalid data type");
        }
        plhs[0] = mxCreateDoubleMatrix(lammps_get_natoms((void *)lmp1),3,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            for (j=0;j<3;j++)
            {
                outaddr[i*3+j] = data[i][j];
            }
        }
    }
    // mass currently unsupported;
    /*else if (strcmp(command,"mass")==0)
    {
        data = (double *)lammps_extract_atom((void *)lmp1,command);
        if (data==NULL)
        {
            mxErrMsgTxt("Invalid data type");
        }
        plhs[0] = mxCreateDoubleMatrix(lammps_get_ntypes((void *)lmp1),1,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_ntypes((void *)lmp1);i++)
        {
            outaddr[i] = data[i];
        }
    }*/
    else if (strcmp(command,"q")==0 || strcmp(command,"radius")==0 || strcmp(command,"rmass")==0 || strcmp(command,"vfrac")==0 || strcmp(command,"s0")==0)
    {
        data = (double *)lammps_extract_atom((void *)lmp1,command);
        if (data==NULL)
        {
            mxErrMsgTxt("Invalid data type");
        }
        plhs[0] = mxCreateDoubleMatrix(lammps_get_natoms((void *)lmp1),1,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            outaddr[i] = data[i];
        }
    }
    else
    {
        data1 = (int *)lammps_extract_atom((void *)lmp1,command);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type");
        }
        plhs[0] = mxCreateNumericMatrix(lammps_get_natoms((void *)lmp1),1, mxINT32_CLASS, mxREAL);
        outaddr1 = mxGetData(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            outaddr1[i] = data1[i];
        }
    }
}