#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
/* ----------------------------------------------------------------------
   extract an internal LAMMPS compute-based entity
   id = compute ID
   style = 0 for global data, 1 for per-atom data, 2 for local data
   type = 0 for scalar, 1 for vector, 2 for array
   length = length of non per-atom array dimensions. For style=0,type=2,
   length = [i j]; else length = [i];
   for global data, returns a pointer to the
     compute's internal data structure for the entity
     caller should cast it to (double *) for a scalar or vector
     caller should cast it to (double **) for an array
   for per-atom or local data, returns a pointer to the
     compute's internal data structure for the entity
     caller should cast it to (double *) for a vector
     caller should cast it to (double **) for an array
   IMPORTANT: if the compute is not current it will be invoked
     LAMMPS cannot easily check here if it is valid to invoke the compute,
     so caller must insure that it is OK
   compute_data=lammps_extract_compute(lmp_pointer,compute_id,style,type,length);
------------------------------------------------------------------------- */

/*function prototype necessary to avoid truncation to 32 bits*/
void * lammps_extract_compute(void * lmp1,char * command, int style, int type);

void mexFunction( int nlhs, mxArray *plhs[],
int nrhs,  const mxArray *prhs[])
{
    double *lmp;
    char *command;
    long lmp1;
    double *style;
    int style1;
    double *type;
    int type1;
    int len,len1;
    double *vectorlength;
    double **data;
    double *data1;
    int i,j;
    double *outaddr;
    if (nlhs != 1)
    {
        mxErrMsgTxt("One output argument required.");
    }
    if ((nrhs != 4 && nrhs != 5) || !mxIsChar(prhs[1]) || !mxIsDouble(prhs[2]) || !mxIsDouble(prhs[3]))
    {
        mxErrMsgTxt("Invalid synatx: Should be like so:\n compute_data=lammps_extract_compute(lmp_pointer,compute_id,style,type,length).\n    style = 0 for global data, 1 for per-atom data, 2 for local data\ntype = 0 for scalar, 1 for vector, 2 for array.\n style and type must be input as standard matlab numbers, i.e., doubles. For local and per-atom arrays, length is the number of per-atom elements. compute_data=lammps_extract_compute(lmp_pointer,compute_id,style,type) may be called when length=1.");
    }
    command=(char *)mxMalloc(mxGetN(prhs[1]) * sizeof(char));
    mxGetString(prhs[1],command,mxGetN(prhs[1])+1);
    lmp = mxGetPr(prhs[0]);
    lmp1 = (long)*lmp;
    style = mxGetPr(prhs[2]);
    style1 = (int)*style;
    type = mxGetPr(prhs[3]);
    type1 = (int)*type;
    if (type1-style1>0)
    {
        if (nrhs !=5 || !mxIsDouble(plhs[4]))
        {
            mxErrMsgTxt("Invalid Syntax. Use compute_data=lammps_extract_compute(lmp_pointer,compute_id,style,type,length). Length must be defined.");
        }
        vectorlength = mxGetPr(prhs[4]);
        len = (int)*vectorlength;
        if (type1==2 && style1==0)
        {
            len1 = (int)vectorlength[1];
        }
    }
    if (type1==2 && style1>0)
    {
        data = (double **)lammps_extract_compute((void *)lmp1,command,style1,type1);
        if (data==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(lammps_get_natoms((void *)lmp1),len,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            for (j=0;j<len;j++)
            {
                outaddr[i*(len)+j] = data[i][j];
            }
        }
    }
    else if (type1==1 && style1>0)
    {
        data1 = (double *)lammps_extract_compute((void *)lmp1,command,style1,type1);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(lammps_get_natoms((void *)lmp1),1,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            outaddr[i] = data1[i];
        }
    }
    else if (type1==2 && style1==0)
    {
        data = (double **)lammps_extract_compute((void *)lmp1,command,style1,type1);
        if (data==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(len,len1,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<len;i++)
        {
            for (j=0;j<len1;j++)
            {
                outaddr[i*(len1)+j] = data[i][j];
            }
        }
    }
    else if (type1==1 && style1==0)
    {
        data1 = (double *)lammps_extract_compute((void *)lmp1,command,style1,type1);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(len,1,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<len;i++)
        {
            outaddr[i] = data1[i];
        }
    }
    else if (type1==0 && style1==0)
    {
        data1 = (double *)lammps_extract_compute((void *)lmp1,command,style1,type1);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<1;i++)
        {
            outaddr[i] = data1[i];
        }
    }
}