#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
/* ----------------------------------------------------------------------
   extract an internal LAMMPS fix-based entity
   id = fix ID
   style = 0 for global data, 1 for per-atom data, 2 for local data
   type = 0 for scalar, 1 for vector, 2 for array
   i,j = indices needed only to specify which global vector or array value
   length = length of non per-atom array dimensions. For style=0,type=2,
   length = [i j]; else length = [i];
   IMPORTANT: for global data,
     this function allocates a double to store the value in,
     so the caller must free this memory to avoid a leak, e.g.
       double *dptr = (double *) lammps_extract_fix();
       double value = *dptr;
       free(dptr);
   IMPORTANT: LAMMPS cannot easily check here when info extracted from
     the fix is valid, so caller must insure that it is OK
   fix_data=lammps_extract_fix(lmp_pointer,fix_id,style,type,length,i,j)
------------------------------------------------------------------------- */


/*function prototype necessary to avoid truncation to 32 bits*/
void * lammps_extract_fix(void * lmp1,char * command, int style, int type,int i, int j);

void mexFunction( int nlhs, mxArray *plhs[],
int nrhs,  const mxArray *prhs[])
{
    double *lmp;
    char *command;
    long lmp1;
    double *style;
    int style1;
    double *type;
    int type1;
    int len;
    double * ifix, *jfix;
    double *vectorlength;
    int i,j;
    double *outaddr;
    double **data;
    double *data1;
    if (nlhs != 1)
    {
        mxErrMsgTxt("One output argument required.");
    }
    if (nrhs < 4 || nrhs > 7 || !mxIsChar(prhs[1]) || !mxIsDouble(prhs[2]) || !mxIsDouble(prhs[3]))
    {
        mxErrMsgTxt("Invalid synatx: Should be like so:\n fix_data=lammps_extract_fix(lmp_pointer,fix_id,style,type,length,i,j).\n    style = 0 for global data, 1 for per-atom data, 2 for local data\ntype = 0 for scalar, 1 for vector, 2 for array.\n style and type must be input as standard matlab numbers, i.e., doubles. For local and per-atom arrays, length is the number of per-atom elements. For type==1, i is the element of the vector requested. For type==2, element i,j is requested. If type==0, the command fix_data=lammps_extract_fix(lmp_pointer,fix_id,style,type) is valid.");
    }
    command=(char *)mxMalloc(mxGetN(prhs[1]) * sizeof(char));
    mxGetString(prhs[1],command,mxGetN(prhs[1])+1);
    lmp = mxGetPr(prhs[0]);
    lmp1 = (long)*lmp;
    style = mxGetPr(prhs[2]);
    style1 = (int)*style;
    type = mxGetPr(prhs[3]);
    type1 = (int)*type;
    if (type1-style1>0)
    {
        if ((nrhs !=5 && nrhs != 6 && nrhs != 7) || !mxIsDouble(plhs[4]))
        {
            mxErrMsgTxt("Invalid Syntax. Use fix_data=lammps_extract_fix(lmp_pointer,fix_id,style,type,length). Length must be defined.");
        }
        vectorlength = mxGetPr(prhs[4]);
        len = (int)*vectorlength;
    }
    if (type1 > 0 && style1 == 0)
    {
        if (type1 == 1)
        {
            ifix = mxGetPr(prhs[5]);
        }
        else if (type1 == 2)
        {
            jfix = mxGetPr(prhs[6]);
        }
    }
    if (type1==2 && style > 0)
    {
        data = (double **)lammps_extract_fix((void *)lmp1,command,style1,type1,-1,-1);
        if (data==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(lammps_get_natoms((void *)lmp1),len,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            for (j=0;j<len;j++)
            {
                outaddr[i*(len)+j] = data[i][j];
            }
        }
    }
    else if (type1==1 && style1>0)
    {
        data1 = (double *)lammps_extract_fix((void *)lmp1,command,style1,type1,-1,-1);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(lammps_get_natoms((void *)lmp1),1,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            outaddr[i] = data1[i];
        }
    }
    else if (type1==1 && style == 0)
    {
        data1 = (double *)lammps_extract_compute((void *)lmp1,command,style1,type1,(int)*ifix,-1);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<1;i++)
        {
            outaddr[i] = data1[i];
        }
        free(data);
    }
    else if (type1==2 && style == 0)
    {
        data1 = (double *)lammps_extract_fix((void *)lmp1,command,style1,type1,(int)*ifix,(int)*jfix);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<1;i++)
        {
            outaddr[i] = data1[i];
        }
        free(data);
    }
    else if (type1==0 && style == 0)
    {
        data1 = (double *)lammps_extract_fix((void *)lmp1,command,style1,type1,-1,-1);
        if (data1==NULL)
        {
            mxErrMsgTxt("Invalid data type or style/type combination");
        }
        plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<1;i++)
        {
            outaddr[i] = data1[i];
        }
        free(data);
    }
}