#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
/* ----------------------------------------------------------------------
   extract an internal LAMMPS evaluated variable
   name = variable name, must be equal-style or atom-style variable
   group = group ID for evaluating an atom-style variable, else NULL
   for equal-style variable, returns a pointer to a memory location
     which is allocated by this function
     which the caller can cast to a (double *) which points to the value
   for atom-style variable, returns a pointer to the
     vector of per-atom values on each processor,
     which the caller can cast to a (double *) which points to the values
   IMPORTANT: for both equal-style and atom-style variables,
     this function allocates memory to store the variable data in
     so the caller must free this memory to avoid a leak
     e.g. for equal-style variables
       double *dptr = (double *) lammps_extract_variable();
       double value = *dptr;
       free(dptr);
     e.g. for atom-style variables
       double *vector = (double *) lammps_extract_variable();
       use the vector values
       free(vector);
   IMPORTANT: LAMMPS cannot easily check here when it is valid to evaluate
     the variable or any fixes or computes or thermodynamic info it references,
     so caller must insure that it is OK
   variable_data=lammps_extract_variable(lmp_pointer,variable_id,group,type);
------------------------------------------------------------------------- */

/*function prototype necessary to avoid truncation to 32 bits*/
void * lammps_extract_variable(void * lmp1,char * command, char* group);

void mexFunction( int nlhs, mxArray *plhs[],
int nrhs,  const mxArray *prhs[])
{
    double *lmp;
    int i;
    char *command;
    char *group;
    double *type;
    int type1;
    long lmp1;
    double *data;
    double *outaddr;
    if (nlhs != 1)
    {
        mxErrMsgTxt("One output argument required.");
    }
    if (nrhs != 4 || !mxIsChar(prhs[1]) || !mxIsChar(prhs[2]) || !mxIsDouble(prhs[3]))
    {
        mxErrMsgTxt("Invalid synatx: Should be like so:\n variable_data=lammps_extract_variable(lmp_pointer,variable_id,group,type).\n    type==0 for global and type==1 for atom style variables");
    }
    command=(char *)mxMalloc(mxGetN(prhs[1]) * sizeof(char));
    group = (char *)mxMalloc(mxGetN(prhs[2]) * sizeof(char));
    mxGetString(prhs[1],command,mxGetN(prhs[1])+1);
    mxGetString(prhs[2],group,mxGetN(prhs[2])+1);
    lmp = mxGetPr(prhs[0]);
    type = mxGetPr(prhs[3]);
    type1=(int)*type;
    lmp1 = (long)*lmp;
    data = (double *)lammps_extract_variable((void *)lmp1,command,group);
    if (data==NULL)
    {
        mxErrMsgTxt("Invalid variable name");
    }
    if (type1==0)
    {
        plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
        outaddr = mxGetPr(plhs[0]);
        outaddr[0] = data[0];
    }
    else if (type1==1)
    {
        plhs[0] = mxCreateDoubleMatrix(lammps_get_natoms((void *)lmp1),1, mxREAL);
        outaddr = mxGetPr(plhs[0]);
        for (i=0;i<lammps_get_natoms((void *)lmp1);i++)
        {
            outaddr[i] = data[i];
        }
    }
    free(data);
}